@@ -1,6 +1,6 @@
 /*
  * Argus Software.  Argus files - Authentication
- * Copyright (c) 2000-2008 QoSient, LLC
+ * Copyright (c) 2000-2012 QoSient, LLC
  * All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -18,6 +18,12 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+/* 
+ * $Id: //depot/argus/argus/argus/ArgusAuth.c#15 $
+ * $DateTime: 2012/01/03 19:15:19 $
+ * $Change: 2274 $
+ */
+
 /*
  * Copyright (c) 2000 Carnegie Mellon University.  All rights reserved.
  *
@@ -65,18 +71,15 @@
  *
  */
 
-/* 
- * $Id: $
- * $DateTime: $
- * $Change: $
- */
-
+#ifdef HAVE_CONFIG_H
+#include "argus_config.h"
+#endif
 
 #if !defined(ArgusSasl)
 #define ArgusSasl
 #endif
  
-#include <compat.h>
+#include <argus_compat.h>
 #include <ArgusModeler.h>
 #include <ArgusOutput.h>
 #include <ArgusSource.h>
