@@ -1,6 +1,6 @@
 /*
  * Argus Software.  Argus files - Fragment processing
- * Copyright (c) 2000-2008 QoSient, LLC
+ * Copyright (c) 2000-2012 QoSient, LLC
  * All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -20,11 +20,15 @@
  */
 
 /* 
- * $Id: $
- * $DateTime: $
- * $Change: $
+ * $Id: //depot/argus/argus/argus/ArgusFrag.c#26 $
+ * $DateTime: 2012/01/03 19:15:19 $
+ * $Change: 2274 $
  */
 
+#ifdef HAVE_CONFIG_H
+#include "argus_config.h"
+#endif
+
 #if !defined(ArgusFrag)
 #define ArgusFrag
 #endif
@@ -34,7 +38,7 @@
 #include <string.h>
 #include <errno.h>
 
-#include <compat.h>
+#include <argus_compat.h>
 #include <ArgusModeler.h>
 
 
@@ -52,12 +56,24 @@ ArgusCreateFRAGFlow (struct ArgusModelerStruct *model, void *ptr, unsigned short
 
    switch (proto) {
       case ETHERTYPE_IP: {
+         struct ip tipbuf, *tip = &tipbuf; 
          struct ip *ip = (struct ip *) ptr;
 
+#ifdef _LITTLE_ENDIAN
+         bzero(tip, sizeof(*tip));
+         tip->ip_len = ntohs(ip->ip_len);
+         tip->ip_id  = ntohs(ip->ip_id);
+         tip->ip_off = ntohs(ip->ip_off);
+         tip->ip_p   = ip->ip_p;
+         tip->ip_src.s_addr =  ntohl(ip->ip_src.s_addr);
+         tip->ip_dst.s_addr =  ntohl(ip->ip_dst.s_addr);
+#else
+         tip = ip;
+#endif
          model->state &= ~ARGUS_DIRECTION;
          if (STRUCTCAPTURED(model, *ip)) {
             if (model->ArgusFlowType == ARGUS_BIDIRECTIONAL) {
-               if (ip->ip_src.s_addr > ip->ip_dst.s_addr) {
+               if (tip->ip_src.s_addr > tip->ip_dst.s_addr) {
                   model->state |= ARGUS_DIRECTION;
                   model->ArgusThisFlow->hdr.argus_dsrvl8.qual |= ARGUS_DIRECTION;
                   model->ArgusThisFlow->hdr.subtype     |= ARGUS_REVERSE;
@@ -67,14 +83,14 @@ ArgusCreateFRAGFlow (struct ArgusModelerStruct *model, void *ptr, unsigned short
             model->ArgusThisFlow->hdr.argus_dsrvl8.qual |= ARGUS_FRAGMENT;
 
             if (model->state & ARGUS_DIRECTION) {
-               model->ArgusThisFlow->ip_flow.ip_src   = ip->ip_dst.s_addr;
-               model->ArgusThisFlow->ip_flow.ip_dst   = ip->ip_src.s_addr;
+               model->ArgusThisFlow->ip_flow.ip_src   = tip->ip_dst.s_addr;
+               model->ArgusThisFlow->ip_flow.ip_dst   = tip->ip_src.s_addr;
             } else {
-               model->ArgusThisFlow->frag_flow.ip_src = ip->ip_src.s_addr;
-               model->ArgusThisFlow->frag_flow.ip_dst = ip->ip_dst.s_addr;
+               model->ArgusThisFlow->frag_flow.ip_src = tip->ip_src.s_addr;
+               model->ArgusThisFlow->frag_flow.ip_dst = tip->ip_dst.s_addr;
             }
-            model->ArgusThisFlow->frag_flow.ip_p       = ip->ip_p;
-            model->ArgusThisFlow->frag_flow.ip_id      = ip->ip_id;
+            model->ArgusThisFlow->frag_flow.ip_p       = tip->ip_p;
+            model->ArgusThisFlow->frag_flow.ip_id      = tip->ip_id;
             model->ArgusThisFlow->frag_flow.pad[0]     = 0;
             model->ArgusThisFlow->frag_flow.pad[1]     = 0;
             retn = model->ArgusThisFlow;
@@ -131,9 +147,11 @@ ArgusUpdateFRAGState (struct ArgusModelerStruct *model, struct ArgusFlowStruct *
 
    switch (proto) {
       case ETHERTYPE_IP: {
-         up_p = ((struct ip *)model->ArgusThisIpHdr)->ip_p;
-         offset = (((struct ip *) model->ArgusThisIpHdr)->ip_off & 0x1fff) << 3;
-         length = model->ArgusThisLength;
+         struct ip *ip = (struct ip *)model->ArgusThisIpHdr;
+
+         up_p = ip->ip_p;
+         offset = (ntohs(ip->ip_off) & 0x1fff) << 3;
+         length = ntohs(ip->ip_len) - (ip->ip_hl * 4);
          end = offset + length;
          break;
       }
@@ -299,9 +317,10 @@ ArgusUpdateFRAGState (struct ArgusModelerStruct *model, struct ArgusFlowStruct *
       switch (proto) {
          case ETHERTYPE_IP: {
             struct ip *iphdr = (struct ip *)model->ArgusThisIpHdr;
-            if (!(iphdr->ip_off & IP_MF))
-               frag->totlen = ((iphdr->ip_off & 0x1fff) << 3) +
-                               (iphdr->ip_len - (iphdr->ip_hl << 2));
+
+            if (!(ntohs(iphdr->ip_off) & IP_MF))
+               frag->totlen = ((ntohs(iphdr->ip_off) & 0x1fff) << 3) +
+                               (ntohs(iphdr->ip_len) - (iphdr->ip_hl << 2));
             break;
          }
          case ETHERTYPE_IPV6: {
@@ -416,14 +435,14 @@ ArgusUpdateParentFlow (struct ArgusModelerStruct *model, struct ArgusFlowStruct
       }
 
       if (found) {
-         struct ArgusTimeObject *ftime = NULL, *otime = NULL;
+         struct ArgusTimeObject *ftime = NULL, *ptime = NULL;
          struct ArgusSystemFlow *flow = NULL;
          struct ArgusMetricStruct *metrics = NULL;
 
          if (parent->status & ARGUS_RECORD_WRITTEN)
             ArgusZeroRecord(parent);
 
-         flow = (struct ArgusSystemFlow *)flowstr->dsrs[ARGUS_FLOW_INDEX];
+         flow = (struct ArgusSystemFlow *)parent->dsrs[ARGUS_FLOW_INDEX];
          metrics = (struct ArgusMetricStruct *) parent->dsrs[ARGUS_METRIC_INDEX];
 
 #define ARGUS_TYPES  (ARGUS_TYPE_IPV4 | ARGUS_TYPE_IPV6 | ARGUS_TYPE_ARP | ARGUS_TYPE_ETHER)
@@ -434,7 +453,7 @@ ArgusUpdateParentFlow (struct ArgusModelerStruct *model, struct ArgusFlowStruct
                case ARGUS_TYPE_IPV6:  {
                   struct ArgusIPAttrStruct *attr = (struct ArgusIPAttrStruct *) parent->dsrs[ARGUS_IPATTR_INDEX];
 
-                  if ((parent->status & ARGUS_DIRECTION) != (flowstr->status & ARGUS_DIRECTION)) {
+                  if ((parent->state & ARGUS_DIRECTION) != (flowstr->state & ARGUS_DIRECTION)) {
                      metrics->dst.pkts  += flowstr->canon.metric.src.pkts;
                      metrics->dst.bytes += flowstr->canon.metric.src.bytes;
                      
@@ -453,33 +472,39 @@ ArgusUpdateParentFlow (struct ArgusModelerStruct *model, struct ArgusFlowStruct
                      if (attr != NULL)
                         attr->hdr.argus_dsrvl8.qual |= ARGUS_IPATTR_SRC_FRAGMENTS;
                   }
-
-                  flow->hdr.argus_dsrvl8.qual |= ARGUS_FRAGMENT;
                   break;
                }
             }
          }
 
          ftime = (struct ArgusTimeObject *) flowstr->dsrs[ARGUS_TIME_INDEX];
-         otime = (struct ArgusTimeObject *)  parent->dsrs[ARGUS_TIME_INDEX];
+         ptime = (struct ArgusTimeObject *)  parent->dsrs[ARGUS_TIME_INDEX];
 
-         if (ftime && otime) {
-            if (otime->hdr.subtype == ARGUS_TIME_ABSOLUTE_TIMESTAMP) {
-               otime->hdr.subtype           = ARGUS_TIME_ABSOLUTE_RANGE;
-               otime->hdr.argus_dsrvl8.qual = ARGUS_TYPE_UTC_MICROSECONDS;
-               otime->hdr.argus_dsrvl8.len  = 5;
-               if ((otime->src.start.tv_sec  < ftime->src.start.tv_sec) ||
-                  ((otime->src.start.tv_sec == ftime->src.start.tv_sec) &&
-                   (otime->src.start.tv_usec < ftime->src.start.tv_usec))) {
-                  otime->src.start = ftime->src.start;
+         if (ftime && ptime) {
+            struct ArgusTimeStruct *pstime;
+
+            if ((parent->state & ARGUS_DIRECTION) != (flowstr->state & ARGUS_DIRECTION)) {
+               pstime = &ptime->dst;
+            } else {
+               pstime = &ptime->src;
+            }
+
+            if (ptime->hdr.subtype == ARGUS_TIME_ABSOLUTE_TIMESTAMP) {
+               ptime->hdr.subtype           = ARGUS_TIME_ABSOLUTE_RANGE;
+               ptime->hdr.argus_dsrvl8.qual = ARGUS_TYPE_UTC_MICROSECONDS;
+               ptime->hdr.argus_dsrvl8.len  = 5;
+               if ((pstime->start.tv_sec  > ftime->src.start.tv_sec) ||
+                  ((pstime->start.tv_sec == ftime->src.start.tv_sec) &&
+                   (pstime->start.tv_usec > ftime->src.start.tv_usec))) {
+                  pstime->start = ftime->src.start;
                }
-               otime->src.end = ftime->src.end;
+               pstime->end = ftime->src.end;
 
             } else {
-               if ((otime->src.end.tv_sec  < ftime->src.end.tv_sec) ||
-                  ((otime->src.end.tv_sec == ftime->src.end.tv_sec) &&
-                   (otime->src.end.tv_usec < ftime->src.end.tv_usec))) {
-                  otime->src.end = ftime->src.end;
+               if ((pstime->end.tv_sec  < ftime->src.end.tv_sec) ||
+                  ((pstime->end.tv_sec == ftime->src.end.tv_sec) &&
+                   (pstime->end.tv_usec < ftime->src.end.tv_usec))) {
+                  pstime->end = ftime->src.end;
                }
             }
             parent->dsrindex |= 1 << ARGUS_TIME_INDEX;
