@@ -1,6 +1,6 @@
 /*
  * Argus Software.  Argus files - ICMP protocol processing
- * Copyright (c) 2000-2008 QoSient, LLC
+ * Copyright (c) 2000-2012 QoSient, LLC
  * All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -20,16 +20,20 @@
  */
 
 /* 
- * $Id: $
- * $DateTime: $
- * $Change: $
+ * $Id: //depot/argus/argus/argus/ArgusIcmp.c#19 $
+ * $DateTime: 2012/01/03 19:15:19 $
+ * $Change: 2274 $
  */
 
+#ifdef HAVE_CONFIG_H
+#include "argus_config.h"
+#endif
+
 #if !defined(ArgusIcmp)
 #define ArgusIcmp
 #endif
 
-#include <compat.h>
+#include <argus_compat.h>
 #include <ArgusModeler.h>
 #include <ArgusOutput.h>
 #include <ArgusSource.h>
@@ -43,29 +47,54 @@ ArgusCreateICMPv6Flow (struct ArgusModelerStruct *model, struct icmp6_hdr *icmp)
 
    if (STRUCTCAPTURED(model, *icmp)) {
       struct ArgusICMPv6Flow *icmpv6Flow = &model->ArgusThisFlow->icmp6_flow;
-      if (icmp->icmp6_type & ICMP6_INFOMSG_MASK) {
 
-         icmpv6Flow->type = icmp->icmp6_type;
+      icmpv6Flow->type = icmp->icmp6_type;
 
-         switch (icmp->icmp6_type) {
-            case ICMP6_ECHO_REPLY:
-               if (model->ArgusFlowType == ARGUS_BIDIRECTIONAL)
-                  icmpv6Flow->type = ICMP6_ECHO_REQUEST;
-               break;
+      switch (icmp->icmp6_type & ICMP6_INFOMSG_MASK) {
+         case ICMP6_INFOMSG_MASK: 
+            switch (icmp->icmp6_type) {
+               case ICMP6_ECHO_REPLY:
+                  if (model->ArgusFlowType == ARGUS_BIDIRECTIONAL)
+                     icmpv6Flow->type = ICMP6_ECHO_REQUEST;
+                  break;
 
-            case ICMP6_MEMBERSHIP_REPORT:
-            case ICMP6_MEMBERSHIP_REDUCTION:
-               if (model->ArgusFlowType == ARGUS_BIDIRECTIONAL)
-                  icmpv6Flow->type = ICMP6_MEMBERSHIP_QUERY;
-               else
-                  icmpv6Flow->type = (icmp->icmp6_type & ICMP6_INFOMSG_MASK);
-               break;
+               case ICMP6_MEMBERSHIP_REPORT:
+               case ICMP6_MEMBERSHIP_REDUCTION:
+                  if (model->ArgusFlowType == ARGUS_BIDIRECTIONAL)
+                     icmpv6Flow->type = ICMP6_MEMBERSHIP_QUERY;
+                  break;
+
+               case ND_ROUTER_ADVERT:
+                  if (model->ArgusFlowType == ARGUS_BIDIRECTIONAL)
+                     icmpv6Flow->type = ND_ROUTER_SOLICIT;
+                  break;
+
+               case ND_NEIGHBOR_ADVERT:
+                  if (model->ArgusFlowType == ARGUS_BIDIRECTIONAL)
+                     icmpv6Flow->type = ND_NEIGHBOR_SOLICIT;
+                  break;
+
+               case ICMP6_HADISCOV_REPLY:
+                  if (model->ArgusFlowType == ARGUS_BIDIRECTIONAL)
+                     icmpv6Flow->type = ICMP6_HADISCOV_REQUEST;
+                  break;
+            }
+            icmpv6Flow->code = icmp->icmp6_code;
+            break;
+
+         default: {
+            switch (icmp->icmp6_type) {
+               case ICMP6_DST_UNREACH:
+               case ICMP6_PACKET_TOO_BIG:
+               case ICMP6_TIME_EXCEEDED:
+               case ICMP6_PARAM_PROB:
+                  break;
+            }
+
+            icmpv6Flow->type = icmp->icmp6_type;
+            icmpv6Flow->code = icmp->icmp6_code;
+            break;
          }
-         icmpv6Flow->code = icmp->icmp6_code;
-
-      } else {
-         icmpv6Flow->type = icmp->icmp6_type;
-         icmpv6Flow->code = icmp->icmp6_code;
       }
 
       if (model->state & ARGUS_DIRECTION) {
@@ -97,8 +126,9 @@ ArgusCreateICMPFlow (struct ArgusModelerStruct *model, struct ip *ip)
       retn->hdr.argus_dsrvl8.qual = ARGUS_TYPE_IPV4;
       retn->hdr.argus_dsrvl8.len  = 5;
 
-      icmpFlow->ip_src = ip->ip_src.s_addr;
-      icmpFlow->ip_dst = ip->ip_dst.s_addr;
+      icmpFlow->ip_src = ntohl(ip->ip_src.s_addr);
+      icmpFlow->ip_dst = ntohl(ip->ip_dst.s_addr);
+
       icmpFlow->type   = icmp->icmp_type;
       icmpFlow->ip_p   = ip->ip_p;
       icmpFlow->tp_p   = 0;
@@ -225,8 +255,9 @@ ArgusUpdateICMPState (struct ArgusModelerStruct *model, struct ArgusFlowStruct *
          icmpObj->icmp_type = icmp->icmp_type;
          icmpObj->icmp_code = icmp->icmp_code;
          icmpObj->iseq      = ntohs(icmp->icmp_seq);
-         icmpObj->osrcaddr  = ((struct ip *)model->ArgusThisIpHdr)->ip_src.s_addr;
-         icmpObj->odstaddr  = ((struct ip *)model->ArgusThisIpHdr)->ip_dst.s_addr;
+
+         icmpObj->osrcaddr  = ntohl(((struct ip *)model->ArgusThisIpHdr)->ip_src.s_addr);
+         icmpObj->odstaddr  = ntohl(((struct ip *)model->ArgusThisIpHdr)->ip_dst.s_addr);
 
          if (ICMP_INFOTYPE(icmp->icmp_type)) {
             switch (icmp->icmp_type) {
@@ -290,16 +321,13 @@ ArgusUpdateICMPState (struct ArgusModelerStruct *model, struct ArgusFlowStruct *
 
          struct ip *oip = &icmp->icmp_ip;
 
-#define ARGUS_MODELER_BUSY               0x0100000
-#define ARGUS_QMANAGER_BUSY              0x0200000
-
          if (STRUCTCAPTURED(model, *oip)) {
             model->ArgusThisIpHdr = oip;
             model->ArgusThisUpHdr = (unsigned char *) oip;
 
             if ((tflow = ArgusCreateFlow(model, oip, model->ArgusThisLength)) != NULL) {
                ArgusCreateFlowKey (model, tflow, &hstruct);
-               if ((flow = ArgusFindFlow (model, &hstruct, ARGUS_MODELER_BUSY)) != NULL) {
+               if ((flow = ArgusFindFlow (model, &hstruct)) != NULL) {
                   struct ArgusDSRHeader *dsr = &flow->canon.icmp.hdr;
                   struct ArgusTimeObject *time = &flow->canon.time;
 
