@@ -1,6 +1,6 @@
 /*
  * Argus Software.  Argus files - IGMP protocol processing
- * Copyright (c) 2000-2008 QoSient, LLC
+ * Copyright (c) 2000-2012 QoSient, LLC
  * All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -20,16 +20,20 @@
  */
 
 /* 
- * $Id: $
- * $DateTime: $
- * $Change: $
+ * $Id: //depot/argus/argus/argus/ArgusIgmp.c#14 $
+ * $DateTime: 2012/01/03 19:15:19 $
+ * $Change: 2274 $
  */
 
+#ifdef HAVE_CONFIG_H
+#include "argus_config.h"
+#endif
+
 #if !defined(ArgusIgmp)
 #define ArgusIgmp
 #endif
 
-#include <compat.h>
+#include <argus_compat.h>
 #include <ArgusModeler.h>
 #include <ArgusOutput.h>
 #include <ArgusSource.h>
@@ -104,8 +108,8 @@ ArgusCreateIGMPFlow (struct ArgusModelerStruct *model, struct ip *ip)
       model->ArgusThisFlow->hdr.argus_dsrvl8.qual = ARGUS_TYPE_IPV4;
       model->ArgusThisFlow->hdr.argus_dsrvl8.len  = 5;
 
-      igmpFlow->ip_src = ip->ip_src.s_addr;
-      igmpFlow->ip_dst = ip->ip_dst.s_addr;
+      igmpFlow->ip_src = ntohl(ip->ip_src.s_addr);
+      igmpFlow->ip_dst = ntohl(ip->ip_dst.s_addr);
       igmpFlow->ip_p   = ip->ip_p;
       igmpFlow->type   = igmp->igmp_type;
       igmpFlow->code   = igmp->igmp_code;
