@@ -1,6 +1,6 @@
 /*
  * Argus Software.  Argus files - Layer 2 processing
- * Copyright (c) 2000-2008 QoSient, LLC
+ * Copyright (c) 2000-2012 QoSient, LLC
  * All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -20,16 +20,20 @@
  */
 
 /* 
- * $Id: $
- * $DateTime: $
- * $Change: $
+ * $Id: //depot/argus/argus/argus/ArgusMac.c#12 $
+ * $DateTime: 2012/01/03 19:15:19 $
+ * $Change: 2274 $
  */
 
+#ifdef HAVE_CONFIG_H
+#include "argus_config.h"
+#endif
+
 #if !defined(ArgusMac)
 #define ArgusMac
 #endif
 
-#include <compat.h>
+#include <argus_compat.h>
 #include <ArgusModeler.h>
 #include <ArgusOutput.h>
 #include <ArgusSource.h>
