@@ -1,6 +1,6 @@
 /*
  * Argus Software.  Argus files - Modeler includes
- * Copyright (c) 2000-2008 QoSient, LLC
+ * Copyright (c) 2000-2012 QoSient, LLC
  * All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -20,9 +20,9 @@
  */
 
 /* 
- * $Id: $
- * $DateTime: $
- * $Change: $
+ * $Id: //depot/argus/argus/argus/ArgusModeler.h#53 $
+ * $DateTime: 2012/01/03 19:15:19 $
+ * $Change: 2274 $
  */
 
 
@@ -32,10 +32,10 @@
 #define ARGUS_MARSTATUSTIMER	"60"
 #define ARGUS_FARSTATUSTIMER	"5"
 
-#define ARGUS_INITIMEOUT	0
+#define ARGUS_INITIMEOUT	5
 #define ARGUS_IPTIMEOUT		30 
 #define ARGUS_ARPTIMEOUT	5 
-#define ARGUS_TCPTIMEOUT	30
+#define ARGUS_TCPTIMEOUT	60
 #define ARGUS_ICMPTIMEOUT	5
 #define ARGUS_IGMPTIMEOUT	30
 #define ARGUS_OTHERTIMEOUT	30
@@ -73,27 +73,7 @@
 #define ARGUS_RTP_MP2T		33
 #define ARGUS_RTP_H263		34
 
-#define ARGUS_ENCAPS_TYPE	20
-#define ARGUS_ENCAPS_MPLS	0x01
-#define ARGUS_ENCAPS_ETHER	0x02
-#define ARGUS_ENCAPS_8021Q	0x04
-#define ARGUS_ENCAPS_LLC	0x08
-#define ARGUS_ENCAPS_PPP  	0x10
-#define ARGUS_ENCAPS_ISL	0x20
-#define ARGUS_ENCAPS_GRE	0x40
-#define ARGUS_ENCAPS_AH 	0x80
-#define ARGUS_ENCAPS_IP 	0x100
-#define ARGUS_ENCAPS_IPV6 	0x200
-#define ARGUS_ENCAPS_HDLC 	0x400
-#define ARGUS_ENCAPS_CHDLC 	0x800
-#define ARGUS_ENCAPS_ATM 	0x1000
-#define ARGUS_ENCAPS_SLL 	0x2000
-#define ARGUS_ENCAPS_FDDI	0x4000
-#define ARGUS_ENCAPS_SLIP	0x8000
-#define ARGUS_ENCAPS_ARCNET	0x10000
-#define ARGUS_ENCAPS_802_11	0x20000
-#define ARGUS_ENCAPS_PRISM	0x40000
-#define ARGUS_ENCAPS_AVS	0x80000
+#define ARGUS_SSH_MONITOR       0x20000
 
 #define ARGUS_ETHER_HDR		1
 #define ARGUS_802_11_HDR	2
@@ -101,7 +81,7 @@
 #define ARGUS_DEBUG		0xFF
 
 #define ARGUS_RECORD_WRITTEN	0x00000001
-#define ARGUSTIMEOUTQS		60
+#define ARGUSTIMEOUTQS		301
 
 #define ARGUS_CLNS    129
 #define ARGUS_ESIS    130
@@ -128,7 +108,7 @@
 #include <pthread.h>
 #endif
 
-#include <argus.h>
+#include <argus_encapsulations.h>
 
 #include <syslog.h>
 #include <sys/types.h>
@@ -218,8 +198,16 @@ struct ArgusHashTable {
 
 #define ARGUS_MAX_MPLS_LABELS	4
 
+struct ArgusKeyStrokeConf {
+   int status, state, n_min;
+   int dc_min, dc_max, gs_max;
+   int ds_min, ds_max, gpc_max;
+   int ic_min, lcs_max;
+   float icr_min, icr_max;
+};
+
 struct ArgusModelerStruct {
-   int state;
+   int state, status;
 #if defined(ARGUS_THREADS)
    pthread_t thread;
    pthread_mutex_t lock;
@@ -248,13 +236,17 @@ struct ArgusModelerStruct {
    unsigned short ArgusOptionIndicator;
 
    int ArgusInProtocol, ArgusThisDir;
+
+   struct ArgusKeyStrokeConf ArgusKeyStroke;
    struct ArgusUniStats *ArgusThisStats;
  
    struct ether_header *ArgusThisEpHdr;
+
+   void *ArgusThisMacHdr;
    void *ArgusThisIpHdr; 
    struct ip6_frag *ArgusThisIpv6Frag;
    void *ArgusThisNetworkHdr;
- 
+
    unsigned char *ArgusThisUpHdr;
    unsigned char *ArgusThisSnapEnd;
  
@@ -262,6 +254,7 @@ struct ArgusModelerStruct {
    int ArgusSnapLength;
    int ArgusGenerateTime;
    int ArgusGeneratePacketSize;
+
    int ArgusThisLength;
    int ArgusThisBytes;
  
@@ -287,6 +280,7 @@ struct ArgusModelerStruct {
    long long ArgusTotalCacheHits;
    long long ArgusTotalRecords;
    long long ArgusTotalSends;
+   long long ArgusTotalQueued;
    long long ArgusTotalBadSends;
    long long ArgusLastRecords;
  
@@ -306,15 +300,18 @@ struct ArgusModelerStruct {
    int ArgusMinorVersion;
    int ArgusSnapLen;
  
+   int ArgusTunnelDiscovery;
    int ArgusUserDataLen;
    int ArgusAflag, ArgusTCPflag, Argusmflag;
+   int ArgusSelfSynchronize, vflag;
 
    int ArgusIPTimeout;
    int ArgusTCPTimeout;
    int ArgusICMPTimeout;
    int ArgusIGMPTimeout;
    int ArgusFRAGTimeout;
-   int ArgusIBTimeout;
+   int ArgusARPTimeout;
+   int ArgusOtherTimeout;
 
    int ArgusReportAllTime;
    int ArgusResponseStatus;
@@ -323,9 +320,6 @@ struct ArgusModelerStruct {
    struct timeval ArgusQueueInterval;
    struct timeval ArgusListenInterval;
 
-   unsigned int ArgusID;
-   unsigned int ArgusIDType;
- 
    unsigned int ArgusSeqNum;
    unsigned int ArgusLocalNet;
    unsigned int ArgusNetMask;
@@ -335,6 +329,7 @@ struct ArgusModelerStruct {
 #include <ArgusUtil.h>
 #include <ArgusSource.h>
 #include <ArgusOutput.h>
+#include <argus_isis.h>
  
 struct ArgusTimeStats {
    unsigned int n;
@@ -347,20 +342,46 @@ struct ArgusTimeStat {
    struct ArgusTimeStats act, idle;
 };
 
+
+#define ARGUS_NUM_KEYSTROKE_PKTS	8
+#define ARGUS_KEYSTROKE_NONE		0
+#define ARGUS_KEYSTROKE_TENTATIVE	1
+#define ARGUS_KEYSTROKE_KNOWN		2
+
+struct ArgusKeyStrokePacket {
+   int status, n_pno;
+   struct ArgusTime ts;
+   unsigned int seq;
+   long long intpkt;
+};
+
+struct ArgusKeyStrokeData {
+   struct ArgusKeyStrokePacket pkts[ARGUS_NUM_KEYSTROKE_PKTS];
+};
+
+struct ArgusKeyStrokeState {
+   int status, n_pkts, n_strokes, prev_pno;
+   struct ArgusKeyStrokeData data;
+   struct ArgusTime prev_c_ts, prev_s_ts;
+};
+
 struct ArgusFlowStruct {
    struct ArgusQueueHeader qhdr;
    struct ArgusHashTableHeader htblbuf, *htblhdr;
-   struct ArgusDSRHeader *dsrs[ARGUSMAXFLOWTYPE];
+   struct ArgusDSRHeader *dsrs[ARGUSMAXDSRTYPE];
    struct ArgusQueueStruct frag;
 
-   unsigned int status, dsrindex;
+   unsigned int state, status, dsrindex;
    unsigned int ArgusEncaps;
 
    unsigned short trans, timeout;
    unsigned short userlen;
-   char srcint, dstint;
+   signed char srcint, dstint;
+   unsigned short sipid, dipid;
+
 
    struct ArgusTimeStat stime, dtime;
+   struct ArgusKeyStrokeState skey;
    struct ArgusCanonRecord canon;
 };
 
@@ -373,9 +394,10 @@ unsigned char ArgusAlignBuffer[ARGUS_MAXALIGNBUF], *ArgusAlignBuf = ArgusAlignBu
 #endif
 
 struct ArgusModelerStruct *ArgusModel = NULL;
+
 struct llc ArgusThisLLCBuffer;
 
-unsigned char argusDSRTypes [ARGUSMAXFLOWTYPE] = {
+unsigned char argusDSRTypes [ARGUSMAXDSRTYPE] = {
    ARGUS_TRANSPORT_DSR, ARGUS_FLOW_DSR, ARGUS_TIME_DSR,
    ARGUS_METER_DSR, ARGUS_AGR_DSR,
 };
@@ -387,6 +409,8 @@ struct timeval ArgusListenTime = {0, 0};
 struct timeval ArgusListenInterval = {0, 250000};
 
 struct ArgusModelerStruct *ArgusNewModeler(void);
+struct ArgusModelerStruct *ArgusCloneModeler(struct ArgusModelerStruct *);
+
 void ArgusInitModeler(struct ArgusModelerStruct *);
 void ArgusCloseModeler(struct ArgusModelerStruct *);
 
@@ -407,6 +431,23 @@ void ArgusTallyTime (struct ArgusFlowStruct *, unsigned char);
 
 unsigned short ArgusParseIPOptions (unsigned char *, int);
 
+void setArgusIpTimeout (struct ArgusModelerStruct *model, int value);
+void setArgusTcpTimeout (struct ArgusModelerStruct *model, int value);
+void setArgusIcmpTimeout (struct ArgusModelerStruct *model, int value);
+void setArgusIgmpTimeout (struct ArgusModelerStruct *model, int value);
+void setArgusFragTimeout (struct ArgusModelerStruct *model, int value);
+void setArgusArpTimeout (struct ArgusModelerStruct *model, int value);
+void setArgusOtherTimeout (struct ArgusModelerStruct *model, int value);
+
+void setArgusSynchronize (struct ArgusModelerStruct *, int);
+
+int getArgusKeystroke(struct ArgusModelerStruct *);
+void setArgusKeystroke(struct ArgusModelerStruct *, int);
+void setArgusKeystrokeVariable(struct ArgusModelerStruct *, char *);
+
+int getArgusTunnelDiscovery (struct ArgusModelerStruct *);
+void setArgusTunnelDiscovery (struct ArgusModelerStruct *, int);
+
 void setArgusFlowKey(struct ArgusModelerStruct *, int);
 void setArgusFlowType(struct ArgusModelerStruct *, int);
 
@@ -430,6 +471,10 @@ void setArgusTimeReport(struct ArgusModelerStruct *, int);
 struct timeval *getArgusQueueInterval(struct ArgusModelerStruct *);
 struct timeval *getArgusListenInterval(struct ArgusModelerStruct *);
 
+
+extern struct udt_control_handshake *ArgusThisUdtHshake;
+extern int ArgusParseUDTHeader (struct ArgusModelerStruct *, struct udt_header *);
+
 int getArgusdflag(struct ArgusModelerStruct *);
 void setArgusdflag(struct ArgusModelerStruct *, int);
 
@@ -442,23 +487,27 @@ void *ArgusCreateIPv4Flow (struct ArgusModelerStruct *, struct ip *);
 void *ArgusCreateIPv6Flow (struct ArgusModelerStruct *, struct ip6_hdr *);
 struct ArgusSystemFlow *ArgusCreateESPv6Flow (struct ArgusModelerStruct *, struct ip6_hdr *);
 struct ArgusSystemFlow *ArgusCreateESPFlow (struct ArgusModelerStruct *, struct ip *);
+struct ArgusSystemFlow *ArgusCreateLcpFlow (struct ArgusModelerStruct *, struct lcp_hdr *);
 struct ArgusSystemFlow *ArgusCreateICMPv6Flow (struct ArgusModelerStruct *, struct icmp6_hdr *);
 struct ArgusSystemFlow *ArgusCreateICMPFlow (struct ArgusModelerStruct *, struct ip *);
 struct ArgusSystemFlow *ArgusCreateIGMPv6Flow (struct ArgusModelerStruct *, struct igmp *);
 struct ArgusSystemFlow *ArgusCreateIGMPFlow (struct ArgusModelerStruct *, struct ip *);
 struct ArgusSystemFlow *ArgusCreateFRAGFlow (struct ArgusModelerStruct *, void *, unsigned short);
+struct ArgusSystemFlow *ArgusCreateIsisFlow (struct ArgusModelerStruct *model, struct isis_common_header *header);
+struct ArgusSystemFlow *ArgusCreateUDTFlow (struct ArgusModelerStruct *, struct udt_header *);
+struct ArgusSystemFlow *ArgusCreate80211Flow (struct ArgusModelerStruct *model, void *ptr);
 
 void ArgusUpdateBasicFlow (struct ArgusModelerStruct *, struct ArgusFlowStruct *, unsigned char);
 
 void *ArgusQueueManager(void *); 
 
 int ArgusCreateFlowKey (struct ArgusModelerStruct *, struct ArgusSystemFlow *, struct ArgusHashStruct *);
-struct ArgusFlowStruct *ArgusFindFlow (struct ArgusModelerStruct *, struct ArgusHashStruct *, unsigned int);
+struct ArgusFlowStruct *ArgusFindFlow (struct ArgusModelerStruct *, struct ArgusHashStruct *);
 
 void ArgusICMPMappedFlowRecord (struct ArgusFlowStruct *, struct ArgusRecord *, unsigned char);
 
-struct ArgusFlowStruct *ArgusUpdateState (struct ArgusModelerStruct *, struct ArgusFlowStruct *, unsigned char);
-struct ArgusFlowStruct *ArgusUpdateFlow (struct ArgusModelerStruct *, struct ArgusFlowStruct *, unsigned char);
+struct ArgusFlowStruct *ArgusUpdateState (struct ArgusModelerStruct *, struct ArgusFlowStruct *, unsigned char, unsigned char);
+struct ArgusFlowStruct *ArgusUpdateFlow (struct ArgusModelerStruct *, struct ArgusFlowStruct *, unsigned char, unsigned char);
 void ArgusUpdateAppState (struct ArgusModelerStruct *, struct ArgusFlowStruct *, unsigned char);
 
 void ArgusModelTransmit (void);
@@ -481,27 +530,27 @@ void setArgusFarReportInterval(struct ArgusModelerStruct *, char *);
 int getArgusResponseStatus(struct ArgusModelerStruct *);
 void setArgusResponseStatus(struct ArgusModelerStruct *, int value);
 
-void ArgusParseSourceID (struct ArgusModelerStruct *, char *);
-unsigned int getArgusID(struct ArgusModelerStruct *);
-void setArgusID(struct ArgusModelerStruct *, unsigned int);
+int getArgusIpTimeout(struct ArgusModelerStruct *);
+void setArgusIpTimeout(struct ArgusModelerStruct *, int);
 
-unsigned int getArgusIDType(struct ArgusModelerStruct *);
-void setArgusIDType(struct ArgusModelerStruct *, unsigned int);
+int getArgusTcpTimeout(struct ArgusModelerStruct *);
+void setArgusTcpTimeout(struct ArgusModelerStruct *, int);
 
-int getArgusIPTimeout(struct ArgusModelerStruct *);
-void setArgusIPTimeout(struct ArgusModelerStruct *, int);
+int getArgusIcmpTimeout(struct ArgusModelerStruct *);
+void setArgusIcmpTimeout(struct ArgusModelerStruct *, int);
 
-int getArgusTCPTimeout(struct ArgusModelerStruct *);
-void setArgusTCPTimeout(struct ArgusModelerStruct *, int);
+int getArgusIgmpTimeout(struct ArgusModelerStruct *);
+void setArgusIgmpTimeout(struct ArgusModelerStruct *, int);
 
-int getArgusICMPTimeout(struct ArgusModelerStruct *);
-void setArgusICMPTimeout(struct ArgusModelerStruct *, int);
+int getArgusFragTimeout(struct ArgusModelerStruct *);
+void setArgusFragTimeout(struct ArgusModelerStruct *, int);
 
-int getArgusIGMPTimeout(struct ArgusModelerStruct *);
-void setArgusIGMPTimeout(struct ArgusModelerStruct *, int);
+int getArgusArpTimeout(struct ArgusModelerStruct *);
+void setArgusArpTimeout(struct ArgusModelerStruct *, int);
+
+int getArgusOtherTimeout(struct ArgusModelerStruct *);
+void setArgusOtherTimeout(struct ArgusModelerStruct *, int);
 
-int getArgusFRAGTimeout(struct ArgusModelerStruct *);
-void setArgusFRAGTimeout(struct ArgusModelerStruct *, int);
 
 unsigned int getArgusLocalNet(struct ArgusModelerStruct *);
 void setArgusLocalNet(struct ArgusModelerStruct *, unsigned int);
@@ -541,6 +590,8 @@ void clearArgusConfiguration (struct ArgusModelerStruct *);
 #endif
 
 extern struct ArgusModelerStruct *ArgusNewModeler(void);
+extern struct ArgusModelerStruct *ArgusCloneModeler(struct ArgusModelerStruct *);
+
 extern void ArgusInitModeler(struct ArgusModelerStruct *);
 extern void ArgusCloseModeler(struct ArgusModelerStruct *);
 
@@ -561,6 +612,23 @@ extern void ArgusTallyTime (struct ArgusFlowStruct *, unsigned char);
 
 extern unsigned short ArgusParseIPOptions (unsigned char *, int);
 
+extern void setArgusIpTimeout (struct ArgusModelerStruct *model, int value);
+extern void setArgusTcpTimeout (struct ArgusModelerStruct *model, int value);
+extern void setArgusIcmpTimeout (struct ArgusModelerStruct *model, int value);
+extern void setArgusIgmpTimeout (struct ArgusModelerStruct *model, int value);
+extern void setArgusFragTimeout (struct ArgusModelerStruct *model, int value);
+extern void setArgusArpTimeout (struct ArgusModelerStruct *model, int value);
+extern void setArgusOtherTimeout (struct ArgusModelerStruct *model, int value);
+
+extern void setArgusSynchronize (struct ArgusModelerStruct *, int);
+
+extern int getArgusKeystroke(struct ArgusModelerStruct *);
+extern void setArgusKeystroke(struct ArgusModelerStruct *, int);
+extern void setArgusKeystrokeVariable(struct ArgusModelerStruct *, char *);
+
+extern int getArgusTunnelDiscovery(struct ArgusModelerStruct *);
+extern void setArgusTunnelDiscovery(struct ArgusModelerStruct *, int);
+
 extern void setArgusFlowKey(struct ArgusModelerStruct *, int);
 extern void setArgusFlowType(struct ArgusModelerStruct *, int);
 
@@ -583,12 +651,20 @@ extern struct timeval *getArgusQueueInterval(void);
 extern struct timeval ArgusListenInterval;
 extern struct timeval *getArgusListenInterval(void);
 
+extern struct udt_control_handshake *ArgusThisUdtHshake;
+extern int ArgusParseUDTHeader (struct ArgusModelerStruct *, struct udt_header *);
+
 extern int getArgusGenerateTime(struct ArgusModelerStruct *);
 extern void setArgusGenerateTime(struct ArgusModelerStruct *, int);
+
 extern int getArgusGeneratePacketSize(struct ArgusModelerStruct *);
 extern void setArgusGeneratePacketSize(struct ArgusModelerStruct *, int);
+
 extern void setArgusTimeReport(struct ArgusModelerStruct *, int);
 
+extern int getArgusKeystroke(struct ArgusModelerStruct *);
+extern void setArgusKeystroke(struct ArgusModelerStruct *, int);
+
 extern int getArgusdflag(struct ArgusModelerStruct *);
 extern struct timeval *getArgusFarReportInterval(struct ArgusModelerStruct *);
 
@@ -608,18 +684,22 @@ extern struct ArgusSystemFlow *ArgusCreateLcpFlow (struct ArgusModelerStruct *,
 extern struct ArgusSystemFlow *ArgusCreateICMPv6Flow (struct ArgusModelerStruct *, struct icmp6_hdr *);
 extern struct ArgusSystemFlow *ArgusCreateICMPFlow (struct ArgusModelerStruct *, struct ip *);
 extern struct ArgusSystemFlow *ArgusCreateFRAGFlow (struct ArgusModelerStruct *, void *, unsigned short);
+extern struct ArgusSystemFlow *ArgusCreateIsisFlow (struct ArgusModelerStruct *model, struct isis_common_header *header);
+extern struct ArgusSystemFlow *ArgusCreateUDTFlow (struct ArgusModelerStruct *, struct udt_header *);
+extern struct ArgusSystemFlow *ArgusCreate80211Flow (struct ArgusModelerStruct *model, void *ptr);
+
 
 extern void ArgusUpdateBasicFlow (struct ArgusModelerStruct *, struct ArgusFlowStruct *, unsigned char);
 
 extern void *ArgusQueueManager(void *); 
 
-extern struct ArgusFlowStruct *ArgusFindFlow (struct ArgusModelerStruct *, struct ArgusHashStruct *, unsigned int);
+extern struct ArgusFlowStruct *ArgusFindFlow (struct ArgusModelerStruct *, struct ArgusHashStruct *);
 extern int ArgusCreateFlowKey (struct ArgusModelerStruct *, struct ArgusSystemFlow *, struct ArgusHashStruct *);
 
 extern void ArgusICMPMappedFlowRecord (struct ArgusFlowStruct *, struct ArgusRecord *, unsigned char);
 
-extern struct ArgusFlowStruct *ArgusUpdateState (struct ArgusModelerStruct *, struct ArgusFlowStruct *, unsigned char);
-extern struct ArgusFlowStruct *ArgusUpdateFlow (struct ArgusModelerStruct *, struct ArgusFlowStruct *, unsigned char);
+extern struct ArgusFlowStruct *ArgusUpdateState (struct ArgusModelerStruct *, struct ArgusFlowStruct *, unsigned char, unsigned char);
+extern struct ArgusFlowStruct *ArgusUpdateFlow (struct ArgusModelerStruct *, struct ArgusFlowStruct *, unsigned char, unsigned char);
 extern void ArgusUpdateAppState (struct ArgusModelerStruct *, struct ArgusFlowStruct *, unsigned char);
 
 extern void ArgusModelTransmit (void);
@@ -642,13 +722,6 @@ extern void setArgusStatusReportInterval(struct ArgusModelerStruct *, int);
 extern int getArgusResponseStatus(struct ArgusModelerStruct *);
 extern void setArgusResponseStatus(struct ArgusModelerStruct *, int value);
 
-extern void ArgusParseSourceID (struct ArgusModelerStruct *, char *);
-extern unsigned int getArgusID(struct ArgusModelerStruct *);
-extern void setArgusID(struct ArgusModelerStruct *, unsigned int);
-
-extern unsigned int getArgusIDType(struct ArgusModelerStruct *);
-extern void setArgusIDType(struct ArgusModelerStruct *, unsigned int);
-
 extern int getArgusIPTimeout(struct ArgusModelerStruct *);
 extern void setArgusIPTimeout(struct ArgusModelerStruct *, int);
 
