@@ -1,13 +1,13 @@
 /*
  * Argus Software.  Argus files - Output include file
- * Copyright (c) 2000-2008 QoSient, LLC
+ * Copyright (c) 2000-2012 QoSient, LLC
  * All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2, or (at your option)
  * any later version.
- 
+
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -19,13 +19,14 @@
  *
  */
 
-/* 
- * $Id: $
- * $DateTime: $
- * $Change: $
+/*
+ * $Id: //depot/argus/argus/argus/ArgusOutput.h#16 $
+ * $DateTime: 2012/01/03 19:15:19 $
+ * $Change: 2274 $
  */
 
 
+
 #ifndef ArgusOutput_h
 #define ArgusOutput_h
 
@@ -33,9 +34,14 @@
 #define ARGUS_MAXLISTEN		16
 
 #include <unistd.h>
+#if defined(HAVE_STDLIB_H)
 #include <stdlib.h>
+#endif
 #include <limits.h>
 
+#include <sys/socket.h>
+#include <netdb.h>
+
 #include <stdio.h>
 #include <errno.h>
 #include <string.h>
@@ -54,21 +60,28 @@
 #include <fcntl.h>
 #include <signal.h>
 
+#if !defined(ARGUS_TILERA)
 #include <pcap.h>
+#endif
+
 #include <argus_filter.h>
 
 #ifdef ARGUS_SASL
 #include <sasl/sasl.h>
 #endif
 
-
 struct ArgusClientData {
    struct ArgusQueueHeader qhdr;
-   int fd, ArgusClientStart, ArgusFilterInitialized;
+   int fd, type, ArgusClientStart;
+   int ArgusFilterInitialized;
    struct ArgusSocketStruct *sock;
    struct nff_program ArgusNFFcode;
    char *filename, *filter;
 
+#if defined(HAVE_GETADDRINFO)
+   struct addrinfo *host;
+#endif
+
 #ifdef ARGUS_SASL
    sasl_conn_t *sasl_conn;
    struct {
@@ -80,9 +93,29 @@ struct ArgusClientData {
 #endif
 };
 
+struct ArgusBindAddrStruct {
+   struct ArgusListObjectStruct *nxt;
+   char *addr;
+};
+
+
+
 #define ARGUS_MAX_LISTEN	5
 
 struct ArgusOutputStruct {
+   struct ArgusSourceStruct *ArgusSrc;
+   struct ArgusModelerStruct *ArgusModel;
+   struct ArgusListStruct *ArgusWfileList;
+
+   struct ArgusListStruct *ArgusOutputList;
+   struct ArgusListStruct *ArgusInputList;
+
+   struct ArgusQueueStruct *ArgusClients;
+   struct ArgusListStruct *ArgusBindAddrs;
+   struct ArgusRecord *ArgusInitMar;
+
+   long long ArgusTotalRecords, ArgusLastRecords;
+
    int status;
 
 #if defined(ARGUS_THREADS)
@@ -90,21 +123,18 @@ struct ArgusOutputStruct {
    pthread_mutex_t lock;
 #endif
 
-   struct ArgusSourceStruct *ArgusSrc;
-   struct ArgusModelerStruct *ArgusModel;
-   struct ArgusListStruct *ArgusWfileList;
-   struct ArgusListStruct *ArgusOutputList;
-   struct ArgusQueueStruct *ArgusClients;
-   struct ArgusRecord *ArgusInitMar;
-
-   long long ArgusTotalRecords, ArgusLastRecords;
    int ArgusWriteStdOut;
    int ArgusOutputSequence;
+
+#if defined(HAVE_GETADDRINFO)
+   struct addrinfo ArgusAddrInfo;
+#endif
+
    int ArgusPortNum;
    int ArgusLfd[ARGUS_MAX_LISTEN];
    int ArgusListens, nflag;
  
-   char *ArgusBindAddr;
+   char *ArgusBindPort;
 
    struct timeval ArgusGlobalTime;
    struct timeval ArgusStartTime;
@@ -124,7 +154,8 @@ struct ArgusWfileStruct {
 
 struct ArgusOutputStruct *ArgusOutputTask = NULL;
 
-struct ArgusOutputStruct * ArgusNewOutput (struct ArgusSourceStruct *, struct ArgusModelerStruct *);
+struct ArgusOutputStruct *ArgusNewOutput (struct ArgusSourceStruct *, struct ArgusModelerStruct *);
+
 void ArgusCloseOutput (struct ArgusOutputStruct *);
  
 void ArgusInitOutput (struct ArgusOutputStruct *);
@@ -142,18 +173,24 @@ void ArgusChildExit (int);
 void ArgusClientError(void);
 void ArgusInitClientProcess(struct ArgusClientData *, struct ArgusWfileStruct *);
 
+#if defined(ARGUS_TILERA)
+#else
 extern int ArgusFilterCompile(struct nff_program *, char *, int);
 extern unsigned int argus_filter (struct bpf_insn *, unsigned char *);
+#endif
 extern int getArgusPortNum(void);
 
 #else
 
 #if defined(Argus)
 int getArgusPortNum(struct ArgusOutputStruct *);
+int getArgusPortType(struct ArgusOutputStruct *);
 void setArgusPortNum(struct ArgusOutputStruct *, int);
  
 void setArgusBindAddr (struct ArgusOutputStruct *, char *);
+void setArgusBindPort (struct ArgusOutputStruct *, char *);
 char *getArgusBindAddr (struct ArgusOutputStruct *);
+char *getArgusBindPort (struct ArgusOutputStruct *);
 #endif
 
 extern struct timeval ArgusReportTime;
@@ -162,7 +199,7 @@ extern char *ArgusBindAddr;
 extern int ArgusOutfd;
 
 extern struct ArgusOutputStruct *ArgusOutputTask;
-extern struct ArgusOutputStruct * ArgusNewOutput (struct ArgusSourceStruct *, struct ArgusModelerStruct *);
+extern struct ArgusOutputStruct *ArgusNewOutput (struct ArgusSourceStruct *, struct ArgusModelerStruct *);
 extern void ArgusCloseOutput (struct ArgusOutputStruct *);
 
 extern void ArgusInitOutput (struct ArgusOutputStruct *);
@@ -181,8 +218,7 @@ extern void setArgusWfile(char *, char *);
 extern struct timeval *getArgusMarReportInterval(struct ArgusOutputStruct *);
 extern void setArgusMarReportInterval(struct ArgusOutputStruct *, char *);
 
-extern void ArgusCheckClientStatus (void);
-
+extern void ArgusCheckClientStatus (struct ArgusOutputStruct *, int);
 extern int ArgusEstablishListen (struct ArgusOutputStruct *, char *);
 extern int  ArgusTcpWrapper (int, struct sockaddr *);
 
