@@ -1,6 +1,6 @@
 /*
  * Argus Software.  Argus files - TCP protocol
- * Copyright (c) 2000-2008 QoSient, LLC
+ * Copyright (c) 2000-2012 QoSient, LLC
  * All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -20,17 +20,21 @@
  */
 
 /* 
- * $Id$
- * $DateTime$
- * $Change$
+ * $Id: //depot/argus/argus/argus/ArgusTcp.c#41 $
+ * $DateTime: 2012/04/04 01:48:10 $
+ * $Change: 2351 $
  */
 
+#ifdef HAVE_CONFIG_H
+#include "argus_config.h"
+#endif
+
 #if !defined(ArgusTcp)
 #define ArgusTcp
 #endif
 
 
-#include <compat.h>
+#include <argus_compat.h>
 #include <ArgusModeler.h>
 
 #include <netinet/tcp.h>
@@ -38,7 +42,6 @@
 
 extern void ArgusZeroRecord(struct ArgusFlowStruct *);
 
-static struct ArgusTCPObjectMetrics *ArgusThisTCPsrc, *ArgusThisTCPdst;
 
 /* These tcp optinos do not have the size octet */
 #define ZEROLENOPT(o) ((o) == TCPOPT_EOL || (o) == TCPOPT_NOP)
@@ -50,21 +53,24 @@ static struct ArgusTCPObjectMetrics *ArgusThisTCPsrc, *ArgusThisTCPdst;
 #define TH_CWR  0x80
 #endif
 
-void ArgusParseTCPOptions(struct ArgusModelerStruct *, struct tcphdr *, int, u_int *);
+void ArgusParseTCPOptions(struct ArgusModelerStruct *, struct tcphdr *, int, u_int *, struct ArgusTCPObjectMetrics *);
 void ArgusInitializeTCP (struct ArgusModelerStruct *, struct ArgusFlowStruct *);
 void ArgusUpdateTCPState (struct ArgusModelerStruct *, struct ArgusFlowStruct *, unsigned char *);
 int ArgusUpdateTCPSequence (struct ArgusModelerStruct *, struct ArgusFlowStruct *, struct tcphdr *);
+void ArgusTCPKeystroke (struct ArgusModelerStruct *, struct ArgusFlowStruct *, unsigned char *);
 int ArgusUpdateTCPStateMachine (struct ArgusModelerStruct *, struct ArgusFlowStruct *, struct tcphdr *);
 void ArgusTCPFlowRecord (struct ArgusNetworkStruct *, unsigned char);
+long long RaDiffTime (struct ArgusTime *, struct ArgusTime *, struct timeval *);
+
 
 
 #include <errno.h>
 #include <string.h>
 
-
 void
 ArgusUpdateTCPState (struct ArgusModelerStruct *model, struct ArgusFlowStruct *flowstr, unsigned char *state)
 {
+   struct ArgusTCPObjectMetrics *ArgusThisTCPsrc, *ArgusThisTCPdst;
    struct tcphdr *thdr = (struct tcphdr *) model->ArgusThisUpHdr;
    struct tcphdr tcpbuf, *tcp = &tcpbuf;
    struct ArgusTCPObject *tcpExt = NULL;
@@ -100,11 +106,16 @@ ArgusUpdateTCPState (struct ArgusModelerStruct *model, struct ArgusFlowStruct *f
          model->ArgusThisLength  = tcpdatalen;
          model->ArgusThisUpHdr  += tcphlen;
 
-         ArgusThisTCPsrc = &tcpExt->src;
-         ArgusThisTCPdst = &tcpExt->dst;
+         if (model->ArgusThisDir) {
+            ArgusThisTCPsrc = &tcpExt->src;
+            ArgusThisTCPdst = &tcpExt->dst;
+         } else {
+            ArgusThisTCPsrc = &tcpExt->dst;
+            ArgusThisTCPdst = &tcpExt->src;
+         }
 
          if ((tcphlen -= sizeof(*tcp)) > 0)
-            ArgusParseTCPOptions (model, thdr, tcphlen, &tcpExt->options);
+            ArgusParseTCPOptions (model, thdr, tcphlen, &tcpExt->options, ArgusThisTCPsrc);
 
          if (flags & TH_RST) {
             tcpExt->status |= ARGUS_RESET;
@@ -115,11 +126,9 @@ ArgusUpdateTCPState (struct ArgusModelerStruct *model, struct ArgusFlowStruct *f
 
             if (tcp->th_seq) {
                ArgusThisTCPsrc->seqbase  = tcp->th_seq - 1;
-               ArgusThisTCPsrc->ackbytes = tcp->th_seq - 1;
                ArgusThisTCPsrc->seq      = tcp->th_seq + model->ArgusThisLength;
             } else {
                ArgusThisTCPsrc->seqbase  = tcp->th_ack - 1;
-               ArgusThisTCPsrc->ackbytes = tcp->th_ack - 1;
                ArgusThisTCPsrc->seq      = tcp->th_ack - 1;
             }
 
@@ -135,7 +144,6 @@ ArgusUpdateTCPState (struct ArgusModelerStruct *model, struct ArgusFlowStruct *f
                   tcpExt->state        = TCPS_SYN_SENT;
                   ArgusThisTCPsrc->bytes   += model->ArgusThisLength;
                   ArgusThisTCPsrc->seqbase  = tcp->th_seq; 
-                  ArgusThisTCPsrc->ackbytes = tcp->th_seq; 
                   ArgusThisTCPsrc->seq      = tcp->th_seq; 
                   ArgusThisTCPsrc->win      = tcp->th_win;
                   ArgusThisTCPsrc->flags   |= flags; 
@@ -149,7 +157,6 @@ ArgusUpdateTCPState (struct ArgusModelerStruct *model, struct ArgusFlowStruct *f
                   tcpExt->state        = TCPS_SYN_RECEIVED;
                   ArgusThisTCPsrc->bytes   += model->ArgusThisLength;
                   ArgusThisTCPsrc->seqbase  = tcp->th_seq;
-                  ArgusThisTCPsrc->ackbytes = tcp->th_seq;
                   ArgusThisTCPsrc->seq      = tcp->th_seq;
                   ArgusThisTCPsrc->win      = tcp->th_win;
                   ArgusThisTCPsrc->flags   |= flags; 
@@ -175,7 +182,6 @@ ArgusUpdateTCPState (struct ArgusModelerStruct *model, struct ArgusFlowStruct *f
                   ArgusThisTCPsrc->bytes   += model->ArgusThisLength;
                   ArgusThisTCPsrc->flags   |= flags; 
                   ArgusThisTCPsrc->seqbase  = tcp->th_seq - 1;
-                  ArgusThisTCPsrc->ackbytes = tcp->th_seq - 1;
 
                   ArgusThisTCPsrc->seq      = tcp->th_seq + model->ArgusThisLength;
                   ArgusThisTCPsrc->win      = tcp->th_win;
@@ -188,7 +194,6 @@ ArgusUpdateTCPState (struct ArgusModelerStruct *model, struct ArgusFlowStruct *f
                   ArgusThisTCPsrc->bytes   += model->ArgusThisLength;
                   ArgusThisTCPsrc->flags   |= flags; 
                   ArgusThisTCPsrc->seqbase  = tcp->th_seq - 1;
-                  ArgusThisTCPsrc->ackbytes = tcp->th_seq - 1;
 
                   ArgusThisTCPsrc->seq      = tcp->th_seq + model->ArgusThisLength;
                   ArgusThisTCPsrc->win      = tcp->th_win;
@@ -200,7 +205,6 @@ ArgusUpdateTCPState (struct ArgusModelerStruct *model, struct ArgusFlowStruct *f
                   ArgusThisTCPsrc->bytes   += model->ArgusThisLength;
                   ArgusThisTCPsrc->flags   |= flags; 
                   ArgusThisTCPsrc->seqbase  = tcp->th_seq - 1;
-                  ArgusThisTCPsrc->ackbytes = tcp->th_seq - 1;
 
                   ArgusThisTCPsrc->seq      = tcp->th_seq + model->ArgusThisLength;
                   
@@ -234,9 +238,10 @@ ArgusUpdateTCPState (struct ArgusModelerStruct *model, struct ArgusFlowStruct *f
                   ArgusInitializeTCP (model, flowstr);
                   ArgusRemoveFromQueue(flowstr->qhdr.queue, &flowstr->qhdr, ARGUS_LOCK);
                   ArgusPushQueue(model->ArgusStatusQueue, &flowstr->qhdr, ARGUS_LOCK);
-                  ArgusUpdateBasicFlow (model, flowstr, ARGUS_START);
+
                   *state = ARGUS_START;
-                  ArgusUpdateTCPState (model, flowstr, state);
+//                ArgusUpdateBasicFlow (model, flowstr, ARGUS_START);
+//                ArgusUpdateTCPState (model, flowstr, state);
                }
                ArgusThisTCPsrc->flags   |= flags;
                break;
@@ -246,14 +251,8 @@ ArgusUpdateTCPState (struct ArgusModelerStruct *model, struct ArgusFlowStruct *f
                if (getArgusTCPflag(model)) {
                   flowstr->canon.net.hdr.subtype          = ARGUS_TCP_PERF;
                   flowstr->canon.net.hdr.argus_dsrvl8.len = ((sizeof(struct ArgusTCPObject)+3))/4 + 1;
-               }
-
-               if (model->ArgusThisDir) {
-                  ArgusThisTCPsrc = &tcpExt->src;
-                  ArgusThisTCPdst = &tcpExt->dst;
                } else {
-                  ArgusThisTCPsrc = &tcpExt->dst;
-                  ArgusThisTCPdst = &tcpExt->src;
+                  flowstr->canon.net.hdr.subtype          = ARGUS_TCP_STATUS;
                }
 
                model->ArgusSnapLength -= tcphlen;
@@ -261,14 +260,17 @@ ArgusUpdateTCPState (struct ArgusModelerStruct *model, struct ArgusFlowStruct *f
                model->ArgusThisUpHdr  += tcphlen;
 
                if ((tcphlen - sizeof(*tcp)) > 0)
-                  ArgusParseTCPOptions (model, thdr, (tcphlen - sizeof(*tcp)), &tcpExt->options);
+                  ArgusParseTCPOptions (model, thdr, (tcphlen - sizeof(*tcp)), &tcpExt->options, ArgusThisTCPsrc);
    
                ArgusThisTCPsrc->flags |= flags;
 
                if ((tcpExt->options & ARGUS_ECN_FLAGS) && (flags & TH_ECE)) {
                   if (flags & TH_ACK) {
-                     tcpExt->status |= ARGUS_ECN_CONGESTED;
-                     ArgusThisTCPdst->state = ARGUS_ECN_CONGESTED;
+                     if (model->ArgusThisDir) {
+                        tcpExt->status |= ARGUS_SRC_CONGESTED;
+                     } else {
+                        tcpExt->status |= ARGUS_DST_CONGESTED;
+                     }
                   }
                }
 
@@ -309,22 +311,39 @@ ArgusUpdateTCPState (struct ArgusModelerStruct *model, struct ArgusFlowStruct *f
 void
 ArgusInitializeTCP (struct ArgusModelerStruct *model, struct ArgusFlowStruct *flow)
 {
+   struct ArgusSystemFlow *fdsr = (struct ArgusSystemFlow *)flow->dsrs[ARGUS_FLOW_INDEX];
    struct ArgusTCPObject *tcpExt = (struct ArgusTCPObject *)&flow->canon.net.net_union.tcp;
    bzero ((char *)tcpExt, sizeof(*tcpExt));
 
+   if (fdsr) {
+      if (model->ArgusThisDir)
+         fdsr->hdr.argus_dsrvl8.qual &= ~ARGUS_DIRECTION;
+      else
+         fdsr->hdr.argus_dsrvl8.qual |=  ARGUS_DIRECTION;
+   }
+
    flow->qhdr.lasttime = model->ArgusGlobalTime;
-   ArgusUpdateFlow (model, flow, ARGUS_START);
+   ArgusUpdateFlow (model, flow, ARGUS_START, 0);
 }
 
 
 int
 ArgusUpdateTCPStateMachine (struct ArgusModelerStruct *model, struct ArgusFlowStruct *flowstr, struct tcphdr *tcp)
 {
+   struct ArgusTCPObjectMetrics *ArgusThisTCPsrc, *ArgusThisTCPdst;
    unsigned char flags = tcp->th_flags;
    struct ArgusTCPObject *tcpExt = (struct ArgusTCPObject *)&flowstr->canon.net.net_union.tcp;
    unsigned int state = tcpExt->state;
    int len = model->ArgusThisLength;
 
+   if (model->ArgusThisDir) {
+      ArgusThisTCPsrc = &tcpExt->src;
+      ArgusThisTCPdst = &tcpExt->dst;
+   } else {
+      ArgusThisTCPsrc = &tcpExt->dst;
+      ArgusThisTCPdst = &tcpExt->src;
+   }
+
    if (flags & TH_RST) {
       tcpExt->status |= ARGUS_RESET;
       ArgusThisTCPsrc->status |= ARGUS_RESET;
@@ -332,10 +351,15 @@ ArgusUpdateTCPStateMachine (struct ArgusModelerStruct *model, struct ArgusFlowSt
       if (state == TCPS_SYN_SENT) {
          if ((ArgusThisTCPdst->seq == ArgusThisTCPsrc->ack))
              state = TCPS_LISTEN;
-      } else
+         else
+             state = TCPS_CLOSING;
+      } else { 
          if ((tcp->th_seq >= ArgusThisTCPsrc->ack) &&
                     (tcp->th_seq < (ArgusThisTCPsrc->ack + (ArgusThisTCPsrc->win >> ArgusThisTCPsrc->winshift))))
             state = TCPS_CLOSED;
+         else
+             state = TCPS_CLOSING;
+      }
 
    } else {
       switch (state) {
@@ -490,6 +514,8 @@ ArgusUpdateTCPStateMachine (struct ArgusModelerStruct *model, struct ArgusFlowSt
 int
 ArgusUpdateTCPSequence (struct ArgusModelerStruct *model, struct ArgusFlowStruct *flowstr, struct tcphdr *tcp)
 {
+   struct ArgusTCPObjectMetrics *ArgusThisTCPsrc, *ArgusThisTCPdst;
+   struct ArgusTCPObject *tcpExt = (struct ArgusTCPObject *)&flowstr->canon.net.net_union.tcp;
    unsigned char flags = tcp->th_flags;
    int len = model->ArgusThisLength;
 
@@ -497,6 +523,28 @@ ArgusUpdateTCPSequence (struct ArgusModelerStruct *model, struct ArgusFlowStruct
    unsigned int maxseq = 0;
    unsigned int seq = tcp->th_seq;
    unsigned int newseq = seq + len;
+   struct ArgusIPAttrStruct *attr = NULL;
+   unsigned short ipid = 0, *tipid;
+
+   if (model->ArgusThisDir) {
+      ArgusThisTCPsrc = &tcpExt->src;
+      ArgusThisTCPdst = &tcpExt->dst;
+   } else {
+      ArgusThisTCPsrc = &tcpExt->dst;
+      ArgusThisTCPdst = &tcpExt->src;
+   }
+ 
+   if ((attr = (struct ArgusIPAttrStruct *) flowstr->dsrs[ARGUS_IPATTR_INDEX]) != NULL) {
+      if (model->ArgusThisDir) {
+         if (attr->hdr.argus_dsrvl8.qual & ARGUS_IPATTR_SRC)
+            ipid = attr->src.ip_id;
+         tipid = &flowstr->sipid;
+      } else {
+         if (attr->hdr.argus_dsrvl8.qual & ARGUS_IPATTR_DST) 
+            ipid = attr->dst.ip_id;
+         tipid = &flowstr->dipid;
+      }
+   }
 
    if (flags == TH_SYN)           /* we started over */
       return (retn);
@@ -534,20 +582,19 @@ ArgusUpdateTCPSequence (struct ArgusModelerStruct *model, struct ArgusFlowStruct
       model->ArgusInProtocol = 0;
 
    if (newseq < seq) {                                 /* we rolled over */
+      ArgusThisTCPsrc->ackbytes += ArgusThisTCPsrc->seq - ArgusThisTCPsrc->seqbase; /* ackbytes holds the number of rollover bytes */
       ArgusThisTCPsrc->seqbase = newseq;
-      ArgusThisTCPsrc->ackbytes = newseq;
       ArgusThisTCPsrc->seq = newseq;
 
    } else {
       if (!ArgusThisTCPsrc->seqbase) {                 /* first packet in this direction */
          ArgusThisTCPsrc->seqbase = seq;
-         ArgusThisTCPsrc->ackbytes = seq;
          ArgusThisTCPsrc->seq = newseq;
       } else {
          if (len) {
             if (ArgusThisTCPdst->win != 0) {           /* not first packet seen in this direction */
                if (tcp->th_seq < ArgusThisTCPdst->ack) {
-                  if ((ArgusThisTCPsrc->ack - tcp->th_seq) < (ArgusThisTCPsrc->win >> ArgusThisTCPsrc->winshift)) {
+                  if ((ArgusThisTCPdst->ack - tcp->th_seq) < (ArgusThisTCPsrc->win >> ArgusThisTCPsrc->winshift)) {
                      ArgusThisTCPsrc->retrans++;
                      ArgusThisTCPsrc->status |= ARGUS_PKTS_RETRANS;
                      ArgusThisTCPsrc->winbytes -= len;
@@ -562,6 +609,10 @@ ArgusUpdateTCPSequence (struct ArgusModelerStruct *model, struct ArgusFlowStruct
                   }
 
                   if (ArgusThisTCPsrc->win) {
+                     int dipid;
+                     if (*tipid && ((dipid = (ipid - *tipid)) < 0) && (dipid > -5000)) {
+                        ArgusThisTCPsrc->status |= ARGUS_OUTOFORDER;
+                     } else
                      if (ArgusThisTCPsrc->winbytes > ((maxseq - 1) - ArgusThisTCPdst->ack)) {
                         ArgusThisTCPsrc->retrans++;
                         ArgusThisTCPsrc->status |= ARGUS_PKTS_RETRANS;
@@ -590,8 +641,6 @@ ArgusUpdateTCPSequence (struct ArgusModelerStruct *model, struct ArgusFlowStruct
       }
    }
 
-   if (flags & TH_FIN)
-      ArgusThisTCPdst->seq = tcp->th_seq;
 
    if (tcp->th_ack && (flags & TH_ACK)) {
       if (ArgusThisTCPsrc->ack) {
@@ -621,12 +670,253 @@ ArgusUpdateTCPSequence (struct ArgusModelerStruct *model, struct ArgusFlowStruct
       }
    }
 
+   if (*tipid < ipid)
+      *tipid = ipid;
+
 /* ArgusInProtocol = 1; */
 
    return (retn);
 }
 
 
+void
+ArgusTCPKeystroke (struct ArgusModelerStruct *model, struct ArgusFlowStruct *flowstr, unsigned char *state)
+{
+   struct ArgusKeyStrokeConf *ArgusKeyStroke = &model->ArgusKeyStroke;
+   struct tcphdr *thdr = (struct tcphdr *) model->ArgusThisUpHdr;
+   struct tcphdr tcpbuf, *tcp = &tcpbuf;
+
+   if (thdr && STRUCTCAPTURED(model, *thdr)) {
+      int tcplen = model->ArgusThisLength;
+      int tcphlen = thdr->th_off * 4;
+      int tcpdatalen = tcplen - tcphlen;
+
+      if (ArgusKeyStroke->status & ARGUS_SSH_KEYSTROKE)
+         if (!(flowstr->status & ARGUS_SSH_MONITOR))
+            return;
+
+      if (++flowstr->skey.n_pkts < ArgusKeyStroke->n_min) {
+#ifdef ARGUSDEBUG
+         ArgusDebug (5, "ArgusTCPKeystroke: flow %p packet %lld flow_n_pkts(%d) < n_min(%d)\n", 
+            flowstr, model->ArgusTotalPacket, flowstr->skey.n_pkts, ArgusKeyStroke->n_min);
+#endif
+         return;
+      } else {
+#ifdef ARGUSDEBUG
+         ArgusDebug (5, "ArgusTCPKeystroke: flow %p packet %lld flow_n_pkts(%d)\n", 
+            flowstr, model->ArgusTotalPacket, flowstr->skey.n_pkts);
+#endif
+      }
+
+      if (tcpdatalen > 0) {
+#ifdef _LITTLE_ENDIAN
+         bzero ((char *)tcp, sizeof(tcpbuf));
+         tcp->th_dport = ntohs(thdr->th_dport);
+         tcp->th_sport = ntohs(thdr->th_sport);
+         tcp->th_seq   = ntohl(thdr->th_seq);
+         tcp->th_ack   = ntohl(thdr->th_ack);
+         tcp->th_win   = ntohs(thdr->th_win);
+         tcp->th_flags = thdr->th_flags;
+#else
+         bcopy ((char *) thdr, (char *)tcp, sizeof(tcpbuf));
+#endif
+
+         if (ArgusKeyStroke->status & ARGUS_SSH_KEYSTROKE) {
+            if (tcpdatalen % 4) {
+               flowstr->status &= ~ARGUS_SSH_MONITOR;
+#ifdef ARGUSDEBUG
+               ArgusDebug (5, "ArgusTCPKeystroke: packet %lld flow %p ssh monitor and tcpdatalen(%d) not mod 4\n", flowstr->skey.n_pkts, flowstr, tcpdatalen);
+#endif
+               return;
+            }
+         }
+
+         if (flowstr->dsrs[ARGUS_BEHAVIOR_INDEX] == NULL) {
+            flowstr->dsrs[ARGUS_BEHAVIOR_INDEX] = (struct ArgusDSRHeader *) &flowstr->canon.actor.hdr;
+            flowstr->dsrindex |= 1 << ARGUS_BEHAVIOR_INDEX;
+         }
+
+#define CLIENT	1
+#define SERVER	0
+
+         switch (model->ArgusThisDir) {
+            case CLIENT: {
+               if ((tcpdatalen >= ArgusKeyStroke->dc_min) && (tcpdatalen <= ArgusKeyStroke->dc_max)) {
+                  int i = 0, found = 0;
+#ifdef ARGUSDEBUG
+                  ArgusDebug (5, "ArgusTCPKeystroke: flow %p packet %lld client packet seq(%u) size(%d) OK\n", flowstr, flowstr->skey.n_pkts, tcp->th_seq, tcpdatalen);
+#endif
+                  for (i = 0; i < ARGUS_NUM_KEYSTROKE_PKTS && !found; i++) {
+                     struct ArgusKeyStrokePacket *pkt = &flowstr->skey.data.pkts[i];
+
+                     if (pkt->n_pno == 0) {   // this is an empty slot, so use it
+                        struct timeval tvpbuf, *tvp = &tvpbuf;
+                        pkt->ts.tv_sec  = model->ArgusGlobalTime.tv_sec;
+                        pkt->ts.tv_usec = model->ArgusGlobalTime.tv_usec;
+                        pkt->intpkt = RaDiffTime (&pkt->ts, &flowstr->skey.prev_c_ts, tvp);
+                        pkt->seq = tcp->th_seq + tcpdatalen;
+                        pkt->n_pno = flowstr->skey.n_pkts;
+                        found = 1;
+                     }
+                  }
+                  if (!found) {   // don't have an empty slot, so create one.  clear earliest packet number client packet. keep tentatives
+                     int lpno = 0x7FFFFFFF, npkt = -1;
+                     for (i = 0; i < ARGUS_NUM_KEYSTROKE_PKTS; i++) {
+                        struct ArgusKeyStrokePacket *pkt = &flowstr->skey.data.pkts[i];
+                        if (pkt->status != ARGUS_KEYSTROKE_TENTATIVE) {
+                           if (pkt->n_pno < lpno) {
+                              npkt = i;
+                              lpno = pkt->n_pno;
+                           }
+                        }
+                     }
+                     if (npkt < 0) {  // nothing but tentatives, so lets clear the tentative at index = 0
+#ifdef ARGUSDEBUG
+                        ArgusDebug (5, "ArgusTCPKeystroke: flow %p packet %lld no non-tentative packets to reject, clearing first\n", flowstr, flowstr->skey.n_pkts);
+#endif
+                        npkt = 0;
+                     }
+                     {
+                        struct ArgusKeyStrokePacket *pkt = &flowstr->skey.data.pkts[npkt];
+                        struct timeval tvpbuf, *tvp = &tvpbuf;
+#ifdef ARGUSDEBUG
+                        ArgusDebug (5, "ArgusTCPKeystroke: flow %p packet %lld clearing client %d seq(%u)\n", flowstr, flowstr->skey.n_pkts, npkt, pkt->seq);
+#endif
+                        pkt->ts.tv_sec  = model->ArgusGlobalTime.tv_sec;
+                        pkt->ts.tv_usec = model->ArgusGlobalTime.tv_usec;
+                        pkt->intpkt = RaDiffTime (&pkt->ts, &flowstr->skey.prev_c_ts, tvp);
+                        pkt->seq = tcp->th_seq + tcpdatalen;
+                        pkt->n_pno = flowstr->skey.n_pkts;
+
+                     }
+                  }
+               }
+#ifdef ARGUSDEBUG
+                 else {
+                  ArgusDebug (5, "ArgusTCPKeystroke: flow %p packet %lld client packet size(%d) NOK reset\n", flowstr, flowstr->skey.n_pkts, tcpdatalen);
+               }
+#endif
+               break;
+            }
+
+            case SERVER: {
+               struct ArgusKeyStrokePacket *pkt = NULL;
+               int i, found = 0, reject = 1;
+
+               for (i = 0; i < ARGUS_NUM_KEYSTROKE_PKTS && !found; i++) {
+                  pkt = &flowstr->skey.data.pkts[i];
+                  if ((pkt->status == 0) && (pkt->seq == tcp->th_ack))
+                     found++;
+               }
+
+               if (found) {
+                  struct ArgusTime stime;
+
+                  stime.tv_sec  = model->ArgusGlobalTime.tv_sec;
+                  stime.tv_usec = model->ArgusGlobalTime.tv_usec;
+
+                  if ((flowstr->skey.n_pkts - pkt->n_pno) <= ArgusKeyStroke->gs_max) {
+                     if ((tcpdatalen >= ArgusKeyStroke->ds_min) && (tcpdatalen <= ArgusKeyStroke->ds_max)) {
+                        struct timeval tvpbuf, *tvp = &tvpbuf;
+
+                        if (pkt->intpkt >= ArgusKeyStroke->ic_min) {
+                           long long slint = RaDiffTime (&stime, &flowstr->skey.prev_s_ts, tvp);
+
+                           float ic_ratio = (pkt->intpkt * 1.0) / (slint * 1.0);
+                           if ((ic_ratio >= ArgusKeyStroke->icr_min) && (ic_ratio <= ArgusKeyStroke->icr_max)) {
+                              if ((pkt->n_pno - flowstr->skey.prev_pno) <= ArgusKeyStroke->gpc_max) {
+                                 int i;
+#ifdef ARGUSDEBUG
+                                 ArgusDebug (5, "ArgusTCPKeystroke: flow %p packet %lld server keystroke ", flowstr, flowstr->skey.n_pkts);
+#endif
+                                 flowstr->skey.n_strokes++;
+                                 for (i = 0; i < ARGUS_NUM_KEYSTROKE_PKTS; i++) {
+                                    struct ArgusKeyStrokePacket *tpkt = &flowstr->skey.data.pkts[i];
+                                    if (tpkt->status == ARGUS_KEYSTROKE_TENTATIVE) {
+#ifdef ARGUSDEBUG
+                                       ArgusDebug (5, "ArgusTCPKeystroke: flow %p packet %lld TENTATIVE packet %lld keystroke ", flowstr, flowstr->skey.n_pkts, tpkt->n_pno);
+#endif
+                                       flowstr->skey.n_strokes++;
+                                       bzero(tpkt, sizeof(*tpkt));
+                                    }
+                                 }
+                                 flowstr->skey.prev_pno  = pkt->n_pno;
+                                 flowstr->skey.prev_c_ts = pkt->ts;
+                                 flowstr->skey.prev_s_ts = stime;
+                                 bzero(pkt, sizeof(*pkt));
+
+                              } else {
+                                 for (i = 0; i < ARGUS_NUM_KEYSTROKE_PKTS; i++) {
+                                    struct ArgusKeyStrokePacket *tpkt = &flowstr->skey.data.pkts[i];
+                                    if (tpkt->status == ARGUS_KEYSTROKE_TENTATIVE)
+                                       bzero(tpkt, sizeof(*tpkt));
+                                 }
+                                 pkt->status = ARGUS_KEYSTROKE_TENTATIVE;
+#ifdef ARGUSDEBUG
+                                 ArgusDebug (5, "ArgusTCPKeystroke: flow %p packet %lld server packet pc_gap(%d) > gpc_max(%d) TENTATIVE\n", 
+                                       flowstr, flowstr->skey.n_pkts, (pkt->n_pno - flowstr->skey.prev_pno) , ArgusKeyStroke->gpc_max);
+#endif
+                                 flowstr->skey.prev_pno  = pkt->n_pno;
+                                 flowstr->skey.prev_c_ts = pkt->ts;
+                                 flowstr->skey.prev_s_ts = stime;
+                              }
+                              reject = 0;
+
+                           } 
+#ifdef ARGUSDEBUG
+                           else {
+                                 ArgusDebug (5, "ArgusTCPKeystroke: flow %p packet %lld server ic_ratio(%f) out of range\n", 
+                                    flowstr, flowstr->skey.n_pkts, ic_ratio);
+                           }
+#endif
+                        } 
+#ifdef ARGUSDEBUG
+                        else {
+                           ArgusDebug (5, "ArgusTCPKeystroke: flow %p packet %lld rejected server IPA(%lld) < ic_min(%d)\n", 
+                                    flowstr, flowstr->skey.n_pkts, pkt->intpkt, ArgusKeyStroke->ic_min);
+                        }
+#endif
+                     } 
+#ifdef ARGUSDEBUG
+                     else {
+                        ArgusDebug (5, "ArgusTCPKeystroke: flow %p packet %lld rejected server packet size(%d)\n", flowstr, flowstr->skey.n_pkts, tcpdatalen);
+                     }
+#endif
+                  } 
+#ifdef ARGUSDEBUG
+                  else {
+                     ArgusDebug (5, "ArgusTCPKeystroke: flow %p packet %lld rejected server packet flow s_gap(%d) > gs_gap(%d)\n", 
+                           flowstr, flowstr->skey.n_pkts, (flowstr->skey.n_pkts - pkt->n_pno), ArgusKeyStroke->gs_max);
+                  }
+#endif
+                  if (reject) {
+                     for (i = 0; i < ARGUS_NUM_KEYSTROKE_PKTS; i++) {
+                        struct ArgusKeyStrokePacket *tpkt = &flowstr->skey.data.pkts[i];
+                        if (tpkt->status == ARGUS_KEYSTROKE_TENTATIVE)
+                           bzero(tpkt, sizeof(*tpkt));
+                     }
+                     bzero(pkt, sizeof(*pkt));
+                  }
+               } 
+#ifdef ARGUSDEBUG
+               else {
+                  ArgusDebug (5, "ArgusTCPKeystroke: flow %p packet %lld server packet ack(%u) no match\n", 
+                     flowstr, flowstr->skey.n_pkts, tcp->th_ack);
+               }
+#endif
+            }
+         }
+
+      } 
+#ifdef ARGUSDEBUG
+      else {
+         ArgusDebug (5, "ArgusTCPKeystroke: flow %p packet %lld %s packet rejected tcpdatalen(%d)\n", 
+               flowstr, flowstr->skey.n_pkts, (model->ArgusThisDir ? "server" : "client"), tcpdatalen);
+      }
+#endif
+   }
+}
+
 #include <argus_out.h>
 
 void
@@ -674,7 +964,7 @@ ArgusTCPFlowRecord (struct ArgusNetworkStruct *net, unsigned char state)
 }
 
 void
-ArgusParseTCPOptions(struct ArgusModelerStruct *model, struct tcphdr *tcp, int len, u_int *options)
+ArgusParseTCPOptions(struct ArgusModelerStruct *model, struct tcphdr *tcp, int len, u_int *options, struct ArgusTCPObjectMetrics *ArgusThisTCPsrc)
 {
    register const u_char *cp;
    register int i, opt, alen, datalen;
