@@ -1,6 +1,6 @@
 /*
  * Argus Software.  Argus files - UDP protocol
- * Copyright (c) 2000-2008 QoSient, LLC
+ * Copyright (c) 2000-2012 QoSient, LLC
  * All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -20,11 +20,15 @@
  */
 
 /* 
- * $Id: $
- * $DateTime: $
- * $Change: $
+ * $Id: //depot/argus/argus/argus/ArgusUdp.c#13 $
+ * $DateTime: 2012/01/03 19:15:19 $
+ * $Change: 2274 $
  */
 
+#ifdef HAVE_CONFIG_H
+#include "argus_config.h"
+#endif
+
 #if !defined(ArgusUdp)
 #define ArgusUdp
 #endif
@@ -33,10 +37,10 @@
 #include <string.h>
 #include <errno.h>
 
-#include <compat.h>
+#include <argus_compat.h>
 #include <ArgusModeler.h>
 
-#include <bootp.h>
+#include <argus/bootp.h>
 struct bootp *bp;
 
 void ArgusUpdateUDPState (struct ArgusModelerStruct *, struct ArgusFlowStruct *, unsigned char *);
