@@ -1,6 +1,6 @@
 /*
  * Argus Software.  Argus files - Utilities
- * Copyright (c) 2000-2008 QoSient, LLC
+ * Copyright (c) 2000-2012 QoSient, LLC
  * All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -20,19 +20,28 @@
  */
 
 /* 
- * $Id: $
- * $DateTime: $
- * $Change: $
+ * $Id: //depot/argus/argus/argus/ArgusUtil.c#70 $
+ * $DateTime: 2012/03/15 00:43:34 $
+ * $Change: 2332 $
  */
 
 /* ArgusUtil.c */
 
+#ifdef HAVE_CONFIG_H
+#include "argus_config.h"
+#endif
+
+#define _GNU_SOURCE
+
 #if !defined(ArgusUtil)
 #define ArgusUtil
 #endif
 
+#include <stdlib.h>
+#include <inttypes.h>
 #include <stdio.h>
 #include <math.h>
+
 #if defined(HAVE_SYS_VFS_H)
 #include <sys/vfs.h>
 #else
@@ -49,23 +58,25 @@
 #include <sasl/sasl.h>
 #endif 
 
-#include <compat.h>
+#define Argus_Parser
+
+#include <argus_compat.h>
+#include <argus_parser.h>
+
 #include <ArgusModeler.h>
+#include <argus_dscodepoints.h>
+#include <argus_encapsulations.h>
+
 
 long long
 ArgusTimeDiff (struct timeval *start, struct timeval *stop)
 {
-   long long retn = 0;
-   int sec, usec;
+   long long retn, stime, etime;
 
-   sec  = start->tv_sec  - stop->tv_sec;
-   usec = start->tv_usec - stop->tv_usec;
+   stime = (start->tv_sec * 1000000LL) + start->tv_usec;
+   etime = (stop->tv_sec  * 1000000LL) +  stop->tv_usec;
 
-   if (usec < 0) {
-      sec--; usec += 1000000;
-   }
-
-   retn = (long long) ((sec * 1000000) + usec);
+   retn = stime - etime;
    return (retn);
 }
 
@@ -74,7 +85,6 @@ ArgusAbsTimeDiff (struct timeval *start, struct timeval *stop)
 {
    unsigned long long retn = 0;
    struct timeval *t1 = start, *t2 = stop;
-   int sec, usec;
 
    if ((stop->tv_sec < start->tv_sec) || ((stop->tv_sec == start->tv_sec) &&
                                           (stop->tv_usec < start->tv_usec))) {
@@ -82,14 +92,9 @@ ArgusAbsTimeDiff (struct timeval *start, struct timeval *stop)
       t1 = stop;
    }
 
-   sec  = t2->tv_sec  - t1->tv_sec;
-   usec = t2->tv_usec - t1->tv_usec;
+   retn = ((t2->tv_sec * 1000000LL) + t2->tv_usec) - 
+          ((t1->tv_sec * 1000000LL) + t1->tv_usec);
 
-   if (usec < 0) {
-      sec--; usec += 1000000;
-   }
-
-   retn = (unsigned long long) ((sec * 1000000) + usec);
    return (retn);
 }
 
@@ -104,13 +109,13 @@ ArgusNewList ()
       retn->count = 0;
 #if defined(ARGUS_THREADS)
       pthread_mutex_init(&retn->lock, NULL);
+      pthread_cond_init(&retn->cond, NULL);
 #endif
    }
 
 #ifdef ARGUSDEBUG
-   ArgusDebug (4, "ArgusNewList () returning 0x%x\n", retn);
+   ArgusDebug (4, "ArgusNewList () returning %p\n", retn);
 #endif
-
    return (retn);
 }
 
@@ -118,6 +123,9 @@ void
 ArgusDeleteList (struct ArgusListStruct *list, int type)
 {
    if (list) {
+#ifdef ARGUSDEBUG
+      ArgusDebug (4, "ArgusDeleteList (%p, %d) %d items on list\n", list, type, list->count);
+#endif
       while (list->start) {
          struct ArgusListRecord *retn = ArgusPopFrontList(list, ARGUS_LOCK);
          switch (type) {
@@ -150,14 +158,35 @@ ArgusDeleteList (struct ArgusListStruct *list, int type)
              case ARGUS_OUTPUT_LIST:
                 ArgusFreeListRecord(retn);
                 break;
+
+             case ARGUS_EVENT_LIST: {
+                struct ArgusListObjectStruct *lobj = (struct ArgusListObjectStruct *) retn;
+                if (lobj->obj != NULL) {
+                   ArgusFree(lobj);
+                }
+                ArgusFree(retn);
+                break;
+            }
+
+             case ARGUS_BIND_ADDR_LIST: {
+                struct ArgusListObjectStruct *lobj = (struct ArgusListObjectStruct *) retn;
+                if (lobj->obj != NULL) {
+                   free(lobj);
+                }
+                ArgusFree(retn);
+                break;
+            }
          }
       }
-
+#if defined(ARGUS_THREADS)
+      pthread_cond_destroy(&list->cond);
+      pthread_mutex_destroy(&list->lock);
+#endif
       ArgusFree (list);
    }
 
 #ifdef ARGUSDEBUG
-   ArgusDebug (4, "ArgusDeleteList (0x%x, %d) returning\n", list, type);
+   ArgusDebug (4, "ArgusDeleteList (%p, %d) returning\n", list, type);
 #endif
 }
 
@@ -202,7 +231,7 @@ ArgusPushFrontList(struct ArgusListStruct *list, struct ArgusListRecord *rec, in
    }
 
 #ifdef ARGUSDEBUG
-   ArgusDebug (6, "ArgusPushFrontList (0x%x, 0x%x, %d) returning 0x%x\n", list, rec, lstat);
+   ArgusDebug (6, "ArgusPushFrontList (%p, %p, %d) returning %p\n", list, rec, lstat);
 #endif
 
    return (retn);
@@ -236,7 +265,7 @@ ArgusPushBackList(struct ArgusListStruct *list, struct ArgusListRecord *rec, int
    }
 
 #ifdef ARGUSDEBUG
-   ArgusDebug (6, "ArgusPushBackList (0x%x, 0x%x, %d) returning %d\n", list, rec, lstat, retn);
+   ArgusDebug (6, "ArgusPushBackList (%p, %p, %d) returning %d\n", list, rec, lstat, retn);
 #endif
 
    return (retn);
@@ -273,7 +302,7 @@ ArgusLoadList(struct ArgusListStruct *l1, struct ArgusListStruct *l2)
 #endif
 
 #ifdef ARGUSDEBUG
-   ArgusDebug (5, "ArgusLoadList (0x%x, 0x%x) load %d objects\n", l1, l2, count);
+   ArgusDebug (5, "ArgusLoadList (%p, %p) load %d objects\n", l1, l2, count);
 #endif
    }
 }
@@ -295,7 +324,7 @@ ArgusPopFrontList(struct ArgusListStruct *list, int lstat)
       if (list->start == NULL) {
          list->end = NULL;
          if (list->count != 0)
-            ArgusLog (LOG_ERR, "ArgusPopFrontList(0x%x, %d) list empty count is %d\n", list, lstat, list->count);
+            ArgusLog (LOG_ERR, "ArgusPopFrontList(%p, %d) list empty count is %d\n", list, lstat, list->count);
       }
    }
 #if defined(ARGUS_THREADS)
@@ -304,7 +333,7 @@ ArgusPopFrontList(struct ArgusListStruct *list, int lstat)
 #endif
 
 #ifdef ARGUSDEBUG
-   ArgusDebug (9, "ArgusPopFrontList (0x%x) returning\n", retn);
+   ArgusDebug (9, "ArgusPopFrontList (%p) returning\n", retn);
 #endif
 
    return (retn);
@@ -326,7 +355,7 @@ ArgusNewQueue ()
    }
 
 #ifdef ARGUSDEBUG
-   ArgusDebug (4, "ArgusNewQueue () returning 0x%x\n", retn);
+   ArgusDebug (4, "ArgusNewQueue () returning %p\n", retn);
 #endif
 
    return (retn);
@@ -342,6 +371,10 @@ ArgusDeleteQueue (struct ArgusQueueStruct *queue)
       pthread_mutex_lock(&queue->lock); 
 #endif
 
+#ifdef ARGUSDEBUG
+      if (queue->count > 0) 
+         ArgusDebug (1, "ArgusDeleteQueue (%p) contains %d items\n", queue, queue->count);
+#endif
       while ((obj = ArgusPopQueue(queue, ARGUS_LOCK)))
          ArgusFree(obj);
 
@@ -361,7 +394,7 @@ ArgusDeleteQueue (struct ArgusQueueStruct *queue)
    }
 
 #ifdef ARGUSDEBUG
-   ArgusDebug (4, "ArgusDeleteQueue (0x%x) returning\n", queue);
+   ArgusDebug (4, "ArgusDeleteQueue (%p) returning\n", queue);
 #endif
 }
 
@@ -372,7 +405,7 @@ ArgusGetQueueCount(struct ArgusQueueStruct *queue)
 {
 
 #ifdef ARGUSDEBUG
-   ArgusDebug (10, "ArgusGetQueueCount (0x%x) returning %d\n", queue, queue->count);
+   ArgusDebug (10, "ArgusGetQueueCount (%p) returning %d\n", queue, queue->count);
 #endif
 
    return (queue->count);
@@ -397,7 +430,7 @@ ArgusPushQueue(struct ArgusQueueStruct *queue, struct ArgusQueueHeader *obj, int
 #endif
 
 #ifdef ARGUSDEBUG
-   ArgusDebug (10, "ArgusPushQueue (0x%x, 0x%x) returning\n", queue, obj);
+   ArgusDebug (10, "ArgusPushQueue (%p, %p) returning\n", queue, obj);
 #endif
 }
 
@@ -430,16 +463,20 @@ ArgusAddToQueue(struct ArgusQueueStruct *queue, struct ArgusQueueHeader *obj, in
             pthread_mutex_unlock(&queue->lock); 
 #endif
          obj->queue = queue;
-         obj->qtime = ArgusModel->ArgusGlobalTime;
+
+         if (ArgusSourceTask->ArgusReadingOffLine)
+            obj->qtime = ArgusModel->ArgusGlobalTime;
+         else
+            gettimeofday(&obj->qtime, 0L);
          retn = 1;
 
       } else
-         ArgusLog (LOG_ERR, "ArgusAddToQueue (0x%x, 0x%x) obj in queue 0x%x\n", queue, obj, obj->queue);
+         ArgusLog (LOG_ERR, "ArgusAddToQueue (%p, %p) obj in queue %p\n", queue, obj, obj->queue);
    } else
-      ArgusLog (LOG_ERR, "ArgusAddToQueue (0x%x, 0x%x) parameter error\n", queue, obj);
+      ArgusLog (LOG_ERR, "ArgusAddToQueue (%p, %p) parameter error\n", queue, obj);
 
 #ifdef ARGUSDEBUG
-   ArgusDebug (10, "ArgusAddToQueue (0x%x, 0x%x) returning %d\n", queue, obj, retn);
+   ArgusDebug (10, "ArgusAddToQueue (%p, %p) returning %d\n", queue, obj, retn);
 #endif
 
    return (retn);
@@ -488,7 +525,7 @@ ArgusPopQueue (struct ArgusQueueStruct *queue, int type)
    }
 
 #ifdef ARGUSDEBUG
-   ArgusDebug (10, "ArgusPopQueue (0x%x) returning 0x%x\n", queue, retn);
+   ArgusDebug (10, "ArgusPopQueue (%p) returning %p\n", queue, retn);
 #endif
    
    return(retn);
@@ -536,7 +573,7 @@ ArgusPopBackQueue (struct ArgusQueueStruct *queue, int type)
    }
 
 #ifdef ARGUSDEBUG
-   ArgusDebug (10, "ArgusPopBackQueue (0x%x) returning 0x%x\n", queue, retn);
+   ArgusDebug (10, "ArgusPopBackQueue (%p) returning %p\n", queue, retn);
 #endif
 
    return(retn);
@@ -581,12 +618,12 @@ ArgusRemoveFromQueue(struct ArgusQueueStruct *queue, struct ArgusQueueHeader *ob
          retn = obj;
 
       } else
-         ArgusLog (LOG_ERR, "ArgusRemoveFromQueue(0x%x, 0x%x) obj not in queue\n", queue, obj);
+         ArgusLog (LOG_ERR, "ArgusRemoveFromQueue(%p, %p) obj not in queue\n", queue, obj);
    } else
-      ArgusLog (LOG_ERR, "ArgusRemoveFromQueue(0x%x, 0x%x) parameter error\n", queue, obj);
+      ArgusLog (LOG_ERR, "ArgusRemoveFromQueue(%p, %p) parameter error\n", queue, obj);
 
 #ifdef ARGUSDEBUG
-   ArgusDebug (10, "ArgusRemoveFromQueue (0x%x, 0x%x) returning 0x%x\n", queue, obj, obj);
+   ArgusDebug (10, "ArgusRemoveFromQueue (%p, %p) returning %p\n", queue, obj, obj);
 #endif
 
    return (retn);
@@ -605,7 +642,7 @@ ArgusProcessQueue(struct ArgusModelerStruct *model, struct ArgusQueueStruct *que
       case ARGUS_STOP:
       case ARGUS_SHUTDOWN:
 #ifdef ARGUSDEBUG
-         ArgusDebug (3, "ArgusProcessQueue (0x%x, %d) Shuting Down with %d records\n", queue, status, queue->count);
+         ArgusDebug (3, "ArgusProcessQueue (%p, %d) Shuting Down with %d records\n", queue, status, queue->count);
 #endif
          while (queue->count) {
             if ((obj = (struct ArgusFlowStruct *) ArgusPopBackQueue(queue, ARGUS_LOCK)) != NULL) {
@@ -618,7 +655,7 @@ ArgusProcessQueue(struct ArgusModelerStruct *model, struct ArgusQueueStruct *que
    }
 
 #ifdef ARGUSDEBUG
-   ArgusDebug (8, "ArgusProcessQueue (0x%x, %d) returning\n", queue, status);
+   ArgusDebug (8, "ArgusProcessQueue (%p, %d) returning\n", queue, status);
 #endif
 }
 
@@ -626,29 +663,23 @@ ArgusProcessQueue(struct ArgusModelerStruct *model, struct ArgusQueueStruct *que
 int
 ArgusCheckTimeout(struct ArgusModelerStruct *model, struct timeval *ts, struct timeval *timeout)
 {
-   unsigned int lapseTimeSec = 0, lapseTimeuSec = 0;
-   struct timeval now = {0, 0};
-   int retn = 0;
+   long long diff = 0, tdiff = 0;
+   int retn;
 
    if ((timeout->tv_sec > 0) || (timeout->tv_usec > 0)) {
-      lapseTimeSec  = ts->tv_sec  + timeout->tv_sec;
-      lapseTimeuSec = ts->tv_usec + timeout->tv_usec;
+      diff  = ArgusTimeDiff (&model->ArgusGlobalTime, ts);
+      tdiff = (timeout->tv_sec * 1000000LL + timeout->tv_usec);
 
-      if (lapseTimeuSec > 1000000) {
-         lapseTimeSec++;
-         lapseTimeuSec -= 1000000;
-      }
-
-      now = model->ArgusGlobalTime;
-
-      if ((now.tv_sec > lapseTimeSec) || ((now.tv_sec == lapseTimeSec) && (now.tv_usec >= lapseTimeuSec)))
+      if (diff >= tdiff)
          retn = 1;
+      else
+         retn = 0;
    } else
       retn = 1;
 
 #ifdef ARGUSDEBUG
-   ArgusDebug (8, "ArgusCheckTimeout (0x%x, %d.%06d, %6d.%06d) lapseTime %d.%06d currentTime %d.%06d returning %d\n", model, ts->tv_sec, ts->tv_usec,
-                     timeout->tv_sec, timeout->tv_usec, lapseTimeSec, lapseTimeuSec, now.tv_sec, now.tv_usec, retn);
+   ArgusDebug (11, "ArgusCheckTimeout (%p, %d.%06d, %d.%06d) diff %f returning %d\n", model, ts->tv_sec, ts->tv_usec,
+                      timeout->tv_sec, timeout->tv_usec, (diff / 1000000.0), retn);
 #endif
 
    return (retn);
@@ -668,7 +699,7 @@ ArgusDeleteObject(struct ArgusFlowStruct *obj)
          if (ArgusRemoveFromQueue (queue, &obj->qhdr, ARGUS_LOCK)) {
             obj->qhdr.queue = NULL;
          } else 
-            ArgusLog (LOG_ERR, "ArgusDeleteObject: race condition on queue 0x%x\n", queue);
+            ArgusLog (LOG_ERR, "ArgusDeleteObject: race condition on queue %p\n", queue);
       }
  
       if ((htblhdr = obj->htblhdr) != NULL)  {
@@ -713,7 +744,7 @@ ArgusDeleteObject(struct ArgusFlowStruct *obj)
    }
 
 #ifdef ARGUSDEBUG
-   ArgusDebug (4, "ArgusDeleteObject (0x%x) returning\n", obj);
+   ArgusDebug (4, "ArgusDeleteObject (%p) returning\n", obj);
 #endif
 }
 
@@ -722,30 +753,71 @@ int
 ArgusUpdateTime (struct ArgusModelerStruct *model)
 {
    int retn = 0;
+   long long diff;
+   long long ival;
 
-   if (model->ArgusUpdateTimer.tv_sec == 0)
+   if (model->ArgusUpdateTimer.tv_sec == 0) 
       model->ArgusUpdateTimer = model->ArgusGlobalTime;
 
-   if ((model->ArgusGlobalTime.tv_sec >  model->ArgusUpdateTimer.tv_sec) ||
-      ((model->ArgusGlobalTime.tv_sec == model->ArgusUpdateTimer.tv_sec) &&
-      (model->ArgusGlobalTime.tv_usec >= model->ArgusUpdateTimer.tv_usec))) {
+   diff = ArgusTimeDiff(&model->ArgusGlobalTime, &model->ArgusUpdateTimer);
+   ival = ((model->ArgusUpdateInterval.tv_sec * 1000000LL) + model->ArgusUpdateInterval.tv_usec);
 
+   if (diff >= 0) {
       retn = 1;
 
+      if (diff > ival) 
+         model->ArgusUpdateTimer = model->ArgusGlobalTime;
+
       model->ArgusUpdateTimer.tv_sec  += model->ArgusUpdateInterval.tv_sec;
       model->ArgusUpdateTimer.tv_usec += model->ArgusUpdateInterval.tv_usec;
 
-      if (model->ArgusUpdateTimer.tv_usec >= 1000000) {
+      while (model->ArgusUpdateTimer.tv_usec >= 1000000) {
          model->ArgusUpdateTimer.tv_sec++; 
          model->ArgusUpdateTimer.tv_usec -= 1000000;
       }
+
+   } else {
+
+      if (ArgusSourceTask != NULL) {
+         if (!(ArgusSourceTask->ArgusReadingOffLine)) {
+            if (llabs(diff) > (ival * 2)) {
+
+// something is wrong, so try to figure out if ArgusGlobalTime needs to be adjusted.
+// Must be kernel time bug, so try to reset the ArgusUpdateTimer, and declare
+// that the timer has popped.  Redefine global timer if needed.
+
+               unsigned long long tdiff;
+               struct timeval now;
+
+               retn = 1;
+
+               gettimeofday(&now, 0L);
+               tdiff =  ArgusAbsTimeDiff(&now, &model->ArgusGlobalTime);
+
+               if (tdiff > (ival * 2))
+                  model->ArgusGlobalTime = now;
+
+               model->ArgusUpdateTimer = model->ArgusGlobalTime;
+
+               model->ArgusUpdateTimer.tv_sec  += model->ArgusUpdateInterval.tv_sec;
+               model->ArgusUpdateTimer.tv_usec += model->ArgusUpdateInterval.tv_usec;
+
+               while (model->ArgusUpdateTimer.tv_usec >= 1000000) {
+                  model->ArgusUpdateTimer.tv_sec++;
+                  model->ArgusUpdateTimer.tv_usec -= 1000000;
+               }
+            }
+         }
+      }
    }
 
 #ifdef ARGUSDEBUG
-   if (retn)
-      ArgusDebug (10, "ArgusUpdateTime (0x%x) global time %d.%06d update %d.%06d returning %d\n",
+   if (retn) {
+      ArgusDebug (8, "ArgusUpdateTime (%p) global time %d.%06d update %d.%06d returning %d\n",
                    model, model->ArgusGlobalTime.tv_sec, model->ArgusGlobalTime.tv_usec,
                    model->ArgusUpdateTimer.tv_sec, model->ArgusUpdateTimer.tv_usec, retn);
+   } else
+      ArgusDebug (8, "ArgusUpdateTime (%p) not time\n", model);
 #endif
 
    return (retn);
@@ -788,7 +860,7 @@ ArgusNewHashTable (size_t size, int status)
 #endif
 
 #ifdef ARGUSDEBUG
-   ArgusDebug (4, "ArgusNewHashTable (%d) returning 0x%x\n", size, retn);
+   ArgusDebug (4, "ArgusNewHashTable (%d) returning %p\n", size, retn);
 #endif
 
    return (retn);
@@ -808,7 +880,7 @@ ArgusDeleteHashTable (struct ArgusHashTable *htbl)
    }
 
 #ifdef ARGUSDEBUG
-   ArgusDebug (4, "ArgusDeleteHashTable (0x%x)\n", htbl);
+   ArgusDebug (4, "ArgusDeleteHashTable (%p)\n", htbl);
 #endif
 }
 
@@ -842,7 +914,7 @@ ArgusEmptyHashTable (struct ArgusHashTable *htbl)
 #endif
  
 #ifdef ARGUSDEBUG
-   ArgusDebug (6, "ArgusEmptyHashTable (0x%x) cleared %d bins %d items\n", htbl, bins, count);
+   ArgusDebug (6, "ArgusEmptyHashTable (%p) cleared %d bins %d items\n", htbl, bins, count);
 #endif
 }
 
@@ -887,7 +959,7 @@ ArgusCreateFlowKey (struct ArgusModelerStruct *model, struct ArgusSystemFlow *fl
 
 
 struct ArgusFlowStruct *
-ArgusFindFlow (struct ArgusModelerStruct *model, struct ArgusHashStruct *hstruct, unsigned int status)
+ArgusFindFlow (struct ArgusModelerStruct *model, struct ArgusHashStruct *hstruct)
 {
    struct ArgusFlowStruct *retn = NULL;
    struct ArgusHashTableHeader *hashEntry = NULL, *target, *head;
@@ -928,13 +1000,12 @@ ArgusFindFlow (struct ArgusModelerStruct *model, struct ArgusHashStruct *hstruct
             if (hashEntry != head)
                table->array[ind] = hashEntry;
             retn = hashEntry->object;
-            retn->status |= status;
          }
       }
    }
  
 #ifdef ARGUSDEBUG
-   ArgusDebug (8, "ArgusFindFlow () returning 0x%x\n", retn);
+   ArgusDebug (8, "ArgusFindFlow () returning %p\n", retn);
 #endif
   
    return (retn);
@@ -984,7 +1055,7 @@ ArgusAddHashEntry (struct ArgusHashTable *table, struct ArgusFlowStruct *flow, s
    }
 
 #ifdef ARGUSDEBUG
-   ArgusDebug (6, "ArgusAddHashEntry (0x%x) returning 0x%x\n", flow, retn);
+   ArgusDebug (6, "ArgusAddHashEntry (%p) returning %p\n", flow, retn);
 #endif
 
    return (retn);
@@ -1022,7 +1093,7 @@ ArgusRemoveHashEntry (struct ArgusHashTableHeader *htblhdr)
    }
 
 #ifdef ARGUSDEBUG
-   ArgusDebug (6, "ArgusRemoveHashEntry (0x%x) returning\n", htblhdr);
+   ArgusDebug (6, "ArgusRemoveHashEntry (%p) returning\n", htblhdr);
 #endif
 }
 
@@ -1032,21 +1103,25 @@ void ArgusZeroRecord (struct ArgusFlowStruct *);
 void
 ArgusZeroRecord (struct ArgusFlowStruct *flow)
 {
-   unsigned int *ptr;
-   int i, x, len;
+   int i;
 
    flow->status &= ~ARGUS_RECORD_WRITTEN;
+   flow->status &= ~0xF0;
 
-   for (i = 0; i < ARGUSMAXFLOWTYPE; i++) {
+   for (i = 0; i < ARGUSMAXDSRTYPE; i++) {
       if (flow->dsrs[i] != NULL) {
          switch (i) {
             default:
                flow->dsrs[i] = NULL;
                break;
 
-            case ARGUS_FLOW_INDEX:
+            case ARGUS_FLOW_INDEX: {
+               struct ArgusFlow *tflow = (struct ArgusFlow *)flow->dsrs[i];
+               tflow->hdr.argus_dsrvl8.qual &= ~ARGUS_FRAGMENT;
+               break;
+            }
+
             case ARGUS_TRANSPORT_INDEX:
-            case ARGUS_ENCAPS_INDEX:
             case ARGUS_MAC_INDEX:
                break;
 
@@ -1063,24 +1138,21 @@ ArgusZeroRecord (struct ArgusFlowStruct *flow)
 
             case ARGUS_IPATTR_INDEX: {
                struct ArgusIPAttrStruct *attr = (void *)flow->dsrs[i];
+               attr->hdr.argus_dsrvl8.qual &= ~(ARGUS_IPATTR_SRC_FRAGMENTS | ARGUS_IPATTR_DST_FRAGMENTS);
                attr->src.status = 0; attr->src.options = 0;
                attr->dst.status = 0; attr->dst.options = 0;
                break;
             }
 
             case ARGUS_TIME_INDEX: {
-               ptr = (unsigned int *) flow->dsrs[i];
-               len = ((sizeof(struct ArgusTimeObject) + 3)/4);
-               for (x = 1, ptr++; x < len; x++)
-                  *ptr++ = 0;
+               struct ArgusTimeObject *tim = (void *)flow->dsrs[i];
+               bzero(&tim->src, sizeof(*tim) - 4);
                break;
             }
 
             case ARGUS_METRIC_INDEX: {
-               ptr = (unsigned int *) flow->dsrs[i];
-               len = ((sizeof(struct ArgusMetricStruct) + 3)/4);
-               for (x = 1, ptr++; x < len; x++)
-                  *ptr++ = 0;
+               struct ArgusMetricStruct *metric = (void *) flow->dsrs[i];
+               bzero(metric, sizeof(*metric));
                break;
             }
 
@@ -1117,6 +1189,10 @@ ArgusZeroRecord (struct ArgusFlowStruct *flow)
                               tcp->dst.bytes    = 0;
                               tcp->src.winbytes = 0;
                               tcp->dst.winbytes = 0;
+                              tcp->src.ackbytes = 0;
+                              tcp->dst.ackbytes = 0;
+                              tcp->src.seqbase  = tcp->src.seq;
+                              tcp->dst.seqbase  = tcp->dst.seq;
                               break;
                            }
 
@@ -1164,10 +1240,15 @@ ArgusZeroRecord (struct ArgusFlowStruct *flow)
    flow->dtime.act.minval  = 0xffffffff;
    flow->dtime.idle.minval = 0xffffffff;
 
+   flow->sipid = 0;
+   flow->dipid = 0;
+   flow->skey.n_strokes = 0;
+
 #ifdef ARGUSDEBUG
-   ArgusDebug (8, "ArgusZeroRecord (0x%x) returning\n", flow);
+   ArgusDebug (8, "ArgusZeroRecord (%p) returning\n", flow);
 #endif
 }
+
 /*
 struct ArgusSocketStruct {
    struct ArgusListStruct *ArgusSocketList;
@@ -1201,7 +1282,7 @@ ArgusNewSocket (int fd)
       ArgusLog(LOG_ERR, "ArgusNewSocket: ArgusCalloc failed %s", strerror(errno));
 
 #ifdef ARGUSDEBUG
-   ArgusDebug (2, "ArgusNewSocket (%d) returning 0x%x\n", fd, retn);
+   ArgusDebug (2, "ArgusNewSocket (%d) returning %p\n", fd, retn);
 #endif
 
    return (retn);
@@ -1219,6 +1300,10 @@ ArgusDeleteSocket (struct ArgusOutputStruct *output, struct ArgusClientData *cli
          if (ArgusWriteOutSocket(output, client) < 0)
             break;
 
+#ifdef ARGUSDEBUG
+      if (!(ArgusListEmpty (list)))
+         ArgusDebug(2, "ArgusDeleteSocket: list not empty");
+#endif
       ArgusDeleteList(asock->ArgusSocketList, ARGUS_OUTPUT_LIST);
 
       close(asock->fd);
@@ -1235,7 +1320,7 @@ ArgusDeleteSocket (struct ArgusOutputStruct *output, struct ArgusClientData *cli
    }
 
 #ifdef ARGUSDEBUG
-   ArgusDebug (2, "ArgusDeleteSocket (0x%x) returning\n", asock);
+   ArgusDebug (2, "ArgusDeleteSocket (%p) returning\n", asock);
 #endif
 }
 
@@ -1253,7 +1338,7 @@ ArgusSetChroot(char *dir)
       ArgusLog(LOG_ERR, "ArgusSetChroot: failed to chdir to \"/\" after chroot: %s", dir, strerror(errno));
 
 #ifdef ARGUSDEBUG
-   ArgusDebug (2, "ArgusSetChroot (0x%x) returning\n", dir);
+   ArgusDebug (2, "ArgusSetChroot (%s) returning\n", dir);
 #endif
 }
 
@@ -1292,6 +1377,7 @@ ArgusWriteSocket (struct ArgusOutputStruct *output, struct ArgusClientData *clie
          if (asock->writen || asock->length)
             ArgusDebug (6, "ArgusWriteSocket: asock stats error %d %d\n", asock->writen, asock->length);
 #endif
+         if (client->host == NULL) {
          if (!(output->ArgusWriteStdOut) && (asock->filename)) {
             if (asock->lastwrite.tv_sec < output->ArgusModel->ArgusGlobalTime.tv_sec) {
                if (((stat (asock->filename, &statbuf)) < 0) || (asock->fd != -1) || (ArgusCloseFile)) {
@@ -1320,7 +1406,7 @@ ArgusWriteSocket (struct ArgusOutputStruct *output, struct ArgusClientData *clie
                            output->ArgusInitMar = ArgusGenerateInitialMar(output);
                            ocnt = ntohs(output->ArgusInitMar->hdr.len) * 4;
 #ifdef ARGUSDEBUG
-                           ArgusDebug (7, "ArgusWriteSocket: write initial mar (%d, 0x%x, %d)\n", 
+                           ArgusDebug (6, "ArgusWriteSocket: write initial mar (%d, %p, %d)\n", 
                               asock->fd, output->ArgusInitMar, ocnt);
 #endif
                            if (((retn = write (asock->fd, output->ArgusInitMar, ocnt))) < ocnt)
@@ -1338,6 +1424,7 @@ ArgusWriteSocket (struct ArgusOutputStruct *output, struct ArgusClientData *clie
                asock->lastwrite = output->ArgusModel->ArgusGlobalTime;
             }
          }
+         }
 
          if (asock->fd != -1) {
             if (ArgusGenerateRecord (output->ArgusModel, rec, 0, (struct ArgusRecord *)&asock->buf)) {
@@ -1357,13 +1444,13 @@ ArgusWriteSocket (struct ArgusOutputStruct *output, struct ArgusClientData *clie
 
                   if (ssfp && (*ssfp > 0)) {
 #ifdef ARGUSDEBUG
-                     ArgusDebug (3, "ArgusHandleClientData: sasl_encode(0x%x, 0x%x, %d, 0x%x, 0x%x)\n",
+                     ArgusDebug (6, "ArgusHandleClientData: sasl_encode(%p, %p, %d, %p, %p)\n",
                                          client->sasl_conn, rec, cnt, &output, &outputlen);
 #endif
                      if ((retn = sasl_encode(client->sasl_conn, (const char *) asock->buf, (unsigned int) cnt,
                                                                           &output, &outputlen)) == SASL_OK) {
 #ifdef ARGUSDEBUG
-                        ArgusDebug (3, "ArgusHandleClientData: sasl_encode returned %d bytes\n", outputlen);
+                        ArgusDebug (6, "ArgusHandleClientData: sasl_encode returned %d bytes\n", outputlen);
 #endif
                         if (outputlen < ARGUS_MAXRECORD) {
                            bcopy(output, asock->buf, outputlen);
@@ -1378,9 +1465,20 @@ ArgusWriteSocket (struct ArgusOutputStruct *output, struct ArgusClientData *clie
 #endif
 
 #ifdef ARGUSDEBUG
-               ArgusDebug (4, "ArgusWriteSocket: write record (%d, 0x%x, %d)\n", asock->fd, &asock->buf, cnt);
+               ArgusDebug (4, "ArgusWriteSocket: write record (%d, %p, %d)\n", asock->fd, &asock->buf, cnt);
 #endif
-               if ((retn = write (asock->fd, &asock->buf, cnt)) >= 0) {
+               if (client->host != NULL) {
+                  retn = sendto (asock->fd, &asock->buf, cnt, 0, client->host->ai_addr, client->host->ai_addrlen);
+#ifdef ARGUSDEBUG
+                  ArgusDebug (6, "ArgusWriteSocket: sendto (%d, %p, %d, ...) %d\n", asock->fd, &asock->buf, cnt, retn);
+#endif
+               } else {
+                  retn = write (asock->fd, &asock->buf, cnt);
+#ifdef ARGUSDEBUG
+                  ArgusDebug (6, "ArgusWriteSocket: write (%d, %p, %d, ...) %d\n", asock->fd, &asock->buf, cnt, retn);
+#endif
+               }
+               if (retn >= 0) {
                   asock->status |= ARGUS_WAS_FUNCTIONAL;
                   asock->errornum = 0;
                   if (retn != cnt) {
@@ -1397,6 +1495,7 @@ ArgusWriteSocket (struct ArgusOutputStruct *output, struct ArgusClientData *clie
                      ArgusDebug (6, "ArgusWriteSocket: write successful %d\n", retn);
 #endif
                   }
+
                } else {
 #ifdef ARGUSDEBUG
                   ArgusDebug (6, "ArgusWriteSocket: write returned %d, errno %d\n", retn, errno);
@@ -1435,6 +1534,7 @@ ArgusWriteSocket (struct ArgusOutputStruct *output, struct ArgusClientData *clie
                         break;
                   }
                }
+
             } else {
 #ifdef ARGUSDEBUG
                ArgusDebug (6, "ArgusWriteSocket: ArgusGenerateRecord returned zero\n");
@@ -1462,7 +1562,7 @@ ArgusWriteSocket (struct ArgusOutputStruct *output, struct ArgusClientData *clie
             asock->rec = (struct ArgusRecordStruct *) ArgusPopFrontList(list, ARGUS_NOLOCK);
 
 #ifdef ARGUSDEBUG
-         ArgusDebug (6, "ArgusWriteSocket (0x%x, 0x%x, 0x%x) schedule record\n", output, asock, rec);
+         ArgusDebug (6, "ArgusWriteSocket (%p, %p, %p) schedule record\n", output, asock, rec);
 #endif
          ArgusPushBackList (list, (struct ArgusListRecord *) ArgusCopyRecordStruct(rec), ARGUS_NOLOCK);
          retn = 0;
@@ -1472,7 +1572,7 @@ ArgusWriteSocket (struct ArgusOutputStruct *output, struct ArgusClientData *clie
 #endif
 
 #ifdef ARGUSDEBUG
-      ArgusDebug (6, "ArgusWriteSocket (0x%x, 0x%x, 0x%x) returning %d\n", output, asock, rec, retn);
+      ArgusDebug (6, "ArgusWriteSocket (%p, %p, %p) returning %d\n", output, asock, rec, retn);
 #endif
 
    return (retn);
@@ -1498,12 +1598,12 @@ ArgusWriteOutSocket (struct ArgusOutputStruct *output, struct ArgusClientData *c
          if (count > ARGUS_MAXWRITENUM)
             count = ARGUS_MAXWRITENUM;
 
+         if (count == 1)
+            count = 2;
+
 #if defined(ARGUS_THREADS)
          pthread_mutex_lock(&list->lock);
 #endif
-         if (asock->fd == -1) {
-         }
-
          while ((asock->fd != -1 ) && count--) {
             if ((rec = asock->rec) != NULL) {
                ptr = (unsigned char *)&asock->buf;
@@ -1541,7 +1641,19 @@ ArgusWriteOutSocket (struct ArgusOutputStruct *output, struct ArgusClientData *c
                if ((asock->writen < asock->length) && ( asock->writen >= 0)) {
                   len = asock->length - asock->writen;
 
-                  if ((retn = write(asock->fd, (unsigned char *)&ptr[asock->writen], len)) >= 0) {
+                  if (client->host != NULL) {
+                     retn = sendto (asock->fd, (unsigned char *)&ptr[asock->writen], len, 0, client->host->ai_addr, client->host->ai_addrlen);
+#ifdef ARGUSDEBUG
+                     ArgusDebug (3, "ArgusWriteSocket: sendto (%d, %p, %d, ...) %d\n", asock->fd, &asock->buf, len, retn);
+#endif
+                  } else {
+                     retn = write(asock->fd, (unsigned char *)&ptr[asock->writen], len);
+#ifdef ARGUSDEBUG
+                     ArgusDebug (3, "ArgusWriteSocket: write (%d, %p, %d, ...) %d\n", asock->fd, &asock->buf, len, retn);
+#endif
+                  }
+
+                  if (retn >= 0) {
                      asock->errornum = 0;
                      asock->writen += retn;
 
@@ -1555,7 +1667,7 @@ ArgusWriteOutSocket (struct ArgusOutputStruct *output, struct ArgusClientData *c
                               asock->writen = 0;
                               asock->length = 0;
                               ArgusFreeListRecord(rec);
-                              while ((rec = (struct ArgusRecordStruct *) ArgusPopFrontList(list, ARGUS_LOCK)) != NULL)
+                              while ((rec = (struct ArgusRecordStruct *) ArgusPopFrontList(list, ARGUS_NOLOCK)) != NULL)
                                  ArgusFreeListRecord(rec);
                            }
                            break;
@@ -1590,11 +1702,14 @@ ArgusWriteOutSocket (struct ArgusOutputStruct *output, struct ArgusClientData *c
                   gettimeofday(&list->outputTime, 0L);
                   ArgusFreeListRecord(rec);
 
+#ifdef ARGUSDEBUG
+                  ArgusDebug (6, "ArgusWriteOutSocket: rec %p complete, %d count\n", rec, count);
+#endif
                   asock->rec = NULL;
                   asock->writen = 0;
                   asock->length = 0;
 
-                  if ((rec = (struct ArgusRecordStruct *) ArgusPopFrontList(list, ARGUS_LOCK)) != NULL) {
+                  if ((rec = (struct ArgusRecordStruct *) ArgusPopFrontList(list, ARGUS_NOLOCK)) != NULL) {
                      if (ArgusGenerateRecord (output->ArgusModel, rec, 0, (struct ArgusRecord *)&asock->buf)) {
                         int cnt = ((struct ArgusRecord *)&asock->buf)->hdr.len * 4;
 #if defined(_LITTLE_ENDIAN)
@@ -1605,13 +1720,13 @@ ArgusWriteOutSocket (struct ArgusOutputStruct *output, struct ArgusClientData *c
                            unsigned int outputlen = 0;
                            const char *output =  NULL;
 #ifdef ARGUSDEBUG
-                           ArgusDebug (3, "ArgusHandleClientData: sasl_encode(0x%x, 0x%x, %d, 0x%x, 0x%x)\n",
+                           ArgusDebug (3, "ArgusHandleClientData: sasl_encode(%p, %p, %d, %p, %p)\n",
                                                                    client->sasl_conn, rec, cnt, &output, &outputlen);
 #endif
                            if ((retn = sasl_encode(client->sasl_conn, (const char *) asock->buf, (unsigned int) cnt,
                                                       &output, &outputlen)) == SASL_OK) {
 #ifdef ARGUSDEBUG
-                              ArgusDebug (3, "ArgusHandleClientData: sasl_encode returned %d bytes\n", outputlen);
+                              ArgusDebug (3, "ArgusWriteOutSocket: sasl_encode returned %d bytes\n", outputlen);
 #endif
                               if (outputlen < ARGUS_MAXRECORD) {
                                  bcopy(output, asock->buf, outputlen);
@@ -1627,29 +1742,32 @@ ArgusWriteOutSocket (struct ArgusOutputStruct *output, struct ArgusClientData *c
                         asock->writen = 0;
                         asock->length = cnt;
                         asock->rec = rec;
+#ifdef ARGUSDEBUG
+                        ArgusDebug (6, "ArgusWriteOutSocket: posted record %p", rec);
+#endif
 
                      } else {
 #ifdef ARGUSDEBUG
-                        ArgusDebug (3, "ArgusHandleClientData: ArgusGenerateRecord error! deleting record");
+                        ArgusDebug (6, "ArgusWriteOutSocket: ArgusGenerateRecord error! deleting record");
 #endif
                         ArgusFreeListRecord(rec);
                      }
                   } else {
 #ifdef ARGUSDEBUG
-                     ArgusDebug (3, "ArgusWriteOutSocket: list 0x%x is now empty", list);
+                     ArgusDebug (3, "ArgusWriteOutSocket: list %p is now empty", list);
 #endif
                   }
 
                } else {
 #ifdef ARGUSDEBUG
-                  ArgusDebug (3, "ArgusWriteOutSocket: still work to be done for 0x%x, len %d writen %d", rec, asock->length, asock->writen);
+                  ArgusDebug (6, "ArgusWriteOutSocket: still work to be done for %p, len %d writen %d", rec, asock->length, asock->writen);
 #endif
                   break;
                }
 
             } else {
 #ifdef ARGUSDEBUG
-               ArgusDebug (3, "ArgusWriteOutSocket: nothing to be done for 0x%x, len %d writen %d", rec, asock->length, asock->writen);
+               ArgusDebug (6, "ArgusWriteOutSocket: nothing to be done for %p, len %d writen %d", rec, asock->length, asock->writen);
 #endif
             }
          }
@@ -1659,29 +1777,38 @@ ArgusWriteOutSocket (struct ArgusOutputStruct *output, struct ArgusClientData *c
 #endif
 
 #ifdef ARGUSDEBUG
-         ArgusDebug (9, "ArgusWriteOutSocket(0x%x): queue empty\n", asock);
+         ArgusDebug (9, "ArgusWriteOutSocket(%p): queue empty\n", asock);
 #endif
          
          if (asock->errornum >= ARGUS_MAXERROR) {
-            ArgusLog (LOG_WARNING, "ArgusWriteOutSocket(0x%x) maximum errors exceeded %d\n", asock, asock->errornum);
+            ArgusLog (LOG_WARNING, "ArgusWriteOutSocket(%p) maximum errors exceeded %d\n", asock, asock->errornum);
             retn = -1;
          }
 
          if ((count = ArgusGetListCount(list)) > ArgusMaxListLength) {
-            ArgusLog (LOG_WARNING, "ArgusWriteOutSocket(0x%x) max queue exceeded %d\n", asock, count);
+            ArgusLog (LOG_WARNING, "ArgusWriteOutSocket(%p) max queue exceeded %d\n", asock, count);
             retn = -1;
          }
 
 #ifdef ARGUSDEBUG
          if (list) {
-            ArgusDebug (5, "ArgusWriteOutSocket (0x%x) %d records waiting. returning %d\n", asock, list->count, retn);
+            ArgusDebug (6, "ArgusWriteOutSocket (%p) %d records waiting. returning %d\n", asock, count, retn);
          } else {
-            ArgusDebug (5, "ArgusWriteOutSocket (0x%x) no list.  returning %d\n", asock, list->count, retn);
+            ArgusDebug (6, "ArgusWriteOutSocket (%p) no list.  returning %d\n", asock, count, retn);
          }
 #endif
       }
+
+   } else {
+#ifdef ARGUSDEBUG
+      ArgusDebug (6, "ArgusWriteOutSocket (%p, %p) no list returning %d\n", output, client, retn);
+#endif
    }
 
+#ifdef ARGUSDEBUG
+   ArgusDebug (6, "ArgusWriteOutSocket (%p, %p) returning %d\n", output, client, retn);
+#endif
+
    return retn;
 }
 
@@ -1718,7 +1845,6 @@ struct rtentry;         /* declarations in <net/if.h> */
 
 /*
  * hash tables for whatever-to-name translations
- */
 
 #define HASHNAMESIZE 4096
 
@@ -1747,7 +1873,7 @@ struct enamemem {
    u_short e_addr1;
    u_short e_addr2;
    char *e_name;
-   u_char *e_nsap;         /* used only for nsaptable[] */
+   u_char *e_nsap; 
    struct enamemem *e_nxt;
 };
 
@@ -1762,6 +1888,7 @@ struct protoidmem {
 };
 
 struct protoidmem protoidtable[HASHNAMESIZE];
+ */
 
 /*
  * A faster replacement for inet_ntoa().
@@ -1802,7 +1929,7 @@ intoa(u_int addr)
  * is assumed to be in network byte order.
  */
 char *
-ArgusGetName(u_char *ap)
+ArgusGetName(struct ArgusParserStruct *parser, u_char *ap)
 {
    static struct hnamemem *p;      /* static for longjmp() */
    u_int addr;
@@ -1832,7 +1959,7 @@ ArgusGetName(u_char *ap)
       break;
    }
 #endif
-   p = &hnametable[addr % (HASHNAMESIZE-1)];
+   p = &parser->hnametable[addr % (HASHNAMESIZE-1)];
    for (; p->nxt; p = p->nxt) {
       if (p->addr == addr)
          if (p->name != NULL)
@@ -1857,7 +1984,7 @@ ArgusGetName(u_char *ap)
 #endif
 
 char *
-ArgusGetV6Name(u_char *ap)
+ArgusGetV6Name(struct ArgusParserStruct *parser, u_char *ap)
 {
    struct in6_addr addr;
    char ntop_buf[INET6_ADDRSTRLEN];
@@ -1866,7 +1993,7 @@ ArgusGetV6Name(u_char *ap)
 
    memcpy(&addr, ap, sizeof(addr));
 
-   p = &h6nametable[*(u_int16_t *)&addr.s6_addr[14] & (HASHNAMESIZE-1)];
+   p = &parser->h6nametable[*(u_int16_t *)&addr.s6_addr[14] & (HASHNAMESIZE-1)];
    for (; p->nxt; p = p->nxt) {
       if (memcmp(&p->addr, &addr, sizeof(addr)) == 0)
          return (p->name);
@@ -1903,6 +2030,26 @@ RaMaxTime (struct timeval *s1, struct timeval *s2)
    return (retn);
 }
 
+long long RaDiffTime (struct ArgusTime *, struct ArgusTime *, struct timeval *);
+
+long long
+RaDiffTime (struct ArgusTime *s1, struct ArgusTime *s2, struct timeval *diff)
+{
+   long long v1 = 0, v2 = 0;
+
+   if (s1 && s2 && diff) {
+      v1 = (s1->tv_sec * 1000000LL) + s1->tv_usec;
+      v2 = (s2->tv_sec * 1000000LL) + s2->tv_usec;
+
+      v1 -= v2;
+
+      diff->tv_sec  = v1 / 1000000;
+      diff->tv_usec = v1 % 1000000;
+   }
+
+   return (v1);
+}
+
 
 float
 RaGetFloatDuration (struct ArgusRecordStruct *argus)
