@@ -1,6 +1,6 @@
 /*
  * Argus Software.  Argus files - Utilities include files
- * Copyright (c) 2000-2008 QoSient, LLC
+ * Copyright (c) 2000-2012 QoSient, LLC
  * All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -42,9 +42,9 @@
  */
 
 /* 
- * $Id: $
- * $DateTime: $
- * $Change: $
+ * $Id: //depot/argus/argus/argus/ArgusUtil.h#28 $
+ * $DateTime: 2012/01/03 19:15:19 $
+ * $Change: 2274 $
  */
 
 
@@ -53,15 +53,20 @@
 #ifndef ArgusUtil_h
 #define ArgusUtil_h
 
+#if defined(HAVE_STDLIB_H)
 #include <stdlib.h>
+#endif
+
+#if defined(HAVE_STRINGS_H)
 #include <strings.h>
+#endif
 
 #include <sys/time.h>
 #include <sys/socket.h>
 
 #include <net/if.h>
 
-#include <compat.h>
+#include <argus_compat.h>
 
 #include <argus_def.h>
 #include <argus_out.h>
@@ -74,6 +79,8 @@
 #define ARGUS_WFILE_LIST	2
 #define ARGUS_DEVICE_LIST	3
 #define ARGUS_OUTPUT_LIST	4
+#define ARGUS_EVENT_LIST	5
+#define ARGUS_BIND_ADDR_LIST	6
 
 struct ArgusListObjectStruct {
    struct ArgusListObjectStruct *nxt;
@@ -100,6 +107,7 @@ struct ArgusListStruct {
 
 #define ARGUS_PROCESS_NEXT_PASS		0x10
 
+/*
 struct ArgusQueueHeader {
    struct ArgusQueueHeader *prv, *nxt;
    struct ArgusQueueStruct *queue;
@@ -116,7 +124,7 @@ struct ArgusQueueStruct {
    struct ArgusQueueHeader *start, *end;
    struct ArgusFlowStruct **array;
 };
-
+*/
 
 #include <ArgusOutput.h>
 
@@ -197,8 +205,8 @@ int ArgusWriteOutSocket (struct ArgusOutputStruct *, struct ArgusClientData *);
 
 char *ArgusGetFlowString (struct ArgusFlowStruct *);
 
-char *ArgusGetName(unsigned char *);
-char *ArgusGetV6Name(unsigned char *);
+char *ArgusGetName(struct ArgusParserStruct *, unsigned char *);
+char *ArgusGetV6Name(struct ArgusParserStruct *, unsigned char *);
 
 void ArgusSetChroot(char *);
 
@@ -278,8 +286,8 @@ extern int ArgusWriteOutSocket (struct ArgusOutputStruct *, struct ArgusClientDa
 
 extern char *ArgusGetFlowString (struct ArgusFlowStruct *);
 
-extern char *ArgusGetName(unsigned char *);
-extern char *ArgusGetV6Name(unsigned char *);
+extern char *ArgusGetName(struct ArgusParserStruct *, unsigned char *);
+extern char *ArgusGetV6Name(struct ArgusParserStruct *, unsigned char *);
 
 extern void ArgusSetChroot(char *);
 
