@@ -1,6 +1,6 @@
 /*
  * Argus Software.  Argus files - main argus includes
- * Copyright (c) 2000-2008 QoSient, LLC
+ * Copyright (c) 2000-2012 QoSient, LLC
  * All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -20,9 +20,9 @@
  */
 
 /* 
- * $Id: $
- * $DateTime: $
- * $Change: $
+ * $Id: //depot/argus/argus/argus/argus.h#17 $
+ * $DateTime: 2012/01/03 19:15:19 $
+ * $Change: 2274 $
  */
 
 
@@ -35,6 +35,21 @@
 #ifndef Argus_h
 #define Argus_h
 
+#include <argus_compat.h>
+#include <argus_def.h>
+#include <ArgusModeler.h>
+#include <ArgusUtil.h>
+#include <ArgusEvents.h>
+#include <ArgusOutput.h>
+#include <ArgusSource.h>
+
+#if defined(ARGUS_NANOSECONDS)
+#define ARGUS_FRACTION_TIME      1000000000
+#else
+#define ARGUS_FRACTION_TIME      1000000
+#endif
+
+
 #define MINOR_VERSION_0    0
 #define MINOR_VERSION_1    1
 #define MINOR_VERSION_2    2
@@ -67,6 +82,10 @@
 #define MAXPATHNAMELEN		BUFSIZ
 #endif
 
+#if defined(ARGUS_THREADS)
+pthread_mutex_t ArgusMainLock;
+#endif
+
 char *RaTimeFormat = "%d %b %y %T";
 char RaFieldDelimiter = '.';
 int nflag = 1, Nflag = -1, uflag = 1, gflag = 0;
@@ -76,10 +95,12 @@ int Dflag = 0, daemonflag = 0;
 
 uid_t new_uid = 0;
 gid_t new_gid = 0;
+
 char *chroot_dir = NULL;
 
 int Argusdflag = 0;
 int ArgusUid = 0;
+int ArgusGid = 0;
 int ArgusShutDownStarted = 0;
 
 int ArgusPortNum = 0;
@@ -104,7 +125,14 @@ void ArgusComplete (void);
 void setArguspidflag (int);
 int getArguspidflag (void);
 
+void setArgusArgv(struct ArgusSourceStruct *, char **);
+void setArgusOptind(struct ArgusSourceStruct *, int);
+void setArgusCmdBuf(struct ArgusSourceStruct *);
+char *ArgusCopyArgv (char **);
+
 void  clearArgusWfile(void);
+void clearArgusEventRecord(void);
+
 
 char *ArgusBindAddr = NULL;
 struct ArgusListStruct *ArgusWfileList = NULL;
@@ -112,6 +140,10 @@ struct ArgusListStruct *ArgusWfileList = NULL;
 
 #else /* defined(Argus) */
 
+#if defined(ARGUS_THREADS)
+extern pthread_mutex_t ArgusMainLock;
+#endif
+
 extern int Argusdflag;
 
 extern char *ArgusProgramName;
@@ -124,6 +156,7 @@ extern int nflag, Nflag, uflag, gflag;
 extern int daemonflag;
 
 extern int ArgusUid;
+extern int ArgusGid;
 extern int ArgusShutDownStarted;
 
 extern char *ArgusBindAddr;
