@@ -1,6 +1,6 @@
 /*
  * Argus Software
- * Copyright (c) 2000-2008 QoSient, LLC
+ * Copyright (c) 2000-2012 QoSient, LLC
  * All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -19,6 +19,12 @@
  *
  */
 
+/* 
+ * $Id: //depot/argus/argus/common/argus_filter.c#17 $
+ * $DateTime: 2012/01/03 19:15:19 $
+ * $Change: 2274 $
+ */
+
 /*
  * Copyright (c) 1990, 1991, 1992, 1993
  *   The Regents of the University of California.  All rights reserved.
@@ -60,12 +66,9 @@
  *
  */
 
-/* 
- * $Id: //depot/argus/argus-3.0/clients/common/argus_filter.c#21 $
- * $DateTime: 2006/04/10 12:27:28 $
- * $Change: 824 $
- */
-
+#ifdef HAVE_CONFIG_H
+#include "argus_config.h"
+#endif
 
 #ifndef ArgusFilter
 #define ArgusFilter
@@ -73,6 +76,7 @@
 
 #include <stdlib.h>
 #include <unistd.h>
+#include <stdio.h>
 
 #include <errno.h>
 #include <string.h>
@@ -86,8 +90,8 @@
 #include <fcntl.h>
 #endif
 
+#include <argus_compat.h>
 
-#include <compat.h>
 #include <netinet/in.h>
 #include <netinet/in_systm.h>
 
@@ -104,14 +108,13 @@
 #include <sys/time.h>
 #include <net/nff.h>
 
-#include <argus_parser.h>
 #include <argus_filter.h>
 
 #ifdef sun
 #include <netinet/in.h>
 #endif
 
-#include <ethertype.h>
+#include <argus_ethertype.h>
 
 extern void ArgusLog (int, char *, ...);
 
@@ -140,7 +143,7 @@ xdtoi(int c)
  * buflen is the amount of data present
  */
 unsigned int ArgusFilterRecord (struct nff_insn *, struct ArgusRecordStruct *);
-unsigned int ArgusFilterOrig (struct nff_insn *, u_char *, int, int);
+unsigned int ArgusFilterOrig (struct nff_insn *, struct ArgusRecordStruct *, int, int);
 
 unsigned int
 ArgusFilterRecord (struct nff_insn *pc, struct ArgusRecordStruct *argus)
@@ -148,17 +151,18 @@ ArgusFilterRecord (struct nff_insn *pc, struct ArgusRecordStruct *argus)
    int buflen = sizeof(*argus);
 
    if (pc)
-      return ArgusFilterOrig (pc, (u_char *)argus, buflen, buflen);
+      return ArgusFilterOrig (pc, argus, buflen, buflen);
    else
       return (unsigned int) -1;
 }
 
 unsigned int
-ArgusFilterOrig (struct nff_insn *pc, u_char *p, int wirelen, int buflen)
+ArgusFilterOrig (struct nff_insn *pc, struct ArgusRecordStruct *argus, int wirelen, int buflen)
 {
    unsigned long long A = 0, X = 0;
    long long k, mem [NFF_MEMWORDS];
-   float F = 0.0;
+   u_char *p = (u_char *)argus;
+   float F;
 
    --pc;
    while (1) {
@@ -173,52 +177,215 @@ ArgusFilterOrig (struct nff_insn *pc, u_char *p, int wirelen, int buflen)
       case NFF_RET|NFF_A:
          return (unsigned int)A;
 
+      case NFF_LD|NFF_D|NFF_DSR: {
+         if (pc->dsr >= 0) {
+            struct ArgusDSRHeader *dsr;
+            if ((dsr = argus->dsrs[pc->dsr]) != NULL) {
+               u_char *ptr = (u_char *)dsr;
+               k = pc->data.k;
+               if (k + sizeof(double) > (dsr->argus_dsrvl8.len * 4)) 
+                  A = -1;
+               else
+                  A = EXTRACT_DOUBLE(&ptr[k]);
+            } else
+               A = -1;
+         } else {
+            struct ArgusRecordHeader *hdr;
+            if ((hdr = &argus->hdr) != NULL) {
+               u_char *ptr = (u_char *)hdr;
+               k = pc->data.k;
+               if (k + sizeof(double) > sizeof(hdr))
+                  A = -1;
+               else
+                  A = EXTRACT_DOUBLE(&ptr[k]);
+            } else
+               A = -1;
+         }
+         continue;
+      }
+
       case NFF_LD|NFF_D|NFF_ABS:
          k = pc->data.k;
          if (k + sizeof(double) > buflen) {
-            return 0;
-         }
-         A = EXTRACT_DOUBLE(&p[k]);
+            A = -1;
+         } else
+            A = EXTRACT_DOUBLE(&p[k]);
          continue;
 
+      case NFF_LD|NFF_F|NFF_DSR: {
+         if (pc->dsr >= 0) {
+            struct ArgusDSRHeader *dsr;
+            if ((dsr = argus->dsrs[pc->dsr]) != NULL) {
+               u_char *ptr = (u_char *)dsr;
+               k = pc->data.k;
+               if (k + sizeof(float) > (dsr->argus_dsrvl8.len * 4))
+                  F = -1;
+               else
+                  F = EXTRACT_FLOAT(&ptr[k]);
+            } else
+               F = -1;
+         } else {
+            struct ArgusRecordHeader *hdr;
+            if ((hdr = &argus->hdr) != NULL) {
+               u_char *ptr = (u_char *)hdr;
+               k = pc->data.k;
+               if (k + sizeof(float) > sizeof(hdr))
+                  F = -1;
+               else
+                  F = EXTRACT_FLOAT(&ptr[k]);
+            } else
+               F = -1;
+         }
+         continue;
+      }
+
       case NFF_LD|NFF_F|NFF_ABS:
          k = pc->data.k;
          if (k + sizeof(float) > buflen) {
-            return 0;
-         }
-         F = EXTRACT_FLOAT(&p[k]);
+            F = -1;
+         } else
+            F = EXTRACT_FLOAT(&p[k]);
          continue;
 
+      case NFF_LD|NFF_L|NFF_DSR: {
+         if (pc->dsr >= 0) {
+            struct ArgusDSRHeader *dsr;
+            if ((dsr = argus->dsrs[pc->dsr]) != NULL) {
+               u_char *ptr = (u_char *)dsr;
+               k = pc->data.k;
+               if (k + sizeof(long long) > (dsr->argus_dsrvl8.len * 4)) 
+                  A = -1;
+               else
+                  A = EXTRACT_LONGLONG(&ptr[k]);
+            } else
+               A = -1;
+         } else {
+            struct ArgusRecordHeader *hdr;
+            if ((hdr = &argus->hdr) != NULL) {
+               u_char *ptr = (u_char *)hdr;
+               k = pc->data.k;
+               if (k + sizeof(long long) > sizeof(hdr))
+                  A = -1;
+               else
+                  A = EXTRACT_LONGLONG(&ptr[k]);
+            } else
+               A = -1;
+         }
+         continue;
+      }
+
       case NFF_LD|NFF_L|NFF_ABS:
          k = pc->data.k;
          if ((k + sizeof(long long)) > buflen) {
-            return 0;
-         }
-         A = EXTRACT_LONGLONG(&p[k]);
+            A = -1;
+         } else
+            A = EXTRACT_LONGLONG(&p[k]);
          continue;
 
+      case NFF_LD|NFF_W|NFF_DSR: {
+         if (pc->dsr >= 0) {
+            struct ArgusDSRHeader *dsr;
+            if ((dsr = argus->dsrs[pc->dsr]) != NULL) {
+               u_char *ptr = (u_char *)dsr;
+               k = pc->data.k;
+               if ((k + sizeof(int)) > (dsr->argus_dsrvl8.len * 4))
+                  A = -1;
+               else
+                  A = EXTRACT_LONG(&ptr[k]);
+            } else
+               A = -1;
+
+         } else {
+            struct ArgusRecordHeader *hdr;
+            if ((hdr = &argus->hdr) != NULL) {
+               u_char *ptr = (u_char *)hdr;
+               k = pc->data.k;
+               if (k + sizeof(int) > sizeof(hdr))
+                  A = -1;
+               else
+                  A = EXTRACT_LONG(&ptr[k]);
+            } else
+               A = -1;
+         }
+         continue;
+      }
+
       case NFF_LD|NFF_W|NFF_ABS:
          k = pc->data.k;
          if ((k + sizeof(int)) > buflen) {
-            return 0;
-         }
-         A = EXTRACT_LONG(&p[k]);
+            A = -1;
+         } else
+            A = EXTRACT_LONG(&p[k]);
          continue;
 
+      case NFF_LD|NFF_H|NFF_DSR: {
+         if (pc->dsr >= 0) {
+            struct ArgusDSRHeader *dsr;
+            if ((dsr = argus->dsrs[pc->dsr]) != NULL) {
+               u_char *ptr = (u_char *)dsr;
+               k = pc->data.k;
+               if ((k + sizeof(short)) > (dsr->argus_dsrvl8.len * 4))
+                  A = -1;
+               else
+                  A = EXTRACT_SHORT(&ptr[k]);
+            } else
+               A = -1;
+         } else {
+            struct ArgusRecordHeader *hdr;
+            if ((hdr = &argus->hdr) != NULL) {
+               u_char *ptr = (u_char *)hdr;
+               k = pc->data.k;
+               if (k + sizeof(short) > sizeof(hdr))
+                  A = -1;
+               else
+                  A = EXTRACT_SHORT(&ptr[k]);
+            } else
+               A = -1;
+         }
+         continue;
+      }
+
       case NFF_LD|NFF_H|NFF_ABS:
          k = pc->data.k;
-         if ((k + sizeof(short)) > buflen) {
-            return 0;
-         }
-         A = EXTRACT_SHORT(&p[k]);
+         if ((k + sizeof(short)) > buflen)
+            A = -1;
+         else
+            A = EXTRACT_SHORT(&p[k]);
          continue;
 
+      case NFF_LD|NFF_B|NFF_DSR: {
+         if (pc->dsr >= 0) {
+            struct ArgusDSRHeader *dsr;
+            if ((dsr = argus->dsrs[pc->dsr]) != NULL) {
+               u_char *ptr = (u_char *)dsr;
+               k = pc->data.k;
+               if (k > (dsr->argus_dsrvl8.len * 4))
+                  A = -1;
+               else
+                  A = ptr[k];
+            } else
+               A = -1;
+         } else {
+            struct ArgusRecordHeader *hdr;
+            if ((hdr = &argus->hdr) != NULL) {
+               u_char *ptr = (u_char *)hdr;
+               k = pc->data.k;
+               if (k > sizeof(hdr))
+                  A = -1;
+               else
+                  A = ptr[k];
+            } else
+               A = -1;
+         }
+         continue;
+      }
+
       case NFF_LD|NFF_B|NFF_ABS:
          k = pc->data.k;
          if (k >= buflen) {
-            return 0;
-         }
-         A = p[k];
+            A = -1;
+         } else
+            A = p[k];
          continue;
 
       case NFF_LD|NFF_W|NFF_LEN:
@@ -232,33 +399,33 @@ ArgusFilterOrig (struct nff_insn *pc, u_char *p, int wirelen, int buflen)
       case NFF_LD|NFF_W|NFF_IND:
          k = X + pc->data.k;
          if ((k + sizeof(int)) > buflen) {
-            return 0;
-         }
-         A = EXTRACT_LONG(&p[k]);
+            A = -1;
+         } else
+            A = EXTRACT_LONG(&p[k]);
          continue;
 
       case NFF_LD|NFF_H|NFF_IND:
          k = X + pc->data.k;
          if ((k + sizeof(short)) > buflen) {
-            return 0;
-         }
-         A = EXTRACT_SHORT(&p[k]);
+            A = -1;
+         } else
+            A = EXTRACT_SHORT(&p[k]);
          continue;
 
       case NFF_LD|NFF_B|NFF_IND:
          k = X + pc->data.k;
          if (k >= buflen) {
-            return 0;
-         }
-         A = p[k];
+            A = -1;
+         } else
+            A = p[k];
          continue;
 
       case NFF_LDX|NFF_MSH|NFF_B:
          k = pc->data.k;
          if (k >= buflen) {
-            return 0;
-         }
-         X = (p[pc->data.k] & 0xf) << 2;
+            X = -1;
+         } else
+            X = (p[pc->data.k] & 0xf) << 2;
          continue;
 
       case NFF_LD|NFF_IMM:
@@ -290,47 +457,47 @@ ArgusFilterOrig (struct nff_insn *pc, u_char *p, int wirelen, int buflen)
          continue;
 
       case NFF_JMP|NFF_JGT|NFF_K:
-         pc += (A > pc->data.k) ? pc->jt : pc->jf;
+         pc += ((A > pc->data.k) && (A != -1)) ? pc->jt : pc->jf;
          continue;
 
       case NFF_JMP|NFF_JGE|NFF_K:
-         pc += (A >= pc->data.k) ? pc->jt : pc->jf;
+         pc += ((A >= pc->data.k) && (A != -1)) ? pc->jt : pc->jf;
          continue;
 
       case NFF_JMP|NFF_JEQ|NFF_K:
-         pc += (A == pc->data.k) ? pc->jt : pc->jf;
+         pc += ((A == pc->data.k) && (A != -1)) ? pc->jt : pc->jf;
          continue;
 
       case NFF_JMP|NFF_JGT|NFF_F:
-         pc += (F > pc->data.f) ? pc->jt : pc->jf;
+         pc += ((F > pc->data.f) && (A != -1)) ? pc->jt : pc->jf;
          continue;
 
       case NFF_JMP|NFF_JGE|NFF_F:
-         pc += (F >= pc->data.f) ? pc->jt : pc->jf;
+         pc += ((F >= pc->data.f) && (A != -1)) ? pc->jt : pc->jf;
          continue;
 
       case NFF_JMP|NFF_JEQ|NFF_F:
-         pc += (F == pc->data.f) ? pc->jt : pc->jf;
+         pc += ((F == pc->data.f) && (A != -1)) ? pc->jt : pc->jf;
          continue;
 
       case NFF_JMP|NFF_JSET|NFF_K:
-         pc += (A & pc->data.k) ? pc->jt : pc->jf;
+         pc += ((A & pc->data.k) && (A != -1)) ? pc->jt : pc->jf;
          continue;
 
       case NFF_JMP|NFF_JGT|NFF_X:
-         pc += (A > X) ? pc->jt : pc->jf;
+         pc += ((A > X) && (A != -1)) ? pc->jt : pc->jf;
          continue;
 
       case NFF_JMP|NFF_JGE|NFF_X:
-         pc += (A >= X) ? pc->jt : pc->jf;
+         pc += ((A >= X) && (A != -1)) ? pc->jt : pc->jf;
          continue;
 
       case NFF_JMP|NFF_JEQ|NFF_X:
-         pc += (A == X) ? pc->jt : pc->jf;
+         pc += ((A == X) && (A != -1)) ? pc->jt : pc->jf;
          continue;
 
       case NFF_JMP|NFF_JSET|NFF_X:
-         pc += (A & X) ? pc->jt : pc->jf;
+         pc += ((A & X) && (A != -1)) ? pc->jt : pc->jf;
          continue;
 
       case NFF_ALU|NFF_ADD|NFF_X:
@@ -722,23 +889,22 @@ ArgusLookupNet(char *device, unsigned int *netp, unsigned int *maskp, char *ebuf
    return (0);
 }
 
-#ifndef HAVE_SYS_ERRLIST
-static char argus_error[128];
+#if !defined(CYGWIN) && !defined(HAVE_SYS_ERRLIST)
+static char ArgusErrorString[128];
 #endif
 
-
 char *
 argus_strerror(int errnum)
 {
-#ifndef HAVE_SYS_ERRLIST
+#if !defined(CYGWIN) && !defined(HAVE_SYS_ERRLIST)
    extern int sys_nerr;
    extern char *sys_errlist[];
 
    if ((unsigned int)errnum < sys_nerr)
       return ((char *) sys_errlist[errnum]);
 
-   (void)snprintf(argus_error, 128, "Unknown error: %d", errnum);
-   return (argus_error);
+   (void)snprintf(ArgusErrorString, 128, "Unknown error: %d", errnum);
+   return (ArgusErrorString);
 #else
    return (strerror(errnum));
 #endif
@@ -1587,10 +1753,15 @@ opt_stmt( struct stmt *s, long long val[], int alter)
    long long v;
 
    switch (s->code) {
+      case NFF_LD|NFF_DSR|NFF_L:
       case NFF_LD|NFF_ABS|NFF_L:
+      case NFF_LD|NFF_DSR|NFF_W:
       case NFF_LD|NFF_ABS|NFF_W:
+      case NFF_LD|NFF_DSR|NFF_H:
       case NFF_LD|NFF_ABS|NFF_H:
+      case NFF_LD|NFF_DSR|NFF_B:
       case NFF_LD|NFF_ABS|NFF_B:
+      case NFF_LD|NFF_DSR|NFF_F:
       case NFF_LD|NFF_ABS|NFF_F:
          v = F(s->code, s->data.k, 0L);
          vstore(s, &val[A_ATOM], v, alter);
@@ -2339,7 +2510,7 @@ Argusnff_optimize(struct ablock **rootp)
    opt_cleanup();
 
 #ifdef ARGUSDEBUG
-   ArgusDebug (6, "Argusnff_optimize (0x%x)", rootp);
+   ArgusDebug (6, "Argusnff_optimize (%p)", rootp);
 #endif
 }
 
@@ -2393,6 +2564,7 @@ static inline int
 eq_blk(struct ablock *b0, struct ablock *b1)
 {
    if (b0->s.code == b1->s.code &&
+       b0->s.dsr == b1->s.dsr &&
        b0->s.data.k == b1->s.data.k &&
        b0->et.succ == b1->et.succ &&
        b0->ef.succ == b1->ef.succ)
@@ -2676,6 +2848,7 @@ convert_code_r(struct ablock *p)
    for (src = p->stmts; src; src = src->next) {
       if (src->s.code == NOP)
          continue;
+      dst->dsr = src->s.dsr;
       dst->code = (u_short)src->s.code;
       dst->data.k = src->s.data.k;
 
@@ -2742,6 +2915,7 @@ filled:
 #ifdef BDEBUG
    bids[dst - fstart] = p->id + 1;
 #endif
+   dst->dsr = p->s.dsr;
    dst->code = (u_short)p->s.code;
    dst->data.k = p->s.data.k;
    if (JT(p)) {
@@ -2814,7 +2988,7 @@ Argusicode_to_fcode(struct ablock *root, int *lenp)
    }
 
 #ifdef ARGUSDEBUG
-   ArgusDebug (10, "Argusicode_to_fcode () returning 0x%x", fp);
+   ArgusDebug (10, "Argusicode_to_fcode () returning %p", fp);
 #endif
 
    return fp;
@@ -2825,51 +2999,57 @@ static void
 opt_dump(struct ablock *root)
 {
    struct nff_program f;
+   char buf[MAXSTRLEN];
 
    memset(bids, 0, sizeof bids);
    f.bf_insns = Argusicode_to_fcode(root, &f.bf_len);
-   nff_dump(&f, 1);
-   putchar('\n');
+   nff_dump(&f, buf, MAXSTRLEN, 1);
+   printf("%s\n", buf);
    free((char *)f.bf_insns);
 }
 #endif
 
 
-extern void nff_dump(struct nff_program *, int);
+extern void nff_dump(struct nff_program *, char *, int, int);
 static char *nff_image(struct nff_insn *p, int n);
 
 void
-nff_dump(struct nff_program *p, int option)
+nff_dump(struct nff_program *p, char *buf, int buflen, int option)
 {
    struct nff_insn *insn;
-   int i;
+   int i, slen;
    int n = p->bf_len;
 
    insn = p->bf_insns;
    if (option > 2) {
-      printf("%d\n", n);
+      slen = strlen(buf);
+      snprintf(&buf[slen], buflen - slen, "%d\n", n);
       for (i = 0; i < n; ++insn, ++i) {
-         printf("%lu %lu %lu %Lu\n", (long)insn->code,
+         slen = strlen(buf);
+         snprintf(&buf[slen], buflen - slen, "%lu %lu %lu %Lu\n", (long)insn->code,
                 (long)insn->jt, (long)insn->jf, insn->data.k);
       }
       return ;
    }
    if (option > 1) {
       for (i = 0; i < n; ++insn, ++i)
-         printf("{ 0x%x, %d, %d, 0x%08Lx },\n",
+         slen = strlen(buf);
+         snprintf(&buf[slen], buflen - slen, "{ 0x%x, %d, %d, 0x%08Lx },\n",
                 insn->code, insn->jt, insn->jf, insn->data.k);
       return;
    }
    for (i = 0; i < n; ++insn, ++i) {
+      slen = strlen(buf);
 #ifdef BDEBUG
       extern int bids[];
-      printf(bids[i] > 0 ? "[%02d]" : " -- ", bids[i] - 1);
+      snprintf(&buf[slen], buflen - slen, bids[i] > 0 ? "[%02d]" : " -- ", bids[i] - 1);
 #endif
-      puts(nff_image(insn, i));
+      snprintf(&buf[slen], buflen - slen, "%s\n", nff_image(insn, i));
    }
 }
 
 
+
 /*
  * Copyright (c) 1990, 1991, 1992, 1994
  *   The Regents of the University of California.  All rights reserved.
@@ -2897,276 +3077,317 @@ static char *nff_image(struct nff_insn *, int);
 static char *
 nff_image(struct nff_insn *p, int n)
 {
-   int v;
-   float f;
-   char *fmt, *op;
+   int v, dsr;
    static char image[256];
-   char operand[64];
+   char operand[64], op[32];
+   char *fmt;
+   float f;
 
+   dsr = p->dsr;
    v = p->data.k;
    f = p->data.f;
 
    switch (p->code) {
 
    default:
-      op = "unimp";
+      snprintf(op, 6, "unimp");
       fmt = "0x%x";
       v = p->code;
       break;
 
    case NFF_RET|NFF_K:
-      op = "ret";
-      fmt = "#%d";
+      snprintf(op, 4, "ret");
+      fmt = " #%d";
       break;
 
    case NFF_RET|NFF_A:
-      op = "ret";
-      fmt = "";
+      snprintf(op, 4, "ret");
+      fmt = " ";
       break;
 
-   case NFF_LD|NFF_F|NFF_ABS:
-      op = "ldf";
+   case NFF_LD|NFF_F|NFF_DSR:
+      if (dsr < 0)
+         snprintf(op, 32, "ldf      hdr");
+      else
+         snprintf(op, 32, "ldf      dsr[%d]", dsr);
       fmt = "[%d]";
       break;
 
-   case NFF_LD|NFF_L|NFF_ABS:
-      op = "ldll";
+   case NFF_LD|NFF_F|NFF_ABS:
+      snprintf(op, 3, "ldf");
+      fmt = "[%d]";
+      break;
+
+   case NFF_LD|NFF_L|NFF_DSR:
+      if (dsr < 0)
+         snprintf(op, 32, "ldll     hdr");
+      else
+      snprintf(op, 32, "ldll      dsr[%d]", dsr);
       fmt = "[%Ld]";
       break;
 
+   case NFF_LD|NFF_L|NFF_ABS:
+      snprintf(op, 5, "ldll");
+      fmt = "[%Ld]";
+      break;
+
+   case NFF_LD|NFF_W|NFF_DSR:
+      if (dsr < 0)
+         snprintf(op, 32, "ld       hdr");
+      else
+         snprintf(op, 32, "ld       dsr[%d]", dsr);
+      fmt = "[%d]";
+      break;
+
    case NFF_LD|NFF_W|NFF_ABS:
-      op = "ld";
+      snprintf(op, 3, "ld");
+      fmt = "[%d]";
+      break;
+
+   case NFF_LD|NFF_H|NFF_DSR:
+      if (dsr < 0)
+         snprintf(op, 32, "ldh      hdr");
+      else
+         snprintf(op, 32, "ldh      dsr[%d]", dsr);
       fmt = "[%d]";
       break;
 
    case NFF_LD|NFF_H|NFF_ABS:
-      op = "ldh";
+      snprintf(op, 4, "ldh");
+      fmt = "[%d]";
+      break;
+
+   case NFF_LD|NFF_B|NFF_DSR:
+      if (dsr < 0)
+         snprintf(op, 32, "ldb      hdr");
+      else
+         snprintf(op, 32, "ldb      dsr[%d]", dsr);
       fmt = "[%d]";
       break;
 
    case NFF_LD|NFF_B|NFF_ABS:
-      op = "ldb";
+      snprintf(op, 4, "ldb");
       fmt = "[%d]";
       break;
 
    case NFF_LD|NFF_W|NFF_LEN:
-      op = "ld";
+      snprintf(op, 3, "ld");
       fmt = "#pktlen";
       break;
 
    case NFF_LD|NFF_F|NFF_IND:
-      op = "ldf";
+      snprintf(op, 4, "ldf");
       fmt = "[x + %d]";
       break;
 
    case NFF_LD|NFF_L|NFF_IND:
-      op = "ldll";
+      snprintf(op, 5, "ldll");
       fmt = "[x + %Ld]";
       break;
 
    case NFF_LD|NFF_W|NFF_IND:
-      op = "ld";
+      snprintf(op, 3, "ld");
       fmt = "[x + %d]";
       break;
 
    case NFF_LD|NFF_H|NFF_IND:
-      op = "ldh";
+      snprintf(op, 3, "ldh");
       fmt = "[x + %d]";
       break;
 
    case NFF_LD|NFF_B|NFF_IND:
-      op = "ldb";
+      snprintf(op, 4, "ldb");
       fmt = "[x + %d]";
       break;
 
    case NFF_LD|NFF_IMM:
-      op = "ld";
+      snprintf(op, 3, "ld");
       fmt = "#0x%x";
       break;
 
    case NFF_LDX|NFF_IMM:
-      op = "ldx";
+      snprintf(op, 4, "ldx");
       fmt = "#0x%x";
       break;
 
    case NFF_LDX|NFF_MSH|NFF_B:
-      op = "ldxb";
+      snprintf(op, 5, "ldxb");
       fmt = "4*([%d]&0xf)";
       break;
 
    case NFF_LD|NFF_MEM:
-      op = "ld";
+      snprintf(op, 3, "ld");
       fmt = "M[%d]";
       break;
 
    case NFF_LDX|NFF_MEM:
-      op = "ldx";
+      snprintf(op, 4, "ldx");
       fmt = "M[%d]";
       break;
 
    case NFF_ST:
-      op = "st";
+      snprintf(op, 3, "st");
       fmt = "M[%d]";
       break;
 
    case NFF_STX:
-      op = "stx";
+      snprintf(op, 4, "stx");
       fmt = "M[%d]";
       break;
 
    case NFF_JMP|NFF_JA:
-      op = "ja";
+      snprintf(op, 3, "ja");
       fmt = "%d";
       v = n + p->data.k;
       break;
 
    case NFF_JMP|NFF_JGT|NFF_F:
-      op = "jgt";
+      snprintf(op, 4, "jgt");
       fmt = "#%f";
       break;
 
    case NFF_JMP|NFF_JGE|NFF_F:
-      op = "jge";
+      snprintf(op, 4, "jge");
       fmt = "#%f";
       break;
 
    case NFF_JMP|NFF_JEQ|NFF_F:
-      op = "jeq";
+      snprintf(op, 4, "jeq");
       fmt = "#%f";
       break;
 
    case NFF_JMP|NFF_JGT|NFF_K:
-      op = "jgt";
+      snprintf(op, 4, "jgt");
       fmt = "#0x%x";
       break;
 
    case NFF_JMP|NFF_JGE|NFF_K:
-      op = "jge";
+      snprintf(op, 4, "jge");
       fmt = "#0x%x";
       break;
 
    case NFF_JMP|NFF_JEQ|NFF_K:
-      op = "jeq";
+      snprintf(op, 4, "jeq");
       fmt = "#0x%x";
       break;
 
    case NFF_JMP|NFF_JSET|NFF_K:
-      op = "jset";
+      snprintf(op, 4, "jset");
       fmt = "#0x%x";
       break;
 
    case NFF_JMP|NFF_JGT|NFF_X:
-      op = "jgt";
+      snprintf(op, 4, "jgt");
       fmt = "x";
       break;
 
    case NFF_JMP|NFF_JGE|NFF_X:
-      op = "jge";
+      snprintf(op, 4, "jge");
       fmt = "x";
       break;
 
    case NFF_JMP|NFF_JEQ|NFF_X:
-      op = "jeq";
+      snprintf(op, 4, "jeq");
       fmt = "x";
       break;
 
    case NFF_JMP|NFF_JSET|NFF_X:
-      op = "jset";
+      snprintf(op, 4, "jset");
       fmt = "x";
       break;
 
    case NFF_ALU|NFF_ADD|NFF_X:
-      op = "add";
+      snprintf(op, 4, "add");
       fmt = "x";
       break;
 
    case NFF_ALU|NFF_SUB|NFF_X:
-      op = "sub";
+      snprintf(op, 4, "sub");
       fmt = "x";
       break;
 
    case NFF_ALU|NFF_MUL|NFF_X:
-      op = "mul";
+      snprintf(op, 4, "mul");
       fmt = "x";
       break;
 
    case NFF_ALU|NFF_DIV|NFF_X:
-      op = "div";
+      snprintf(op, 4, "div");
       fmt = "x";
       break;
 
    case NFF_ALU|NFF_AND|NFF_X:
-      op = "and";
+      snprintf(op, 4, "and");
       fmt = "x";
       break;
 
    case NFF_ALU|NFF_OR|NFF_X:
-      op = "or";
+      snprintf(op, 3, "or");
       fmt = "x";
       break;
 
    case NFF_ALU|NFF_LSH|NFF_X:
-      op = "lsh";
+      snprintf(op, 4, "lsh");
       fmt = "x";
       break;
 
    case NFF_ALU|NFF_RSH|NFF_X:
-      op = "rsh";
+      snprintf(op, 4, "rsh");
       fmt = "x";
       break;
 
    case NFF_ALU|NFF_ADD|NFF_K:
-      op = "add";
-      fmt = "#%d";
+      snprintf(op, 4, "add");
+      fmt = " #%d";
       break;
 
    case NFF_ALU|NFF_SUB|NFF_K:
-      op = "sub";
-      fmt = "#%d";
+      snprintf(op, 4,"sub");
+      fmt = " #%d";
       break;
 
    case NFF_ALU|NFF_MUL|NFF_K:
-      op = "mul";
-      fmt = "#%d";
+      snprintf(op, 4, "mul");
+      fmt = " #%d";
       break;
 
    case NFF_ALU|NFF_DIV|NFF_K:
-      op = "div";
-      fmt = "#%d";
+      snprintf(op, 4, "div");
+      fmt = " #%d";
       break;
 
    case NFF_ALU|NFF_AND|NFF_K:
-      op = "and";
-      fmt = "#%d";
+      snprintf(op, 4, "and");
+      fmt = " #%d";
       break;
 
    case NFF_ALU|NFF_OR|NFF_K:
-      op = "or";
-      fmt = "#%d";
+      snprintf(op, 3, "or");
+      fmt = " #%d";
       break;
 
    case NFF_ALU|NFF_LSH|NFF_K:
-      op = "lsh";
-      fmt = "#%d";
+      snprintf(op, 4, "lsh");
+      fmt = " #%d";
       break;
 
    case NFF_ALU|NFF_RSH|NFF_K:
-      op = "rsh";
-      fmt = "#%d";
+      snprintf(op, 4, "rsh");
+      fmt = " #%d";
       break;
 
    case NFF_ALU|NFF_NEG:
-      op = "neg";
+      snprintf(op, 4, "neg");
       fmt = "";
       break;
 
    case NFF_MISC|NFF_TAX:
-      op = "tax";
+      snprintf(op, 4, "tax");
       fmt = "";
       break;
 
    case NFF_MISC|NFF_TXA:
-      op = "txa";
+      snprintf(op, 4, "txa");
       fmt = "";
       break;
    }
@@ -3180,7 +3401,7 @@ nff_image(struct nff_insn *p, int n)
             (NFF_CLASS(p->code) == NFF_JMP &&
              NFF_OP(p->code) != NFF_JA) ?
             "(%03d) %-8s %-16s jt %d\tjf %d"
-            : "(%03d) %-8s %s",
+            : "(%03d) %-8s%s",
             n, op, operand, n + 1 + p->jt, n + 1 + p->jf);
    return image;
 }
