@@ -18,125 +18,159 @@
  * WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
  * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  *
+ * @(#) $Header: //depot/argus/argus/include/argus_ethertype.h#7 $ (LBL)
  */
 
 /* 
- * $Id: $
- * $DateTime: $
- * $Change: $
+ * $Id: //depot/argus/argus/include/argus_ethertype.h#7 $
+ * $DateTime: 2011/01/26 17:11:49 $
+ * $Change: 2087 $
  */
 
-#if !defined(Argus_ethertype_h)
-#define Argus_ethertype_h
+#ifndef  Argus_Ethertype_h
+#define Argus_Ethertype_h
+
+
+#ifdef __cplusplus
+extern "C" {
+#endif
 
 /* Types missing from some systems */
 
-#if !defined(ETHERTYPE_SPRITE)
+#ifndef ETHER_HDRLEN
+#define ETHER_HDRLEN            14
+#endif
+
+#ifndef ETHERTYPE_JUMBO         
+#define ETHERTYPE_JUMBO         0x8870
+#endif
+
+#ifndef	ETHERTYPE_ISIS
+#define	ETHERTYPE_ISIS		0x0083
+#endif
+#ifndef ETHERTYPE_GRE_ISO
+#define ETHERTYPE_GRE_ISO       0x00FE
+#endif
+#ifndef ETHERTYPE_PUP
+#define ETHERTYPE_PUP           0x0200
+#endif
+#ifndef	ETHERTYPE_SPRITE
 #define	ETHERTYPE_SPRITE	0x0500
 #endif
-#if !defined(ETHERTYPE_NS)
+#ifndef ETHERTYPE_NS
 #define ETHERTYPE_NS		0x0600
 #endif
-#if !defined(ETHERTYPE_IP)
+#ifndef ETHERTYPE_IP
 #define ETHERTYPE_IP		0x0800
 #endif
-#if !defined(ETHERTYPE_X25L3)
+#ifndef ETHERTYPE_X25L3
 #define ETHERTYPE_X25L3		0x0805
 #endif
-#if !defined(ETHERTYPE_ARP)
+#ifndef ETHERTYPE_ARP
 #define ETHERTYPE_ARP		0x0806
 #endif
-#if !defined(ETHERTYPE_VINES)
+#ifndef ETHERTYPE_VINES
 #define ETHERTYPE_VINES		0x0bad
 #endif
-#if !defined(ETHERTYPE_TRAIL)
+#ifndef ETHERTYPE_TRAIL
 #define ETHERTYPE_TRAIL		0x1000
 #endif
-#if !defined(ETHERTYPE_TRAIN)
+#ifndef ETHERTYPE_TRAIN
 #define ETHERTYPE_TRAIN		0x1984
 #endif
-#if !defined(ETHERTYPE_3C_NBP_DGRAM)
+#ifndef ETHERTYPE_3C_NBP_DGRAM
 #define ETHERTYPE_3C_NBP_DGRAM	0x3c07
 #endif
-#if !defined(ETHERTYPE_DEC)
+#ifndef ETHERTYPE_DEC
 #define ETHERTYPE_DEC		0x6000
 #endif
-#if !defined(ETHERTYPE_MOPDL)
+#ifndef	ETHERTYPE_MOPDL
 #define	ETHERTYPE_MOPDL		0x6001
 #endif
-#if !defined(ETHERTYPE_MOPRC)
+#ifndef	ETHERTYPE_MOPRC
 #define	ETHERTYPE_MOPRC		0x6002
 #endif
-#if !defined(ETHERTYPE_DN)
+#ifndef	ETHERTYPE_DN
 #define	ETHERTYPE_DN		0x6003
 #endif
-#if !defined(ETHERTYPE_LAT)
+#ifndef ETHERTYPE_LAT
 #define ETHERTYPE_LAT		0x6004
 #endif
-#if !defined(ETHERTYPE_DEC_DIAG)
+#ifndef ETHERTYPE_DEC_DIAG
 #define ETHERTYPE_DEC_DIAG	0x6005
 #endif
-#if !defined(ETHERTYPE_DEC_CUST)
+#ifndef ETHERTYPE_DEC_CUST
 #define ETHERTYPE_DEC_CUST	0x6006
 #endif
-#if !defined(ETHERTYPE_SCA)
+#ifndef ETHERTYPE_SCA
 #define ETHERTYPE_SCA		0x6007
 #endif
-#if !defined(ETHERTYPE_REVARP)
+#ifndef ETHERTYPE_REVARP
 #define ETHERTYPE_REVARP	0x8035
 #endif
-#if !defined(ETHERTYPE_LANBRIDGE)
+#ifndef	ETHERTYPE_LANBRIDGE
 #define	ETHERTYPE_LANBRIDGE	0x8038
 #endif
-#if !defined(ETHERTYPE_DECDNS)
+#ifndef	ETHERTYPE_DECDNS
 #define	ETHERTYPE_DECDNS	0x803c
 #endif
-#if !defined(ETHERTYPE_DECDTS)
+#ifndef	ETHERTYPE_DECDTS
 #define	ETHERTYPE_DECDTS	0x803e
 #endif
-#if !defined(ETHERTYPE_VEXP)
+#ifndef	ETHERTYPE_VEXP
 #define	ETHERTYPE_VEXP		0x805b
 #endif
-#if !defined(ETHERTYPE_VPROD)
+#ifndef	ETHERTYPE_VPROD
 #define	ETHERTYPE_VPROD		0x805c
 #endif
-#if !defined(ETHERTYPE_ATALK)
+#ifndef ETHERTYPE_ATALK
 #define ETHERTYPE_ATALK		0x809b
 #endif
-#if !defined(ETHERTYPE_AARP)
+#ifndef ETHERTYPE_AARP
 #define ETHERTYPE_AARP		0x80f3
 #endif
-#if !defined(ETHERTYPE_8021Q)
+#ifndef	ETHERTYPE_8021Q
 #define	ETHERTYPE_8021Q		0x8100
 #endif
-#if !defined(ETHERTYPE_IPX)
+#ifndef ETHERTYPE_IPX
 #define ETHERTYPE_IPX		0x8137
 #endif
-#if !defined(ETHERTYPE_SNMP)
+#ifndef ETHERTYPE_SNMP
 #define ETHERTYPE_SNMP		0x814c
 #endif
-#if !defined(ETHERTYPE_IPV6)
+#ifndef ETHERTYPE_IPV6
 #define ETHERTYPE_IPV6		0x86dd
 #endif
-#if !defined(ETHERTYPE_PPP)
+#ifndef ETHERTYPE_PPP
 #define ETHERTYPE_PPP           0x880b
-#endif
-#if !defined(ETHERTYPE_SLOW)
-#define ETHERTYPE_SLOW          0x8809
-#endif
-#if !defined(ETHERTYPE_MPLS)
+#endif 
+#ifndef ETHERTYPE_MPLS
 #define ETHERTYPE_MPLS		0x8847
 #endif
-#if !defined(ETHERTYPE_MPLS_MULTI)
+#ifndef ETHERTYPE_MPLS_MULTI
 #define ETHERTYPE_MPLS_MULTI	0x8848
 #endif
-#if !defined(ETHERTYPE_PPPOED)
+#ifndef ETHERTYPE_PPPOED
 #define ETHERTYPE_PPPOED	0x8863
 #endif
-#if !defined(ETHERTYPE_PPPOES)
+#ifndef ETHERTYPE_PPPOES
 #define ETHERTYPE_PPPOES	0x8864
 #endif
-#if !defined(ETHERTYPE_LOOPBACK)
+#ifndef	ETHERTYPE_LOOPBACK
 #define	ETHERTYPE_LOOPBACK	0x9000
 #endif
+#ifndef ETHERTYPE_VMAN
+#define ETHERTYPE_VMAN          0x9100
 #endif
+#ifndef ETHERTYPE_ISO
+#define ETHERTYPE_ISO           0xfefe
+#endif
+#ifndef ETHERTYPE_UDTOE
+#define ETHERTYPE_UDTOE         0xBEEF
+#endif
+
+#ifdef __cplusplus
+}
+#endif
+#endif
+
