@@ -1,6 +1,6 @@
 /*
  * Argus Software.  Common include files - interface
- * Copyright (C) 2000-2008 QoSient, LLC.
+ * Copyright (C) 2000-2012 QoSient, LLC.
  * All Rights Reserved
  *
  * This program is free software; you can redistribute it and/or modify
@@ -20,9 +20,9 @@
  */
 
 /* 
- * $Id: $
- * $DateTime: $
- * $Change: $
+ * $Id: //depot/argus/argus/include/argus_int.h#15 $
+ * $DateTime: 2012/01/03 19:15:19 $
+ * $Change: 2274 $
  */
 
 #if !defined(Argus_int_h)
@@ -70,11 +70,6 @@ extern int dflag;		/* print interval code */
 extern int wflag;		/* write tcp connection data */
 extern int nflag;		/* leave addresses as numbers*/
 extern int debugflag;		/* set debug level */
-extern int tcptimeout;		/* set TCP timeout value (default TCPTIMEOUT) */
-extern int udptimeout;		/* set UDP timeout value (default UDPTIMEOUT) */
-extern int  iptimeout;		/* set  IP timeout value (default  IPTIMEOUT) */
-extern int icmptimeout;		/* set ICMP timeout (default ICMPTIMEOUT) */
-extern int fragtimeout;		/* set ICMP timeout (default ICMPTIMEOUT) */
 extern int Nflag;               /* remove domains from printed host names */
 
 extern double update_interval;
@@ -100,11 +95,9 @@ extern void timestampinit(void);
 
 extern int fn_print(const u_char *, const u_char *);
 extern int fn_printn(const u_char *, u_int, const u_char *);
-extern const char *tok2str(const struct ArgusTokenStruct *, const char *, int);
 extern char *dnaddr_string(u_short);
 extern char *savestr(const char *);
 
-extern char *isonsap_string(const u_char *);
 extern char *llcsap_string(u_char);
 extern char *protoid_string(const u_char *);
 extern char *dnname_string(u_short);
