@@ -1,6 +1,6 @@
 /*
  * Argus Software Common include files -  output structures
- * Copyright (c) 2000-2008 QoSient, LLC
+ * Copyright (c) 2000-2012 QoSient, LLC
  * All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -20,16 +20,16 @@
  */
 
 /* 
- * $Id: $
- * $DateTime: $
- * $Change: $
+ * $Id: //depot/argus/argus/include/argus_out.h#37 $
+ * $DateTime: 2012/01/03 19:15:19 $
+ * $Change: 2274 $
  */
 
 
 #if !defined(Argus_out_h)
 #define Argus_out_h
 
-#include <compat.h>
+#include <argus_compat.h>
 
 #if defined(HAVE_SOLARIS) || defined(__FreeBSD__) || defined(__NetBSD__) || defined(__OpenBSD__)
 #include <sys/types.h>
@@ -43,18 +43,13 @@
 
 #include <netinet/if_ether.h>
 #include <netinet/rtp.h>
+
+#include <argus_util.h>
 #include <argus_llc.h>
+#include <argus_isis.h>
+#include <argus_udt.h>
 
 #include <sys/time.h>
-
-#if PRAGMA_STRUCT_ALIGN //{
-#pragma options align=packed //{
-#elif PRAGMA_STRUCT_PACKPUSH //}{
-#pragma pack(push, 1)
-#elif PRAGMA_STRUCT_PACK //}{
-#pragma pack(2)
-#endif //}
-
 #pragma pack(2)
 
 struct ArgusDSRfixLen {
@@ -101,24 +96,6 @@ struct ArgusTime {
 #define argus_dsrvl8	dsr_un.vl8
 #define argus_dsrvl16	dsr_un.vl16
 
-struct ArgusMarStruct {
-   unsigned int status, argusid;
-   unsigned int localnet, netmask, nextMrSequenceNum; 
-   struct ArgusTime startime, now;
-   unsigned char  major_version, minor_version;
-   unsigned char interfaceType, interfaceStatus;
-   unsigned short reportInterval, argusMrInterval;
-   unsigned long long pktsRcvd, bytesRcvd;
-   long long drift;
-
-   unsigned int records, flows, dropped;
-   unsigned int queue, output, clients;
-   unsigned int bufs, bytes;
-   unsigned short suserlen, duserlen;
-   unsigned int pad[3];
-   unsigned int thisid, record_len;
-};
-
 struct ArgusICMPObject {
    unsigned char icmp_type, icmp_code;
    unsigned short iseq;
@@ -147,8 +124,8 @@ struct ArgusTCPStatus {
 struct ArgusTCPObjectMetrics {
    struct ArgusTime lasttime;
    unsigned int status, seqbase, seq, ack, winnum;
-   unsigned int bytes, retrans, ackbytes;
-   unsigned short state, win, winbytes;
+   unsigned int bytes, retrans, ackbytes, winbytes;
+   unsigned short win;
    unsigned char flags, winshift;
 };
  
@@ -159,6 +136,19 @@ struct ArgusTCPObject {
    struct ArgusTCPObjectMetrics src, dst;
 };
 
+struct ArgusUDTObjectMetrics {
+   struct ArgusTime lasttime;
+   unsigned int seq, tstamp, ack, rtt, var, bsize, rate, lcap;
+   int solo, first, middle, last, drops, retrans, nacked;
+};
+
+struct ArgusUDTObject {
+   unsigned int state, status;
+   struct udt_control_handshake hshake;
+   struct ArgusUDTObjectMetrics src, dst;
+};
+
+
 struct ArgusRTPObject {
    unsigned int state;
    struct rtphdr  src, dst;
@@ -197,6 +187,38 @@ struct ArgusFragObject {
    struct ArgusFragOffsetStruct offsets;
 };
 
+struct ArgusIsisLspFlow {
+   unsigned char  lspid[LSP_ID_LEN];
+   unsigned int   seqnum;
+};
+
+struct ArgusIsisHelloFlow {
+  unsigned char srcid[SYSTEM_ID_LEN];
+  unsigned char lanid[NODE_ID_LEN];
+  u_int8_t circuit_id; /* for point-to-point */
+};
+
+struct ArgusIsisCsnpFlow {
+   unsigned char srcid[NODE_ID_LEN];
+};
+
+struct ArgusIsisPsnpFlow {
+   unsigned char srcid[NODE_ID_LEN];
+};
+
+struct ArgusIsisFlow {
+   int pdu_type;
+   char esrc[ETHER_ADDR_LEN], edst[ETHER_ADDR_LEN];
+   char proto_version, pad[3];
+   union {
+      struct ArgusIsisHelloFlow hello;
+      struct ArgusIsisLspFlow   lsp;
+      struct ArgusIsisCsnpFlow  csnp;
+      struct ArgusIsisPsnpFlow  psnp;
+   } isis_un;
+   int chksum;
+};
+
 struct ArgusESPObject {
    unsigned int status, spi, lastseq, lostseq;
 };
@@ -215,6 +237,11 @@ struct ArgusAHObject {
    unsigned int src_replay, dst_replay;
 };
 
+struct ArgusLcpFlow {
+   struct ether_header ehdr;
+   unsigned char code, id;
+};
+
 struct ArgusEtherMacFlow {
    struct ether_header ehdr;
    unsigned char dsap, ssap;
@@ -225,6 +252,20 @@ struct Argus80211MacFlow {
    unsigned char dsap, ssap;
 };
 
+struct ArgusMacFlow {
+   union {
+      struct ArgusEtherMacFlow ether;
+      struct Argus80211MacFlow wlan;
+   } mac_union;
+};
+
+struct ArgusWlanFlow {
+   unsigned char dhost[ETHER_ADDR_LEN];
+   unsigned char shost[ETHER_ADDR_LEN];
+   unsigned char bssid[ETHER_ADDR_LEN];
+   unsigned char ssid[32];
+};
+
 struct ArgusIPAttrObject {
    unsigned char ttl, tos;
    unsigned short ip_id;
@@ -232,6 +273,61 @@ struct ArgusIPAttrObject {
    unsigned short options;
 };
 
+
+struct ArgusIsisObject {
+   struct isis_common_header common;
+   union {
+      struct isis_iih_lan_header iih_lan;
+      struct isis_iih_ptp_header iih_ptp;
+      struct isis_lsp_header lsp;
+      struct isis_csnp_header csnp;
+      struct isis_psnp_header psnp;
+   } isis_un;
+};
+
+struct ArgusLCPObject {
+   unsigned int status;
+   unsigned int state, options;
+};
+
+#include <argus/ieee802_11.h>
+
+struct Argus80211Beacon {
+   u_int8_t        timestamp[IEEE802_11_TSTAMP_LEN];
+   u_char          ap[IEEE802_11_AP_LEN];
+
+   u_int16_t       beacon_interval, listen_interval;
+   u_int16_t       status_code, aid, reason_code;
+   u_int16_t       auth_alg, auth_trans_seq_num;
+   u_int16_t       capability_info;
+   elem_status_t   challenge_status;
+   struct challenge_t  challenge;
+   elem_status_t   ssid_status;
+   struct ssid_t   ssid;
+   elem_status_t   rates_status;
+   struct rates_t  rates;
+   elem_status_t   ds_status;
+   struct ds_t     ds;
+   elem_status_t   cf_status;
+   struct cf_t     cf;
+   elem_status_t   fh_status;
+   struct fh_t     fh;
+   elem_status_t   tim_status;
+   struct tim_t    tim;
+};
+
+struct Argus80211Probe {
+};
+
+struct Argus80211Object {
+   union {
+      struct Argus80211Beacon beacon;
+      struct Argus80211Probe  probe;
+   } o_un;
+};
+
+
+
 struct ArgusESPFlow {
    unsigned int ip_src, ip_dst;
    unsigned char ip_p, tp_p;
@@ -359,7 +455,7 @@ struct ArgusIPFlow {
    unsigned int ip_src, ip_dst;
    unsigned char ip_p, tp_p;
    unsigned short sport, dport;
-   unsigned short pad;
+   unsigned char smask, dmask;
 };
 
 struct ArgusIPv6Flow {
@@ -376,17 +472,12 @@ struct ArgusIPv6Flow {
    unsigned short sport, dport;
 };
 
-struct ArgusIBFlow {
-   unsigned int ip_src[4], ip_dst[4];
-#if defined(_LITTLE_ENDIAN)
-   unsigned int vl:4, lver:4, sl:4, rsv1:2, lnh:2, rsv2:8, ip_p:8;
-#else
-   unsigned int ip_p:8, rsv2:8, lnh:2, rsv1:2, sl:4, lver:4, vl:4;
-#endif
-   unsigned short slid, dlid;
-   unsigned int qp;
+struct ArgusUdtFlow {
+   struct ArgusMacFlow mac;
+   unsigned int sockid;
 };
 
+
 struct ArgusFlow {
    struct ArgusDSRHeader hdr;
    union {
@@ -403,6 +494,10 @@ struct ArgusFlow {
       struct ArgusRarpFlow       rarp;
       struct ArgusIPv6FragFlow fragv6;
       struct ArgusIPFragFlow     frag;
+      struct ArgusLcpFlow         lcp;
+      struct ArgusIsisFlow       isis;
+      struct ArgusWlanFlow       wlan;
+      struct ArgusUdtFlow         udt;
   } flow_un;
 };
 
@@ -422,6 +517,10 @@ struct ArgusSystemFlow {
       struct ArgusRarpFlow       rarp;
       struct ArgusIPv6FragFlow fragv6;
       struct ArgusIPFragFlow     frag;
+      struct ArgusLcpFlow         lcp;
+      struct ArgusIsisFlow       isis;
+      struct ArgusWlanFlow       wlan;
+      struct ArgusUdtFlow         udt;
   } flow_un;
 #if defined(ALIGN_64BIT)
    int tail[1]; 
@@ -441,45 +540,22 @@ struct ArgusSystemFlow {
 #define     esp_flow flow_un.esp
 #define  fragv6_flow flow_un.fragv6
 #define    frag_flow flow_un.frag
-
-
-struct ArgusFarStruct {
-   struct ArgusFlow flow;
-};
-
-/*
-   The ArgusRecordHeader is composed of 4 4-bit fields, the
-   type, version, cause and options fields.  For the purpose
-   of portability, we define them as two char values, so that
-   little endian machines don't mess up the order, and
-   to avoid compiler problems with using bit fields.
-
-   unsigned char type:4;
-   unsigned char vers:4;
-   unsigned char cause:4;
-   unsigned char opt:4;
-*/
-
-struct ArgusRecordHeader {
-   unsigned char type;
-   unsigned char cause;
-   u_int16_t len;
-};
-
-struct ArgusRecord {
-   struct ArgusRecordHeader hdr;
-   union {
-      struct ArgusMarStruct mar;
-      struct ArgusFarStruct far;
-   } ar_un;
-};
-
-#define argus_mar	ar_un.mar
-#define argus_far	ar_un.far
+#define     lcp_flow flow_un.lcp
+#define    isis_flow flow_un.isis
+#define    wlan_flow flow_un.wlan
+#define     udt_flow flow_un.udt
 
 
 struct ArgusAddrStruct {
-   unsigned int value;
+   union {
+      unsigned int value;
+      unsigned int ipv4;
+      char str[4];
+/*
+      unsigned int ipv6[4];
+      unsigned char ethersrc[6];
+*/
+   } a_un;
 };
 
 struct ArgusTransportStruct {
@@ -502,10 +578,31 @@ struct ArgusTimeObject {
    struct ArgusTimeStruct src, dst;
 };
 
+struct ArgusEventObject {
+   struct ArgusDSRHeader hdr;
+
+};
+
+struct ArgusEventTimeStruct {
+   struct ArgusDSRHeader hdr;
+   struct ArgusTime start;
+   unsigned int duration;
+};
+
+
 struct ArgusUniStats {
    unsigned int pkts, bytes, appbytes;
 };
 
+struct ArgusStatObject {
+   int n;
+   float minval;
+   float meanval;
+   float stdev;
+   float maxval;
+   unsigned char fdist[8];
+};
+
 struct ArgusStatsObject {
    int n;
    float minval;
@@ -514,6 +611,14 @@ struct ArgusStatsObject {
    float maxval;
 };
 
+struct ArgusAgrStruct {
+   struct ArgusDSRHeader hdr;
+   unsigned int count;
+   struct ArgusTime laststartime, lasttime;
+   struct ArgusStatObject act, idle;
+};
+
+
 struct ArgusPacketSizeObject {
    unsigned short psizemin, psizemax;
 };
@@ -534,10 +639,7 @@ struct ArgusJitterStruct {
 
 struct ArgusMacStruct {
    struct ArgusDSRHeader hdr;
-   union {
-      struct ArgusEtherMacFlow ether;
-      struct Argus80211MacFlow wlan;
-   } mac_union;
+   struct ArgusMacFlow mac;
 };
    
 struct ArgusVlanStruct {
@@ -587,6 +689,139 @@ struct ArgusIcmpStruct {
    unsigned int igwaddr;
 };
 
+struct ArgusKeyStrokeMetrics {
+   int n_strokes;
+};
+
+struct ArgusKeyStrokeStruct {
+   struct ArgusKeyStrokeMetrics src, dst;
+};
+
+struct ArgusBehaviorStruct {
+   struct ArgusDSRHeader hdr;
+   union {
+      struct ArgusKeyStrokeStruct keyStroke;
+   } behavior_union;
+};
+
+#define     keyStroke behavior_union.keyStroke
+
+struct ArgusCorMetrics {
+   struct ArgusAddrStruct srcid;
+   int deltaDur, deltaStart, deltaLast;
+   int deltaSrcPkts, deltaDstPkts;
+};
+
+struct ArgusCorrelateStruct {
+   struct ArgusDSRHeader hdr;
+   struct ArgusCorMetrics metrics;
+};
+
+struct ArgusCountryCodeStruct {
+   struct ArgusDSRHeader hdr;
+   char src[2], dst[2];
+};
+
+struct ArgusCoordinates {
+   float lat, lon;
+};
+
+struct ArgusCoordinateRange {
+   struct ArgusCoordinates max, min;
+};
+
+struct ArgusSiteLocation {
+   unsigned int status;
+   union {
+      struct ArgusCoordinates cor;
+      struct ArgusCoordinateRange range;
+   } dist_union;
+};
+
+struct ArgusGeoLocationStruct {
+   struct ArgusDSRHeader hdr;
+   struct ArgusSiteLocation src, dst;
+};
+
+struct ArgusLabelStruct {
+   struct ArgusDSRHeader hdr;
+   union {
+      char *svc;
+      char *label;
+   }l_un;
+};
+
+
+struct ArgusEventStruct {
+   struct ArgusDSRHeader       event;   /* immediate data */
+   struct ArgusTransportStruct trans;
+   struct ArgusTimeObject       time;
+   struct ArgusDataStruct       data;
+};
+
+
+struct ArgusMarStruct {
+   unsigned int status, argusid;
+   unsigned int localnet, netmask, nextMrSequenceNum;
+   struct ArgusTime startime, now;
+   unsigned char  major_version, minor_version;
+   unsigned char interfaceType, interfaceStatus;
+   unsigned short reportInterval, argusMrInterval;
+   unsigned long long pktsRcvd, bytesRcvd;
+   long long drift;
+
+   unsigned int records, flows, dropped;
+   unsigned int queue, output, clients;
+   unsigned int bufs, bytes;
+   unsigned short suserlen, duserlen;
+   unsigned int pad[3];
+   unsigned int thisid, record_len;
+};
+
+struct ArgusFarStruct {
+   struct ArgusFlow flow;
+};
+
+/*
+   The ArgusRecordHeader is composed of 4 4-bit fields, the
+   type, version, cause and options fields.  For the purpose
+   of portability, we define them as two char values, so that
+   little endian machines don't mess up the order, and
+   to avoid compiler problems with using bit fields.
+
+   unsigned char type:4;
+   unsigned char vers:4;
+   unsigned char cause:4;
+   unsigned char opt:4;
+*/
+
+struct ArgusRecordHeader {
+   unsigned char type;
+   unsigned char cause;
+   u_int16_t len;
+};
+
+struct ArgusRecord {
+   struct ArgusRecordHeader hdr;
+   union {
+      struct ArgusMarStruct mar;
+      struct ArgusFarStruct far;
+      struct ArgusEventStruct event;
+   } ar_un;
+};
+
+#define argus_mar       ar_un.mar
+#define argus_far       ar_un.far
+#define argus_event     ar_un.event
+
+struct ArgusAsnStruct {
+   struct ArgusDSRHeader hdr;
+   uint32_t src_as;        /* originating AS of source address */
+   uint32_t dst_as;        /* originating AS of destination address */
+   uint32_t inode_as;      /* originating AS of intermediate node address, if present */
+};
+
+
 struct ArgusNetworkStruct {
    struct ArgusDSRHeader hdr;
    union {
@@ -595,6 +830,7 @@ struct ArgusNetworkStruct {
       struct ArgusTCPObject    tcp;
       struct ArgusICMPObject   icmp;
       struct ArgusICMPv6Object icmpv6;
+      struct ArgusUDTObject    udt;
       struct ArgusRTPObject    rtp;
       struct ArgusRTCPObject   rtcp;
       struct ArgusIGMPObject   igmp;
@@ -603,41 +839,57 @@ struct ArgusNetworkStruct {
       struct ArgusARPObject    arp;
       struct ArgusAHObject     ah;
       struct ArgusFragObject   frag;
+      struct ArgusLCPObject    lcp;
+      struct ArgusIsisObject   isis;
    } net_union;
 };
 
 
 struct ArgusCanonRecord {
-   struct ArgusRecordHeader     hdr;
-   struct ArgusFlow             flow;
-   struct ArgusTransportStruct  trans;
-   struct ArgusTimeObject       time;
-   struct ArgusEncapsStruct     encaps;
-   struct ArgusIPAttrStruct     attr;
-   struct ArgusMetricStruct     metric;
-   struct ArgusNetworkStruct    net;
-   struct ArgusMacStruct        mac;
-   struct ArgusVlanStruct       vlan;
-   struct ArgusMplsStruct       mpls;
-   struct ArgusIcmpStruct       icmp;
-   struct ArgusJitterStruct     jitter;
-   struct ArgusPacketSizeStruct psize;
-   struct ArgusDataStruct       data;
+   struct ArgusRecordHeader      hdr;
+   struct ArgusFlow              flow;
+   struct ArgusTransportStruct   trans;
+   struct ArgusTimeObject        time;
+   struct ArgusEncapsStruct      encaps;
+   struct ArgusAsnStruct         asn;
+   struct ArgusIPAttrStruct      attr;
+   struct ArgusMetricStruct      metric;
+   struct ArgusNetworkStruct     net;
+   struct ArgusMacStruct         mac;
+   struct ArgusVlanStruct        vlan;
+   struct ArgusMplsStruct        mpls;
+   struct ArgusIcmpStruct        icmp;
+   struct ArgusAgrStruct         agr;
+   struct ArgusCorrelateStruct   cor;
+   struct ArgusJitterStruct      jitter;
+   struct ArgusPacketSizeStruct  psize;
+   struct ArgusBehaviorStruct    actor;
+   struct ArgusCountryCodeStruct cocode;
+   struct ArgusLabelStruct       label;
+   struct ArgusDataStruct        data;
 };
 
 
-#define ARGUS_NSR_STICKY         0x00100000
+#define ARGUS_NSR_STICKY                0x00100000
+#define ARGUS_RECORD_MODIFIED           0x0100
 
 struct ArgusRecordStruct {
-   struct ArgusListObjectStruct *nxt;
-   struct ArgusDSRHeader *dsrs[ARGUSMAXFLOWTYPE];
-   unsigned int status, dsrindex, trans, timeout, idle;
+   struct ArgusQueueHeader qhdr;
+   unsigned int status, dsrindex, trans, autoid;
+   unsigned short timeout, idle;
+   struct RaBinProcessStruct *bins;
+   struct ArgusInput *input;
+   struct ArgusDSRHeader *dsrs[ARGUSMAXDSRTYPE];
+   struct ArgusCorStruct *correlates;
+   float srate, drate, sload, dload, dur, mean;
+   float sploss, dploss;
+   long long offset;
+
    struct ArgusHashTableHdr *htblhdr;
+   struct ArgusHashTableHdr *hinthdr;
    struct ArgusQueueStruct *nsq;
    struct ArgusRecordHeader hdr;
    struct ArgusCanonRecord canon;
-   float srate, drate, sload, dload, dur, avgdur;
-   float sploss, dploss;
 };
 
 
