@@ -1,6 +1,6 @@
 /*
  * Argus Software Common include files - Parser
- * Copyright (c) 2000-2008 QoSient, LLC
+ * Copyright (c) 2000-2012 QoSient, LLC
  * All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -20,17 +20,29 @@
  */
 
 /* 
- * $Id: $
- * $DateTime: $
- * $Change: $
+ * $Id: //depot/argus/argus/include/argus_parser.h#15 $
+ * $DateTime: 2012/03/15 01:13:08 $
+ * $Change: 2335 $
  */
 
 
 #if !defined(Argus_parser_h)
 #define Argus_parser_h
 
+#include <argus_compat.h>
+
+#if defined(ARGUS_THREADS)
+#include <pthread.h>
+#endif
+
+#ifdef ARGUS_SASL
+#include <sasl/sasl.h>
+#endif
+
+#include <argus_debug.h>
 #include <argus_def.h>
 #include <argus_out.h>
+#include <argus_int.h>
 
 #define ARGUS_PRINTSRC	0x0001
 #define ARGUS_PRINTDST	0x0002
@@ -96,6 +108,7 @@ struct ArgusParserStruct {
    int RaAllocHashTableHealers, RaAllocArgusRecord;
    int RaThisActiveIndex, RaThisFlowNum;
    int RaThisModelNum, RaParseError;
+   int ArgusSessionId, NonBlockingDNS, ArgusDSCodePoints;
 
    char *RaFlowModelFile, *ArgusFlowModelFile, *ArgusRemoteFilter;
    struct RaFlowModelStruct *RaFlowModel;
@@ -125,10 +138,22 @@ struct ArgusParserStruct {
    struct hnamemem  llcsaptable[HASHNAMESIZE];
 };
 
-#if defined(ArgusParser)
+
+#define ARGUSMONITOR_EQUAL      0x01
+#define ARGUSMONITOR_NOTEQUAL   0x02
+
+struct ArgusCIDRAddr *RaParseCIDRAddr (struct ArgusParserStruct *, char *);
+
+#if defined(Argus_Parser)
 struct ArgusParserStruct *ArgusNewParser(char *);
+void ArgusCloseParser(struct ArgusParserStruct *);
+struct ArgusParserStruct *ArgusParser;
+
 #else
 extern struct ArgusParserStruct *ArgusNewParser(char *);
+extern void ArgusCloseParser(struct ArgusParserStruct *);
+extern struct ArgusParserStruct *ArgusParser;
+
 #endif
 
 #endif
