@@ -1,6 +1,6 @@
 /*
  * Argus Software.  Common include files - Utilities
- * Copyright (c) 2000-2008 QoSient, LLC
+ * Copyright (c) 2000-201! QoSient, LLC
  * All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -19,32 +19,10 @@
  *
  */
 
-/*
- * Copyright (c) 1993, 1994 Carnegie Mellon University.
- * All rights reserved.
- *
- * Permission to use, copy, modify, and distribute this software and
- * its documentation for any purpose and without fee is hereby granted, 
- * provided that the above copyright notice appear in all copies and
- * that both that copyright notice and this permission notice appear
- * in supporting documentation, and that the name of CMU not be
- * used in advertising or publicity pertaining to distribution of the
- * software without specific, written prior permission.  
- * 
- * CMU DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
- * ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
- * CMU BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
- * ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
- * WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
- * ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
- * SOFTWARE.
- *
- */
-
 /* 
- * $Id: $
- * $DateTime: $
- * $Change: $
+ * $Id: //depot/argus/argus/include/argus_util.h#17 $
+ * $DateTime: 2012/04/13 11:28:33 $
+ * $Change: 2361 $
  */
 
 #if !defined(Argus_util_h)
@@ -57,55 +35,50 @@
 #include <netinet/ip.h>
 
 #include <argus_def.h>
-#include <argus_out.h>
-#include <cons_out.h>
+#include <argus/cons_out.h>
 
-#include <CflowdFlowPdu.h>
+#include <argus/CflowdFlowPdu.h>
 
 
 typedef void (*proc)(void);
+typedef char *(*strproc)(void);
 
-struct QUEUE_HEADER {
-   struct QUEUE_HEADER *nxt;
-   struct QUEUE_HEADER *prv;
-   struct QUEUE *queue;
-   struct timeval last_time, logtime;
+struct ArgusQueueHeader {
+   struct ArgusQueueHeader *prv, *nxt;
+   struct ArgusQueueStruct *queue;
+   struct timeval lasttime, qtime;
+   unsigned int status;
+};
+
+struct ArgusQueueStruct {
+   struct ArgusQueueHeader qhdr;
+   int count, turns, timeout, status, reclaim;
+#if defined(ARGUS_THREADS)
+   pthread_mutex_t lock;
+#endif
+   struct ArgusQueueHeader *start, *end;
+   struct ArgusFlowStruct **array;
 };
 
 
-struct QUEUE {
-   struct QUEUE_HEADER *start;
-   proc timerRoutine, logRoutine;
-   unsigned int count;
+struct ArgusMemoryHeader {
+   struct ArgusMemoryHeader *nxt, *prv;
+#if defined(__GNUC__)
+   void *frame[3];
+#endif
+   unsigned int tag;
+   unsigned short len;
+   unsigned short offset;
 };
 
-struct HASH_TABLE_HEADER {
-   struct HASH_TABLE_HEADER *nxt;
-   struct HASH_TABLE_HEADER *prv;
-   unsigned int hash;
+struct ArgusMemoryList {
+   struct ArgusMemoryHeader *start, *end;
+#if defined(ARGUS_THREADS)
+   pthread_mutex_t lock;
+#endif
+   int total, count, size;
+   int out, in, freed;
 };
- 
-struct HASH_TABLE {
-   struct HASH_TABLE_HEADER **hash_array;
-   unsigned int size;
-};
-
-
-struct FILE_ENTRY {
-   struct FILE_ENTRY *nxt;
-   char *str;
-};
-
-
-#define Version1        1
-#define Version5        5
-#define Version6        6
-#define Version7        7
-#define Version8        8
-
-struct ArgusRecord *ArgusNetFlowCallRecord (u_char **);
-struct ArgusRecord *ArgusNetFlowDetailInt  (u_char **);
-struct ArgusRecord *ArgusParseCiscoRecord (u_char **);
 
 #if defined(ARGUS_SASL)
 #include <sasl/sasl.h>
@@ -115,11 +88,12 @@ struct ArgusRecord *ArgusParseCiscoRecord (u_char **);
 
 #define ARGUS_DATA_SOURCE		0x01
 #define ARGUS_CISCO_DATA_SOURCE		0x10
+#define ARGUS_SFLOW_DATA_SOURCE		0x20
 
 
 #define TSEQ_HASHSIZE      9029
 
-#define ipaddr_string(p) ArgusGetName((u_char *)(p))
+#define ipaddr_string(p) ArgusGetName(ArgusParser, (u_char *)(p))
 
 #if defined(argus_util)
 
@@ -182,7 +156,6 @@ void *ArgusMallocListRecord (int length);
 
 void ArgusLog (int, char *, ...);
 
-
 #else
 
 extern void ArgusAdjustGlobalTime (struct timeval *);
